/**
 ** sparkDistanceAndAngleAndPointingCodels.cc
 **
 ** Codels called by execution task sparkMainTask
 **
 ** Author: Mathieu Warnier
 ** Date: October 2013
 **
 **
 ** It is concerned with systematic computatioon of some distance , angle 
 ** and pointing computations 
 ** They are exported in a poster that is used by the oprs supervisor.
 **
 **/

#include <portLib.h>

#include "server/sparkHeader.h"
#include "sparkm3d.h"

#include "libeuler/pomEuler.h"

/// index of the perceiving robot in the distance info agent list.
int mySelfRobotAgentIndexInDistancesInfo = -1;


/* ------------------------------------------------------------- */
/* ---------  Manage Distance Poster  ------------------- */
/* ------------------------------------------------------------- */

void publishDistancesInfoPoster( void )
{ 
  sparkDistancesInfodistancesInfoPosterWrite(SPARK_DISTANCESINFO_POSTER_ID, &SDI_F->distancesInfo);
}

/* Vec Oprerations */

void createVec(double x,double y, double z,SPARK_3D_COORD & newVec){
  newVec.x =x;
  newVec.y =y;
  newVec.z =z;
}

void copyVec(SPARK_3D_COORD vec,SPARK_3D_COORD & newVec){
  newVec.x =vec.x;
  newVec.y =vec.y;
  newVec.z =vec.z;
}

void getVecABFromPointAAndPointB(SPARK_3D_COORD pointA,SPARK_3D_COORD pointB,SPARK_3D_COORD & vecAB){
  vecAB.x = pointB.x - pointA.x;
  vecAB.y = pointB.y - pointA.y;
  vecAB.z = pointB.z - pointA.z;
}

void getPointBFromPointAAndVecAB(SPARK_3D_COORD pointA,SPARK_3D_COORD vecAB,SPARK_3D_COORD & pointB){
  pointB.x = vecAB.x + pointA.x;
  pointB.y = vecAB.y + pointA.y;
  pointB.z = vecAB.z + pointA.z;
}

void getVecACFromVecABAndVecBC(SPARK_3D_COORD vecAB,SPARK_3D_COORD vecBC,SPARK_3D_COORD & vecAC){
  vecAC.x = vecAB.x + vecBC.x;
  vecAC.y = vecAB.y + vecBC.y;
  vecAC.z = vecAB.z + vecBC.z;
}

void getVecBCFromVecACAndVecAB(SPARK_3D_COORD vecAC,SPARK_3D_COORD vecAB,SPARK_3D_COORD & vecBC){
  vecBC.x = vecAC.x - vecAB.x;
  vecBC.y = vecAC.y - vecAB.y;
  vecBC.z = vecAC.z - vecAB.z;
}


void getMultVec(SPARK_3D_COORD & vec,double mult){
  vec.x = vec.x*mult;
  vec.y = vec.y*mult;
  vec.z = vec.z*mult;
}

void getVecsSum(SPARK_3D_COORD vec1,SPARK_3D_COORD vec2,SPARK_3D_COORD & vecSum){
  vecSum.x = vec1.x + vec2.x;
  vecSum.y = vec1.y + vec2.y;
  vecSum.z = vec1.z + vec2.z;
}

void getVecProd(SPARK_3D_COORD vec1,SPARK_3D_COORD vec2,SPARK_3D_COORD & vecProd){
  vecProd.x = vec1.y*vec2.z - vec1.z*vec2.y;
  vecProd.y = vec1.z*vec2.x - vec1.x*vec2.z;
  vecProd.z = vec1.x*vec2.y - vec1.y*vec2.x;
}

bool getIntersectionWithHorizontalPlan(SPARK_3D_COORD startPoint,SPARK_3D_COORD directionVec,double planHeight,SPARK_3D_COORD &planInterPoint){
  if((directionVec.z != 0.0) || (startPoint.z == planHeight)){
    if((directionVec.z == 0.0) &&  (startPoint.z == planHeight)){
      getPointBFromPointAAndVecAB(startPoint,directionVec,planInterPoint);
      return true;
    }
    else{
      planInterPoint.x = (planHeight - startPoint.z)*directionVec.x/directionVec.z + startPoint.x;
      planInterPoint.y = (planHeight - startPoint.z)*directionVec.y/directionVec.z + startPoint.y;
      planInterPoint.z = planHeight;
      return true;
    }
  }
  else
    return false;
}

double getVecScalProd(SPARK_3D_COORD vec1,SPARK_3D_COORD vec2){
  return vec1.x*vec2.x+vec1.y*vec2.y+vec1.z*vec2.z;
}

void getVecProdVecOnPlanZ0(SPARK_3D_COORD vec1,SPARK_3D_COORD vec2,SPARK_3D_COORD & vecProd){
  vecProd.x = -vec1.z*vec2.y;
  vecProd.y = vec1.z*vec2.x;
  vecProd.z = vec1.x*vec2.y - vec1.y*vec2.x;
}


void getVecProdVecOfHorizontalVec(SPARK_3D_COORD vec1,SPARK_3D_COORD vec2,SPARK_3D_COORD & vecProd){
  vecProd.x = 0;
  vecProd.y = 0;
  vecProd.z = vec1.x*vec2.y - vec1.y*vec2.x;
}


double getVecDistance(SPARK_3D_COORD vec){
  return D3D(0,0,0,vec.x,vec.y,vec.z);
}

double getDistanceBetweenPoints(SPARK_3D_COORD point1,SPARK_3D_COORD point2){
  return D3D(point1.x,point1.y,point1.z,point2.x,point2.y,point2.z);
}

double get2DDistanceBetweenPoints(SPARK_3D_COORD point1,SPARK_3D_COORD point2){
  return D2D(point1.x,point1.y,point2.x,point2.y);
}

void getPointAtAlphaFromVecAB(double alpha,SPARK_3D_COORD pointA,SPARK_3D_COORD pointB,SPARK_3D_COORD &pointAlphaAB){
  pointAlphaAB.x = alpha*pointB.x +  (1-alpha)*pointA.x;
  pointAlphaAB.y = alpha*pointB.y + (1-alpha)*pointA.y;
  pointAlphaAB.z = alpha*pointB.z + (1-alpha)*pointA.z;
}


void normalizeVec(SPARK_3D_COORD & vec){
  getMultVec(vec,1/getVecDistance(vec));
}

void printVec(SPARK_3D_COORD & vec,int printInt){
  printf("Print num %d of vector : x : %f ; y : %f ; z : %f\n",printInt,vec.x,vec.y,vec.z);
}

/* end vec operations */


bool needToUpdateTableCorners(int entityIndex,SPARK_3D_COORD corners[],double threshold){
  SPARK_BB_POINTS bbPoints;
  bool cornerUsed[4] = {false,false,false,false};
  double dist,minDist = 0;
  int minDistIndex = 0;
  double totalDist = 0;
  bool minDistInitialized;

  //we get min distance between a new corner and an old corner
  if(getBBPoints( gEntities->entities[entityIndex]->robotPt,&bbPoints)){
    //update table corners
    for(int i = 0; i < 4 ; i++){
      //find first corner not already used
      minDistInitialized = false;
      for(int j = 0; j < 4 ; j++){
	if(!cornerUsed[j] && !minDistInitialized){
	  minDist = get2DDistanceBetweenPoints(bbPoints.bbPointsTable[i],corners[j]);
	  minDistIndex = j;
	  minDistInitialized = true;
	}
	if(!cornerUsed[j] && minDistInitialized && (minDistIndex < j)){
	  dist = get2DDistanceBetweenPoints(bbPoints.bbPointsTable[i],corners[j]);
	  if(dist<minDist){
	    minDist = dist;
	    minDistIndex = j;
	  }
	}
      }
      //closest old corner from new corner i is corner with index minDistIndex  (not alread used as closest from previous corner)
      cornerUsed[minDistIndex] = true;
      totalDist += minDist; 
    }
    totalDist = totalDist/4;
    if(totalDist>threshold){
      printf("Table corner points change score is %f. More than %f --> table update\n",totalDist,threshold);
      return true;
    }
    else
      return false;
  }
  else 
    return true;
}

bool UpdateTablePointsInDistanceManagement(int entityIndex,int tableIndex){
  // For table points management
  //getBBPoints give 8 points of BB. 3 four points are on the uper horizontal plane. 
  //point 0 - point 1 is the side 1 , point 0 - point 2 is the side 2 these are not the diagonal.
  SPARK_BB_POINTS bbPoints;
  SPARK_3D_COORD table0; // first table point in BB table
  SPARK_3D_COORD table1; // second table point in BB table -> side1
  SPARK_3D_COORD table2; // third table point in BB table -> side2
  SPARK_3D_COORD pointStart;
  SPARK_3D_COORD newPoint;
  SPARK_3D_COORD vec1; //normalized vector along side 1
  SPARK_3D_COORD vec2;
  SPARK_3D_COORD vecTrans1; //normalized vector along side 1
  SPARK_3D_COORD vecTrans2;
  SPARK_3D_COORD vecTrans;

  double distSide1; //side1
  double distSide2; //side2
  int numPointsSide1;
  int numPointsSide2;
  double tableStep;
  int iter = 0;
  int numTablesPoints =  SDI_F->distancesInfo.numTablePoints;

  if(getBBPoints( gEntities->entities[entityIndex]->robotPt,&bbPoints)){

    //update table corners
    for(int i = 0; i < 4 ; i++)
      copyVec(bbPoints.bbPointsTable[i],SDI_F->distancesInfo.tables[tableIndex].tableCornersPositions[i]);
	
    tableStep = SDI_F->distancesInfo.tableStep;
    distSide1=bbPoints.side1Length; //side1
    distSide2=bbPoints.side2Length; //side2
    numPointsSide1 = floor(distSide1/tableStep); 
    numPointsSide2 = floor(distSide2/tableStep);

    copyVec(bbPoints.bbPointsTable[0],table0);
    copyVec(bbPoints.bbPointsTable[1],table1);
    copyVec(bbPoints.bbPointsTable[2],table2);
      
    getVecABFromPointAAndPointB(table0,table1,vec1);
    normalizeVec(vec1);
    getVecABFromPointAAndPointB(table0,table2,vec2);
    normalizeVec(vec2);

    copyVec(vec1,vecTrans1);
    getMultVec(vecTrans1,(distSide1-numPointsSide1*tableStep)/2);
    copyVec(vec2,vecTrans2);	  
    getMultVec(vecTrans2,(distSide2-numPointsSide2*tableStep)/2);

    getVecsSum(vecTrans1,vecTrans2,vecTrans);
    getPointBFromPointAAndVecAB(table0,vecTrans,pointStart);
      
    for(int i = 0; i <= numPointsSide1 ; i++){
      for(int j = 0; j <= numPointsSide2 ; j++){
	if( numTablesPoints ==  SDI_F->distancesInfo.numTablePointsMax ){
	  printf("Too many table points in distances info poster. Max is %d\n",SDI_F->distancesInfo.numTablePointsMax);
	  return false;
	}
	else{ 
	  if(SDI_F->distancesInfo.tables[tableIndex].firstTablePointIndex == -1)
	    SDI_F->distancesInfo.tables[tableIndex].firstTablePointIndex = numTablesPoints;
	  copyVec(vec1,vecTrans1);
	  getMultVec(vecTrans1,i*tableStep);
	  copyVec(vec2,vecTrans2);	  
	  getMultVec(vecTrans2,j*tableStep);

	  getVecsSum(vecTrans1,vecTrans2,vecTrans);
	  getPointBFromPointAAndVecAB(pointStart,vecTrans,newPoint);
	  //printVec(newPoint,6+i*numPointsSide1+j);
	    
	  SDI_F->distancesInfo.tablePoints[numTablesPoints].mainEntityIndex = entityIndex;
	  SDI_F->distancesInfo.tablePoints[numTablesPoints].entityIndex = -1;
	  SDI_F->distancesInfo.tablePoints[numTablesPoints].mainEntityFamillyIndex = iter;

	  copyVec(newPoint,SDI_F->distancesInfo.tablePoints[numTablesPoints].position);

	  numTablesPoints++;
	  iter++;
	}
      }
    }
    SDI_F->distancesInfo.numTablePoints = numTablesPoints;
      
    if(SDI_F->distancesInfo.tables[tableIndex].firstTablePointIndex != -1)
      SDI_F->distancesInfo.tables[tableIndex].numTablePoints = numTablesPoints - SDI_F->distancesInfo.tables[tableIndex].firstTablePointIndex;
    return true;
  }
  else 
    return false;	
}


// void updateAngles( void ){
//   //get robot direction vector
//   //POM_EULER *jointAbsPose;
//   int someIter = 0;
//   int numPositions = SDI_F->distancesInfo.numPositions;
//   double angle = 0.0;
//   SPARK_3D_COORD uX; // 
//   SPARK_3D_COORD uY; // 
//   SPARK_3D_COORD uRobot; // 
//   SPARK_3D_COORD uRobotToObject; 
//   SPARK_3D_COORD robotPoint;
//   SPARK_3D_COORD vecProd;

//   p3d_jnt* jnt;
//   jnt = p3d_get_robot_jnt_by_name( gRobot->robotPt, "platformJoint");
//   T3D* jointToOrigin = t3dNew( T3D_MATRIX, T3D_ALLOW_CONVERSION );
//   for(unsigned int i=0; i<4; i++) {
//     for(unsigned int j=0; j<4; j++) {
//       jointToOrigin->matrix[4*i+j] = jnt->abs_pos[i][j];
//       }
//    }
//   t3dConvertTo( T3D_BRYAN, jointToOrigin );
//   double robotYaw = jointToOrigin->euler.yaw;
//   robotPoint.x = jointToOrigin->euler.x;
//   robotPoint.y = jointToOrigin->euler.y;
//   robotPoint.z = 0;

//   //
//   uX.x=cos(robotYaw);
//   uX.y=0;
//   uX.z=0;

//   uY.x=0;
//   uY.y=sin(robotYaw);
//   uY.z=0;

//   SPARK_3D_COORD vecTrans;
//   getVecsSum(uX,uY,uRobot);

//   someIter = 0;
//   for(int i = 0; i< numPositions; i++){
//     if( someIter == SPARK_DISTANCES_INFO_NUM_ANGLES_MAX){
//       printf("Too many angles in distances info poster. Max is %d\n",SPARK_DISTANCES_INFO_NUM_ANGLES_MAX);
//       return;
//     }
//     getVecABFromPointAAndPointB(robotPoint,SDI_F->distancesInfo.positions[i].elemPosition,uRobotToObject);
//     uRobotToObject.z = 0;
//     normalizeVec(uRobotToObject);
//     getVecProd(uRobot,uRobotToObject,vecProd);
//     angle = asin(vecProd.z);
//     SDI_F->distancesInfo.angles[someIter].indexElem1 = -1;
//     SDI_F->distancesInfo.angles[someIter].indexElem2 = i;
//     SDI_F->distancesInfo.angles[someIter].angle = angle; 
//     someIter++;
//   }
//   SDI_F->distancesInfo.numAngles = someIter;
// }


// void deleteEntityInDistanceManagement(int entityIndex){
//   //delete all positions related to this entity
//   int offSetIndex = 0;
//   int numPositions = SDI_F->distancesInfo.numPositions;
//   for(int i = 0; i< numPositions; i++){
//     if(SDI_F->distancesInfo.positions[i].mainEntityIndex == entityIndex){
//       if(SDI_F->distancesInfo.positions[i].elemType == SPARK_DIST_SIMPLE_OBJECT)
// 	SDI_F->distancesInfo.numSimpleObjects--;
//       else if (SDI_F->distancesInfo.positions[i].elemType == SPARK_DIST_SUPPORT_POINT)
// 	SDI_F->distancesInfo.numSupportPoints--;
//       offSetIndex++;
//     }
//     else{
//       if ( offSetIndex > 0){
// 	SDI_F->distancesInfo.positions[i-offSetIndex].mainEntityIndex = SDI_F->distancesInfo.positions[i].mainEntityIndex;
// 	SDI_F->distancesInfo.positions[i-offSetIndex].entityIndex = SDI_F->distancesInfo.positions[i].entityIndex;
// 	SDI_F->distancesInfo.positions[i-offSetIndex].mainEntityFamillyIndex = SDI_F->distancesInfo.positions[i].mainEntityFamillyIndex;
// 	SDI_F->distancesInfo.positions[i-offSetIndex].elemType = SDI_F->distancesInfo.positions[i].elemType;
// 	//position will be udpated.
// 	// SDI_F->distancesInfo.positions[i-offSetIndex].mainEntityIndex = SDI_F->distancesInfo.positions[i].mainEntityIndex;
//       }
//     }
//   } 
//   // if(SDI_F->distancesInfo.numSupportPoints < 0)
//   //   SDI_F->distancesInfo.numSupportPoints = 0;

//   SDI_F->distancesInfo.numPositions = numPositions-offSetIndex;
// }


bool addAgentEntity(int entityIndex,int modifyParameterCommand , double doubleParameterValue,  int intParameterValue, GEN_STRING64  baseJointName,GEN_STRING64  shoulderJointName0,GEN_STRING64  shoulderJointName1){
  if(SDI_F->distancesInfo.numAgents == SDI_F->distancesInfo.numAgentsMax){
    printf("Too many agentts for distance management. Max is %d\n",SDI_F->distancesInfo.numAgentsMax);
    return false;
  }
  int e_i,a_i,h_i,curIndex,numHandsInDistManagementStart,numHeadsInDistManagementStart;
  int curAgentIndex = SDI_F->distancesInfo.numAgents;  
  SDI_F->distancesInfo.agents[curAgentIndex].entityIndex = entityIndex; 
  strcpy(SDI_F->distancesInfo.agents[curAgentIndex].entityName.name,gEntities->entities[entityIndex]->robotPt->name);
  strcpy(SDI_F->distancesInfo.agents[curAgentIndex].baseJointName.name,baseJointName.name);

  SDI_F->distancesInfo.agents[curAgentIndex].allowPointingComputation = FALSE;
  SDI_F->distancesInfo.agents[curAgentIndex].allowGestureComputation = FALSE;
  SDI_F->distancesInfo.agents[curAgentIndex].allowAngleComputation = TRUE;
  

  //find which agent it is.
  //for all agent hand find which entity index it is.
  for(a_i=0; a_i<gAgents->all_agents_no; a_i++){
    if (!strcmp(gEntities->entities[entityIndex]->robotPt->name, gAgents->all_agents[a_i]->robotPt->name)){ 
      ///is it the perceiving agent?
      if(gRobot == gAgents->all_agents[a_i]){
	mySelfRobotAgentIndexInDistancesInfo = curAgentIndex;
	printf("Perceiving robot with index %d in gAgents structure has index %d in distances info structure\n",a_i,curAgentIndex);
      }
      numHandsInDistManagementStart = SDI_F->distancesInfo.numHands;
      SDI_F->distancesInfo.agents[curAgentIndex].firstHandIndex = -1;
      for(h_i=0; h_i<gAgents->all_agents[a_i]->hand_nb; h_i++){
	//is there still place for a new hand and arm
	if(SDI_F->distancesInfo.numHands == SDI_F->distancesInfo.numHandsMax)
	  printf("Too many hands for distance management. num hands is %d max is %d.\n",SDI_F->distancesInfo.numHands,SDI_F->distancesInfo.numHandsMax);
	else{
	  if(h_i>1)
	    printf("We expect as much two hands for agent\n");
	  else{
	    //get hand entity	  
	    for(e_i=0; e_i<gEntities->entities_nb; e_i++) {
	      if (gAgents->all_agents[a_i]->hand[h_i] == gEntities->entities[e_i]){
		curIndex = SDI_F->distancesInfo.numHands;
		//add hand
		if(numHandsInDistManagementStart == curIndex)
		  SDI_F->distancesInfo.agents[curAgentIndex].firstHandIndex = curIndex;
		SDI_F->distancesInfo.hands[curIndex].mainEntityIndex = entityIndex;
		SDI_F->distancesInfo.hands[curIndex].entityIndex = e_i;
		SDI_F->distancesInfo.hands[curIndex].mainEntityFamillyIndex = curIndex-numHandsInDistManagementStart;
	      
		//add arm
		SDI_F->distancesInfo.arms[curIndex].agentEntityIndex = entityIndex;
		SDI_F->distancesInfo.arms[curIndex].mainEntityFamillyIndex = curIndex-numHandsInDistManagementStart;
		if(h_i == 0)
		  strcpy(SDI_F->distancesInfo.arms[curIndex].shoulderJointName.name,shoulderJointName0.name);
		else if(h_i == 1) 
		  strcpy(SDI_F->distancesInfo.arms[curIndex].shoulderJointName.name,shoulderJointName1.name);
		else
		  printf("We expect not more than two hands for agent\n");

		SDI_F->distancesInfo.numHands++;
	      
		//exit look for hand entity
		break;
	      }	    	    	    
	    } //end lok for entity for loop
	  }
	}
      } // end loop on hands
      SDI_F->distancesInfo.agents[curAgentIndex].numHands = SDI_F->distancesInfo.numHands-numHandsInDistManagementStart;

      // Add heads
      numHeadsInDistManagementStart = SDI_F->distancesInfo.numHeads;
      SDI_F->distancesInfo.agents[curAgentIndex].headIndex = -1;

      for(h_i=0; h_i<gAgents->all_agents[a_i]->head_nb; h_i++){
	//is there still place for a new head
	if(SDI_F->distancesInfo.numHeads == SDI_F->distancesInfo.numHeadsMax)
	  printf("Too many heads for distance management. num heads is %d max is %d.\n",SDI_F->distancesInfo.numHeads,SDI_F->distancesInfo.numHeadsMax);
	else{
	  if(h_i>0)
	    printf("We expect at most one heads for agent\n");
	  else{
	    //get head entity	  
	    for(e_i=0; e_i<gEntities->entities_nb; e_i++) {
	      if (gAgents->all_agents[a_i]->head[h_i] == gEntities->entities[e_i]){
		curIndex = SDI_F->distancesInfo.numHeads;
		SDI_F->distancesInfo.agents[curAgentIndex].headIndex = curIndex;

		//add head
		SDI_F->distancesInfo.heads[curIndex].mainEntityIndex = entityIndex;
		SDI_F->distancesInfo.heads[curIndex].entityIndex = e_i;
		SDI_F->distancesInfo.heads[curIndex].mainEntityFamillyIndex = curIndex-numHeadsInDistManagementStart;
		SDI_F->distancesInfo.numHeads++;
		break;
	      }		
	    }
	  }
	}
      }
      SDI_F->distancesInfo.numAgents++;    
      break;
    }    
  }
  if(a_i<gAgents->all_agents_no)
    return true;
  else{
    printf("Agent was not found and cannot be added in Distance Management\n");
    return false;
  }
}

bool deleteAgentEntity(int entityIndex){
  int targetEntityIndex = -1;
  int deletedAgentNumHands = 0;
  int deletedAgentNumHeads = 0;
  int deletedAgentHeadIndex = 0;
  int deletedAgentFirstHandIndex = 0;

  for(int i = 0; i< SDI_F->distancesInfo.numAgents; i++){
    if(SDI_F->distancesInfo.agents[i].entityIndex == entityIndex){
      targetEntityIndex = i;
      break;
    }
  }
  if(targetEntityIndex > -1){
    deletedAgentNumHands = SDI_F->distancesInfo.agents[targetEntityIndex].numHands;
    deletedAgentHeadIndex = SDI_F->distancesInfo.agents[targetEntityIndex].headIndex;
    deletedAgentFirstHandIndex = SDI_F->distancesInfo.agents[targetEntityIndex].firstHandIndex;
    if(deletedAgentHeadIndex != -1)
      deletedAgentNumHeads = 1;
    
    for(int i = targetEntityIndex; i< SDI_F->distancesInfo.numAgents-1;i++){
      SDI_F->distancesInfo.agents[i].entityIndex = SDI_F->distancesInfo.agents[i+1].entityIndex; 
      strcpy(SDI_F->distancesInfo.agents[i].entityName.name,gEntities->entities[i+1]->robotPt->name);

      strcpy(SDI_F->distancesInfo.agents[i].baseJointName.name,SDI_F->distancesInfo.agents[i+1].baseJointName.name);
      SDI_F->distancesInfo.agents[i].allowPointingComputation = SDI_F->distancesInfo.agents[i+1].allowPointingComputation;
      SDI_F->distancesInfo.agents[i].allowGestureComputation = SDI_F->distancesInfo.agents[i+1].allowGestureComputation;
      SDI_F->distancesInfo.agents[i].allowAngleComputation = SDI_F->distancesInfo.agents[i+1].allowAngleComputation;
      SDI_F->distancesInfo.agents[i].firstHandIndex = SDI_F->distancesInfo.agents[i+1].firstHandIndex-deletedAgentNumHands;
      if( SDI_F->distancesInfo.agents[i+1].headIndex == -1)
	SDI_F->distancesInfo.agents[i].headIndex = -1;
      else
	SDI_F->distancesInfo.agents[i].headIndex = SDI_F->distancesInfo.agents[i+1].headIndex - deletedAgentNumHeads;
      SDI_F->distancesInfo.agents[i].numHands = 1;
    }
    SDI_F->distancesInfo.numAgents--;   
    
    //clean hands and arms
    if(deletedAgentNumHands>0){
      for(int i = deletedAgentFirstHandIndex; i< SDI_F->distancesInfo.numHands-deletedAgentNumHands;i++){
	SDI_F->distancesInfo.hands[i].mainEntityIndex = SDI_F->distancesInfo.hands[i+deletedAgentNumHands].mainEntityIndex;
	SDI_F->distancesInfo.hands[i].entityIndex = SDI_F->distancesInfo.hands[i+deletedAgentNumHands].entityIndex;
	SDI_F->distancesInfo.hands[i].mainEntityFamillyIndex = SDI_F->distancesInfo.hands[i+deletedAgentNumHands].mainEntityFamillyIndex;
	      
	//arm
	SDI_F->distancesInfo.arms[i].agentEntityIndex = SDI_F->distancesInfo.arms[i+deletedAgentNumHands].agentEntityIndex;
	SDI_F->distancesInfo.arms[i].mainEntityFamillyIndex = SDI_F->distancesInfo.arms[i+deletedAgentNumHands].mainEntityFamillyIndex;
	strcpy(SDI_F->distancesInfo.arms[i].shoulderJointName.name,SDI_F->distancesInfo.arms[i+deletedAgentNumHands].shoulderJointName.name);
      }
      
      SDI_F->distancesInfo.numHands-=deletedAgentNumHands;
    }

    // clean heads
    if(deletedAgentNumHeads>0){
      for(int i = deletedAgentHeadIndex; i< SDI_F->distancesInfo.numHeads-deletedAgentNumHeads;i++){
	strcpy(SDI_F->distancesInfo.heads[i].eyesJointName.name,SDI_F->distancesInfo.heads[i+deletedAgentNumHeads].eyesJointName.name);
	SDI_F->distancesInfo.heads[i].mainEntityIndex = SDI_F->distancesInfo.heads[i+deletedAgentNumHeads].mainEntityIndex;
	SDI_F->distancesInfo.heads[i].entityIndex = SDI_F->distancesInfo.heads[i+deletedAgentNumHeads].entityIndex;
	SDI_F->distancesInfo.heads[i].mainEntityFamillyIndex = SDI_F->distancesInfo.heads[i+deletedAgentNumHeads].mainEntityFamillyIndex;
      }
      SDI_F->distancesInfo.numHeads-=deletedAgentNumHeads;
    }
  }
  return true;
}


bool checkIsPresentAgentEntity(int entityIndex){
  int targetEntityIndex = -1;

  for(int i = 0; i< SDI_F->distancesInfo.numAgents; i++){
    if(SDI_F->distancesInfo.agents[i].entityIndex == entityIndex){
      targetEntityIndex = i;
      break;
    }
  }
  if(targetEntityIndex > -1){
    return true;
  }
  return false;
}


bool addTableEntity(int entityIndex){
  if(SDI_F->distancesInfo.numTables == SDI_F->distancesInfo.numTablesMax){
    printf("Too many tables for distance management. Max is %d\n",SDI_F->distancesInfo.numTablesMax);
    return false;
  }

  int curIndex = SDI_F->distancesInfo.numTables;  
  SDI_F->distancesInfo.tables[curIndex].entityIndex = entityIndex; 
  strcpy(SDI_F->distancesInfo.tables[curIndex].entityName.name,gEntities->entities[entityIndex]->robotPt->name);
  SDI_F->distancesInfo.tables[curIndex].firstTablePointIndex = -1;

  UpdateTablePointsInDistanceManagement(entityIndex,curIndex);
  SDI_F->distancesInfo.numTables++;
  return true;
}

bool deleteTableEntityTablePoints(int entityIndex,int deletedTableNumTablePoints,int deletedTableFirstTablePointIndex){
  for(int i = deletedTableFirstTablePointIndex; i< SDI_F->distancesInfo.numTablePoints-deletedTableNumTablePoints;  i++){
    SDI_F->distancesInfo.tablePoints[i].mainEntityIndex = SDI_F->distancesInfo.tablePoints[i+deletedTableNumTablePoints].mainEntityIndex;
    SDI_F->distancesInfo.tablePoints[i].entityIndex = -1;
    SDI_F->distancesInfo.tablePoints[i].mainEntityFamillyIndex = SDI_F->distancesInfo.tablePoints[i+deletedTableNumTablePoints].mainEntityFamillyIndex;      
  }
  SDI_F->distancesInfo.numTablePoints-=deletedTableNumTablePoints;
  return true;
}


bool deleteTableEntity(int entityIndex){
  int targetEntityIndex = -1;
  int deletedTableNumTablePoints = 0;
  int deletedTableFirstTablePointIndex = 0;
  for(int i = 0; i< SDI_F->distancesInfo.numTables; i++){
    if(SDI_F->distancesInfo.tables[i].entityIndex == entityIndex){
      targetEntityIndex = i;
      break;
    }
  }
  if(targetEntityIndex > -1){
    deletedTableNumTablePoints = SDI_F->distancesInfo.tables[targetEntityIndex].numTablePoints;

    if(deletedTableNumTablePoints > 0){
      deletedTableFirstTablePointIndex = SDI_F->distancesInfo.tables[targetEntityIndex].firstTablePointIndex;
      deleteTableEntityTablePoints(entityIndex,deletedTableNumTablePoints,deletedTableFirstTablePointIndex);
    }

    for(int i = targetEntityIndex; i< SDI_F->distancesInfo.numTables-1; i++){
      SDI_F->distancesInfo.tables[i].entityIndex = SDI_F->distancesInfo.tables[i+1].entityIndex; 
      strcpy(SDI_F->distancesInfo.tables[i].entityName.name,SDI_F->distancesInfo.tables[i+1].entityName.name);
      SDI_F->distancesInfo.tables[i].numTablePoints = SDI_F->distancesInfo.tables[i+1].numTablePoints;
      SDI_F->distancesInfo.tables[i].firstTablePointIndex = SDI_F->distancesInfo.tables[i+1].firstTablePointIndex-deletedTableNumTablePoints;

      for(int j = 0; j< 4; j++)
	copyVec(SDI_F->distancesInfo.tables[i+1].tableCornersPositions[j],SDI_F->distancesInfo.tables[i].tableCornersPositions[j]);
      
    }
    SDI_F->distancesInfo.numTables--;
  }
  return true;
}

bool checkIsPresentTableEntity(int entityIndex){
  int targetEntityIndex = -1;
  for(int i = 0; i< SDI_F->distancesInfo.numTables; i++){
    if(SDI_F->distancesInfo.tables[i].entityIndex == entityIndex){
      targetEntityIndex = i;
      break;
    }
  }
  if(targetEntityIndex > -1){
    return true;
  }
  return false;
}


bool addObjectEntity(int entityIndex){
  if(SDI_F->distancesInfo.numObjects == SDI_F->distancesInfo.numObjectsMax){
    printf("Too many objects for distance management. Max is %d\n",SDI_F->distancesInfo.numObjectsMax);
    return false;
  }
  int curIndex = SDI_F->distancesInfo.numObjects;  
  SDI_F->distancesInfo.objects[curIndex].entityIndex = entityIndex; 
  strcpy(SDI_F->distancesInfo.objects[curIndex].entityName.name,gEntities->entities[entityIndex]->robotPt->name);
  SDI_F->distancesInfo.numObjects++;
  return true;
}

bool deleteObjectEntity(int entityIndex){
  int targetEntityIndex = -1;
  for(int i = 0; i< SDI_F->distancesInfo.numObjects; i++){
    if(SDI_F->distancesInfo.objects[i].entityIndex == entityIndex){
      targetEntityIndex = i;
      break;
    }
  }
  if(targetEntityIndex > -1){
    for(int i = targetEntityIndex; i< SDI_F->distancesInfo.numObjects-1; i++){
      SDI_F->distancesInfo.objects[i].entityIndex = SDI_F->distancesInfo.objects[i+1].entityIndex;
      strcpy(SDI_F->distancesInfo.objects[i].entityName.name,SDI_F->distancesInfo.objects[i+1].entityName.name);
    }
    SDI_F->distancesInfo.numObjects--;
  }
  return true;
}

bool checkIsPresentObjectEntity(int entityIndex){
  int targetEntityIndex = -1;
  for(int i = 0; i< SDI_F->distancesInfo.numObjects; i++){
    if(SDI_F->distancesInfo.objects[i].entityIndex == entityIndex){
      targetEntityIndex = i;
      break;
    }
  }
  if(targetEntityIndex > -1){
    return true;
  }
  else
    return false;
}

bool checkEntityIsPresentInDistanceManagement(int entityIndex){

  //right now, just copy the entity in table.
  if(gEntities->entities[entityIndex]->subtype == HRI_OBJECT_SUPPORT){
    return checkIsPresentTableEntity(entityIndex);
  }
   
  else if(gEntities->entities[entityIndex]->subtype == HRI_MOVABLE_OBJECT){
    return checkIsPresentObjectEntity(entityIndex);    
  }

  else if(gEntities->entities[entityIndex]->type == HRI_ISAGENT){
    return checkIsPresentAgentEntity(entityIndex);
  }
  else{ 
    printf("Unmanaged entity type %d and subtype %d for distance management.\n",gEntities->entities[entityIndex]->type,gEntities->entities[entityIndex]->subtype);
    return false;
  }
}

bool deleteEntityInDistanceManagement(int entityIndex){

  //right now, just copy the entity in table.
  if(gEntities->entities[entityIndex]->subtype == HRI_OBJECT_SUPPORT){
    return deleteTableEntity(entityIndex);
  }
   
  else if(gEntities->entities[entityIndex]->subtype == HRI_MOVABLE_OBJECT){
    return deleteObjectEntity(entityIndex);    
  }

  else if(gEntities->entities[entityIndex]->type == HRI_ISAGENT){
    return deleteAgentEntity(entityIndex);
  }
  else{ 
    printf("Unmanaged entity type %d and subtype %d for distance management.\n",gEntities->entities[entityIndex]->type,gEntities->entities[entityIndex]->subtype);
    return false;
  }
}


bool addEntityInDistanceManagement(int entityIndex,int modifyParameterCommand , double doubleParameterValue,  int intParameterValue, GEN_STRING64  baseJointName,GEN_STRING64  shoulderJointName0,GEN_STRING64  shoulderJointName1){

  //right now, just copy the entity in table.
  if(gEntities->entities[entityIndex]->subtype == HRI_OBJECT_SUPPORT){
    return addTableEntity(entityIndex);
  }
   
  else if( (gEntities->entities[entityIndex]->subtype == HRI_MOVABLE_OBJECT) || (gEntities->entities[entityIndex]->subtype == HRI_OBJECT_CONTAINER)) {
    return addObjectEntity(entityIndex);    
  }

  else if(gEntities->entities[entityIndex]->type == HRI_ISAGENT){
    return addAgentEntity(entityIndex,modifyParameterCommand, doubleParameterValue, intParameterValue,baseJointName,shoulderJointName0,shoulderJointName1);
  }
  else{ 
    printf("Unmanaged entity type %d and subtype %d for distance management.\n",gEntities->entities[entityIndex]->type,gEntities->entities[entityIndex]->subtype);
    return false;
  }
}

void getPointDistToTableFromPointOutsideTableRectangle (SPARK_3D_COORD point, SPARK_3D_COORD tableCorners[], double &dist,SPARK_3D_COORD &closestTablePoint){

  double minDist = 0.0;
  double maxDist = 0.0;
  int minDistIndex = 0;
  int maxDistIndex = 0;

  double some2DDist[4];
  double rectangleL = 0;
  double alpha;

  SPARK_3D_COORD pointA;
  SPARK_3D_COORD pointB[2];
  SPARK_3D_COORD closestTablePointTmp[2];
  double distTmp[2];

  //get closest and furthest corners.
  for(int i = 0; i<4 ; i++){
    some2DDist[i] = get2DDistanceBetweenPoints(point,tableCorners[i]);
    if(i == 0){
      minDist = some2DDist[0];
      maxDist = some2DDist[0];
      minDistIndex = 0;
      maxDistIndex = 0;
    }
    else{
      if(some2DDist[i] < minDist){
	minDist = some2DDist[i];
	minDistIndex = i;
      }
      if(some2DDist[i] > maxDist){
	maxDist = some2DDist[i];
	maxDistIndex = i;
      }	        
    }
  }
  
  // to get the closest point, we have to compute the orthogonal projection with the two rectangle side that link the closest corner and the not furthest two other corners.
  
  copyVec(tableCorners[minDistIndex],pointA);
  int iterOnOtherCorners = 0;
  for(int i = 0; i<4 ; i++){
    if( (i != minDistIndex) && (i != maxDistIndex) ){
      copyVec(tableCorners[i],pointB[iterOnOtherCorners]);
      //distToB[i]=some2DDist[i];

      rectangleL = get2DDistanceBetweenPoints(pointA,pointB[iterOnOtherCorners]);
      /// vecAI = alpha * vecAB
      ///use pythagore to compute alpha so that vecAI = alpha*vecAB
      ///if alpha > 0 I is the closest point otherwise it is A.      
      alpha = (pow(minDist,2)-pow(some2DDist[iterOnOtherCorners],2))/(2*pow(rectangleL,2))+0.5;
      if(alpha>0)
	getPointAtAlphaFromVecAB(alpha,pointA,pointB[iterOnOtherCorners],closestTablePointTmp[iterOnOtherCorners]);
      else
	copyVec(pointA,closestTablePointTmp[iterOnOtherCorners]);
  
      distTmp[iterOnOtherCorners] = getDistanceBetweenPoints(point,closestTablePointTmp[iterOnOtherCorners]);
      iterOnOtherCorners++;
    }    
  }
  
  // find which distance is the smallest. Save it in the iterator.
  if(distTmp[0]<distTmp[1])
    iterOnOtherCorners = 0;
  else
    iterOnOtherCorners = 1;

  dist = distTmp[iterOnOtherCorners]; 
  copyVec(closestTablePointTmp[iterOnOtherCorners],closestTablePoint);
  
  closestTablePoint.z = point.z;
}

// bool getIntersectionWithHorizontalPlan(SPARK_3D_COORD startPoint,SPARK_3D_COORD directionVec,double planHeight,SPARK_3D_COORD &planInterPoint){

bool isPointInsideTableRectangle(SPARK_3D_COORD point, SPARK_3D_COORD tableCorners[]){
  SPARK_3D_COORD vecToCorners[4];
  SPARK_3D_COORD prodVec[3];

  double angles[3];
  double minAngle = 0;
  double maxAngle = 0;
  double angleRange = 0;
  double scalProd = 0;

  // compute angle between vecPointToCorner.
  // printf("Check is inside table rectangle\n");
  // printf("point coordinates : %f %f %f\n",point.x,point.y,point.z);
  for(int i = 0; i<4 ; i++){
    getVecABFromPointAAndPointB(point,tableCorners[i],vecToCorners[i]);
    vecToCorners[i].z = 0;
    normalizeVec(vecToCorners[i]);

    if( i > 0){
      getVecProdVecOnPlanZ0(vecToCorners[0],vecToCorners[i],prodVec[i-1]);

      angles[i-1] = asin(prodVec[i-1].z);
      ///check if angle is actually bigger than m
      scalProd = getVecScalProd(vecToCorners[0],vecToCorners[i]);
      if(scalProd < 0){
	if(angles[i-1]>0)
	  angles[i-1] = M_PI - angles[i-1];
	else
	  angles[i-1] = -1*M_PI - angles[i-1];
      }
      if (i == 1){
	minAngle = angles[i-1];
	maxAngle = angles[i-1];
      }
      else{
	if(angles[i-1]<minAngle)
	  minAngle = angles[i-1];
	if(angles[i-1]>maxAngle)
	  maxAngle = angles[i-1];
      }      
    }
  }
  angleRange = maxAngle - minAngle;
    
  //point is inside table rectangle if angleRange > PI
  if(angleRange > M_PI)
    return true;
  else
    return false;  
}

void getPointDistToTable(SPARK_3D_COORD point, SPARK_3D_COORD tableCorners[], double &dist, int& isAbove,SPARK_3D_COORD &closestTablePoint){

  double tableHeight = tableCorners[0].z;

  if(isPointInsideTableRectangle(point,tableCorners)){
    dist = fabs(point.z - tableHeight);
    copyVec(point,closestTablePoint);
    //closestTablePoint.z = tableHeight; 
    isAbove = 1;
  }
  else{
    getPointDistToTableFromPointOutsideTableRectangle(point,tableCorners,dist,closestTablePoint);
    isAbove = 0;
  }
}

bool isArmPointingTowardTable(double maxDistFromTable,SPARK_3D_COORD shoulderPoint,SPARK_3D_COORD armVector, SPARK_3D_COORD tableCorners[],SPARK_3D_COORD &pointPointedOnTablePlan){
  double tableHeight = tableCorners[0].z;
  double someDist = 0;
  SPARK_3D_COORD intersectionWithTablePlan;
  SPARK_3D_COORD closestTablePlan;
  
  if(getIntersectionWithHorizontalPlan(shoulderPoint,armVector,tableHeight,intersectionWithTablePlan)){
    bool isPointInsideTableRectangleBool = isPointInsideTableRectangle(intersectionWithTablePlan,tableCorners);
    copyVec(intersectionWithTablePlan,pointPointedOnTablePlan);
    if(isPointInsideTableRectangleBool){
      return true;
    }
    else{
      getPointDistToTableFromPointOutsideTableRectangle(intersectionWithTablePlan,tableCorners,someDist,closestTablePlan);
      if(someDist<maxDistFromTable)
	return true;
      else
	return false;
    }
  }
  else
    return false;
}

void getJointXYZYaw (p3d_rob* rob , GEN_STRING64 jointName ,SPARK_3D_COORD &jointCoord, double &yaw,double &pitch){
  p3d_jnt* jnt;
  jnt = p3d_get_robot_jnt_by_name( rob, jointName.name);
  T3D* jointToOrigin = t3dNew( T3D_MATRIX, T3D_ALLOW_CONVERSION );
  for(unsigned int i=0; i<4; i++) {
    for(unsigned int j=0; j<4; j++) {
      jointToOrigin->matrix[4*i+j] = jnt->abs_pos[i][j];
    }
  }
  t3dConvertTo( T3D_BRYAN, jointToOrigin );
  yaw = jointToOrigin->euler.yaw;
  pitch = jointToOrigin->euler.pitch;
  jointCoord.x = jointToOrigin->euler.x;
  jointCoord.y = jointToOrigin->euler.y;
  jointCoord.z = jointToOrigin->euler.z;
}

void getOrientationVector(double yaw, SPARK_3D_COORD &orientation){
  orientation.x=cos(yaw);
  orientation.y=sin(yaw);
  orientation.z=0;
}


void getEntityBBCenter( HRI_ENTITY * entity, bool useZMax, SPARK_3D_COORD &centerPoint){
  if((entity->type == HRI_OBJECT_PART) || (entity->type == HRI_AGENT_PART)) {
    centerPoint.x = (entity->partPt->BB.xmax + entity->partPt->BB.xmin)/2;
    centerPoint.y = (entity->partPt->BB.ymax + entity->partPt->BB.ymin)/2;
    //we want table top
    if( useZMax )
      centerPoint.z = entity->partPt->BB.zmax;
    else
      centerPoint.z = (entity->partPt->BB.zmax + entity->partPt->BB.zmin)/2;
  }
  else {
    centerPoint.x = (entity->robotPt->BB.xmax + entity->robotPt->BB.xmin)/2;
    centerPoint.y = (entity->robotPt->BB.ymax + entity->robotPt->BB.ymin)/2;
    //we want table top
    if( useZMax )
      centerPoint.z = entity->robotPt->BB.zmax;
    else
      centerPoint.z = (entity->robotPt->BB.zmax + entity->robotPt->BB.zmin)/2;
  }
}

void updateDistAndMinDist( int iter1 , int iter2, SPARK_3D_COORD point1, SPARK_3D_COORD point2, int forceReinitMinDist, double aTime,int &index1,int &index2, double &curDist, double &minDist , double &minDistTime ){

  index1 = iter1;
  index2 = iter2;
  curDist = getDistanceBetweenPoints(point1,point2);
  
  //update minDist if needed
  if ( (curDist < minDist) || forceReinitMinDist){
    minDist = curDist;
    minDistTime = aTime;
  }
}

void updateAngleNew( int indexInput1 , int indexInput2, SPARK_3D_COORD orientation, SPARK_3D_COORD pointStart, SPARK_3D_COORD pointEnd,int &index1,int &index2, double &curAngle){
  SPARK_3D_COORD vec;
  SPARK_3D_COORD prodVec;
  index1 = indexInput1;
  index2 = indexInput2;
 
  getVecABFromPointAAndPointB(pointStart,pointEnd,vec);
  vec.z = 0;
  normalizeVec(vec);
  
  getVecProdVecOnPlanZ0(orientation,vec,prodVec);
  curAngle = asin(prodVec.z);  
}


void updatePointing( int indexInput1 , int indexInput2, SPARK_3D_COORD shoulderToHandVec, SPARK_3D_COORD shoulder, SPARK_3D_COORD pointEnd,double armLength,double armLengthMax,int &index1,int &index2, SPARK_3D_COORD & optimalPointingVec, SPARK_3D_COORD & optimalPointingPoint , double & curAngleFromOptimalPointingVec ,double & pointingScore){

  SPARK_3D_COORD normalizedPointingVec;
  //SPARK_3D_COORD normalizedArmVec;
  double scalProd = 0;

  index1 = indexInput1;
  index2 = indexInput2;
 
  //get normalized pointing vec.
  getVecABFromPointAAndPointB(shoulder,pointEnd,normalizedPointingVec);
  normalizeVec(normalizedPointingVec);

  //compute angle between pointing vec and arm vec
  scalProd = getVecScalProd(shoulderToHandVec,normalizedPointingVec);
  curAngleFromOptimalPointingVec = acos(scalProd);

  //get optimal pointing vec.
  copyVec(normalizedPointingVec,optimalPointingVec);
  getMultVec(optimalPointingVec,0.9*armLengthMax);

  //compute optimal pointing point.
  getPointBFromPointAAndVecAB(shoulder,optimalPointingVec,optimalPointingPoint);
  
  //update score.
  if(armLength > armLength*0.8){
    if(curAngleFromOptimalPointingVec < M_PI/8){
      if(pointingScore<1.0)
	pointingScore+=0.1;    
      else{
	if(pointingScore>0)
	  pointingScore-=0.1;    
      }
    }
    else{
      if(pointingScore>0)
	pointingScore-=0.1; 
    }
  }
  else{
    if(pointingScore>0)
      pointingScore-=0.1;   
  }
}

bool getTrajIndexAndTimeStepIndex(int agentIndex,int handIndex, int entityIndex, int& trajIndex,int &timeStepIndex){
  int firstFreeTrajIndex = -1;
  trajIndex = -1;
  timeStepIndex = 0;
  for(int i = 0; i< SPARK_DISTANCES_INFO_NUM_TRAJECTORIES_MAX; i++){

    if((firstFreeTrajIndex == -1) && (SDI_F->distancesInfo.trajectories[i].agentIndex == -1))
      firstFreeTrajIndex = i;
    
    if((agentIndex == SDI_F->distancesInfo.trajectories[i].agentIndex) && (handIndex == SDI_F->distancesInfo.trajectories[i].handIndex) && (entityIndex == SDI_F->distancesInfo.trajectories[i].entityIndex)){
      trajIndex = i;
      timeStepIndex = SDI_F->distancesInfo.trajectories[i].curTimeStepIndex+1;
      break;
    }
  }
  
  if(trajIndex == -1){
    if(firstFreeTrajIndex != -1){
      trajIndex = firstFreeTrajIndex;
      timeStepIndex = 0;
    }
    else{
      printf("no more free trajectories index\n");	 
      return false;
    }
  }
  else if(timeStepIndex == SPARK_DISTANCES_INFO_NUM_STEP_IN_TRAJECTORY_MAX){
    //We put 
    //trajIndex = firstFreeTrajIndex;
    timeStepIndex = 0;
  }
  return true;
}


bool getDistHistoricIndexAndTimeStepIndex(int agentIndex,int handIndex, int entityIndex, int& distHistoricIndex,int &timeStepIndex){
  int firstFreeDistHistoricIndex = -1;
  distHistoricIndex = -1;
  timeStepIndex = 0;
  for(int i = 0; i< SPARK_DISTANCES_INFO_NUM_TRAJECTORIES_MAX; i++){

    if((firstFreeDistHistoricIndex == -1) && (SDI_F->distancesInfo.distanceHistories[i].agentIndex == -1))
      firstFreeDistHistoricIndex = i;
    
    if((agentIndex == SDI_F->distancesInfo.distanceHistories[i].agentIndex) && (handIndex == SDI_F->distancesInfo.distanceHistories[i].handIndex) && (entityIndex == SDI_F->distancesInfo.distanceHistories[i].entityIndex)){
      distHistoricIndex = i;
      timeStepIndex = SDI_F->distancesInfo.distanceHistories[i].curTimeStepIndex+1;
      break;
    }
  }
  
  if((distHistoricIndex == -1) || (timeStepIndex == SPARK_DISTANCES_INFO_NUM_STEP_IN_TRAJECTORY_MAX)){
    if( firstFreeDistHistoricIndex != -1){
      distHistoricIndex = firstFreeDistHistoricIndex;
      timeStepIndex = 0;
    }
    else{
      printf("no more free histories index\n");	 
      return false;
    }
      
  }
  return true;
}

// bool findHandTrajectoryFreeIndex(int &freeIndex){
//   for(int i = 0; i< SPARK_DISTANCES_INFO_NUM_TRAJECTORIES_MAX; i++){
//     if(SDI_F->distancesInfo.trajectories[i].handIndex == -1){
//       freeIndex = i;
//       return true;
//     }
//   }
//   // all slot are taken. We take first one
//   freeIndex = 0;
//   return false;
// }

// bool findDistanceHistoricFreeIndex(int &freeIndex){
//   for(int i = 0; i< SPARK_DISTANCES_INFO_NUM_TRAJECTORIES_MAX; i++){
//     if(SDI_F->distancesInfo.distanceHistories[i].handIndex == -1){
//       freeIndex = i;
//       return true;
//     }
//   }
//   // all slot are taken. We take first one
//   freeIndex = 0;
//   return false;
// }

void updateDistAndMinDistToTable( int agentIndex, int indexInput1 ,int indexInput2, bool isHand , bool saveHistory , double saveHistoryDistThreshold,  SPARK_3D_COORD point1, SPARK_3D_COORD tableCorners[], int forceReinitMinDist, double aTime,int &index1,int &index2, double &curDist,  int &isAbove , SPARK_3D_COORD &closestPoint ,double &minDist , double &minDistTime , SPARK_3D_COORD &minDistPoint ,int &trajIndex , int &distHistIndex , int &timeStepIndex ){

  double dist = 0;
  int handIndex = indexInput1;
  int entityIndex = indexInput2;
  getPointDistToTable(point1,tableCorners,dist,isAbove,closestPoint);
  curDist = dist;

  if ( isHand && saveHistory && (dist < saveHistoryDistThreshold)){
    bool saveTraj = getTrajIndexAndTimeStepIndex(agentIndex,handIndex, entityIndex,trajIndex,timeStepIndex);

    if( saveTraj && (timeStepIndex == 0) ){
      SDI_F->distancesInfo.trajectories[trajIndex].agentIndex = agentIndex;
      SDI_F->distancesInfo.trajectories[trajIndex].handIndex = handIndex;
      SDI_F->distancesInfo.trajectories[trajIndex].entityIndex = entityIndex;
    }
    
    bool saveValue = saveTraj;
    //check distance between previous value
    if( saveValue && (timeStepIndex>0)){
      if(getDistanceBetweenPoints(SDI_F->distancesInfo.trajectories[trajIndex].positions[timeStepIndex-1],closestPoint)<0.05)
	saveValue = false;      
    }
    
    
    if( saveValue ){ 
      //copyVec(closestPoint,SDI_F->distancesInfo.trajectories[trajIndex].positions[timeStepIndex]);
      copyVec(point1,SDI_F->distancesInfo.trajectories[trajIndex].positions[timeStepIndex]);
      SDI_F->distancesInfo.trajectories[trajIndex].timeSteps[timeStepIndex] = aTime;
      SDI_F->distancesInfo.trajectories[trajIndex].curTimeStepIndex = timeStepIndex;
    }

    // bool saveDistHistories = getDistHistoricIndexAndTimeStepIndex(agentIndex,handIndex, entityIndex,distHistIndex,timeStepIndex);
    //   SDI_F->distancesInfo.distanceHistories[distHistIndex].handIndex = indexInput1;
    //   SDI_F->distancesInfo.distanceHistories[distHistIndex].index = indexInput2;
    //   timeStepIndex = 0;
    // }
    // SDI_F->distancesInfo.distanceHistories[distHistIndex].distances[timeStepIndex] = dist;
    // SDI_F->distancesInfo.distanceHistories[distHistIndex].timeSteps[timeStepIndex] = aTime;
    // timeStepIndex++;
  }
  
  //update minDist if needed
  if ( (curDist < minDist) || forceReinitMinDist){
    minDist = curDist;
    minDistTime = aTime;
    copyVec(point1,minDistPoint);
  }
}

void updateArmSemaphoreAlphabet(SPARK_3D_COORD orientation,SPARK_3D_COORD armVector,double armVectorLength,double armVectorLengthMax, SPARK_DISTANCES_ARM_ANGLE_INFO &armAngleInfo){

  // check length is big enough
  if(armVectorLength > armVectorLengthMax*0.8){
    // normalize arm vector
    SPARK_3D_COORD normalizedArmVector;
    SPARK_3D_COORD verticalVec;
    SPARK_3D_COORD prodVec;
    SPARK_3D_COORD someTranslation;

    double scalProd,angle,angleFromPlan;
    copyVec(armVector,normalizedArmVector);
    normalizeVec(normalizedArmVector);
    scalProd = getVecScalProd(normalizedArmVector,orientation);

    armAngleInfo.curAngleScore = 1-fabs(scalProd);
    
    // check arm vector is in plan orthogonal to orientation
    angleFromPlan = acos(scalProd);
    angleFromPlan -= M_PI/2;
    if(fabs(angleFromPlan)<M_PI/6){

      // get vertical
      createVec(0,0,1,verticalVec);
      
      //get exact normalized projection
      copyVec(orientation,someTranslation);
      getMultVec(someTranslation,-scalProd);
      getVecsSum(normalizedArmVector,someTranslation,normalizedArmVector);
      normalizeVec(normalizedArmVector);

      // get angle
      getVecProd(verticalVec,normalizedArmVector,prodVec);
      scalProd=getVecScalProd(prodVec,orientation);
      angle = asin(scalProd);
      
      // compare to average angle
      armAngleInfo.curAngle = angle;

      if(fabs(armAngleInfo.angleRef - angle) < M_PI/8){
	if(armAngleInfo.angleRefScore<1.0)
	  armAngleInfo.angleRefScore+=0.1;
      }
      else{
	if(armAngleInfo.angleRefScore>0)
	  armAngleInfo.angleRefScore-=0.1;
	else
	  armAngleInfo.angleRef = angle;
      }
    }
    else{
      if(armAngleInfo.angleRefScore>0)
	armAngleInfo.angleRefScore-=0.1;
    }
  }
  else{
    if(armAngleInfo.angleRefScore>0)
      armAngleInfo.angleRefScore-=0.1;
  }
}

///
/// pan = yaw : angle around Z axis : horizontal component of the look at vector
/// tilt = pitch : angle around the horizontal vector  orthogonal to the horizontal component of the look at vector

void getLookAtPanTilt(int agentIndex,double & pan,double & tilt,SPARK_3D_COORD & agentEyesCenter,SPARK_3D_COORD & agentLookAtUnitaryVector,double & yawBase){
  int entityIndex;
  HRI_ENTITY * entity;
  p3d_rob* rob;
  SPARK_3D_COORD aPoint;
  //SPARK_3D_COORD vecX, vecY, vecXY, vecZ;
  double pitchBase,yawEyes,pitchEyes,yawDiff,pitchDiff;
  GEN_STRING64 eyesJointName;
  strcpy(eyesJointName.name,"Eyes");
  entityIndex = SDI_F->distancesInfo.agents[agentIndex].entityIndex;
  entity = gEntities->entities[entityIndex];

  //getEntityBBCenter(entity,false,SDI_F->distancesInfo.agents.position); 
  rob = entity->robotPt;
  getJointXYZYaw ( rob, SDI_F->distancesInfo.agents[agentIndex].baseJointName ,aPoint, yawBase,pitchBase);
  getJointXYZYaw ( rob, eyesJointName ,agentEyesCenter, yawEyes,pitchEyes);
  yawDiff = yawEyes - yawBase;
  
  pitchDiff  = pitchEyes;/// We assume pitchBase = 0. This is not true for human. TO CHECK!!!1; 
  
  ///we want value in the [-M_PI ; M_PI] range.
  if(yawDiff > M_PI )
    yawDiff = yawDiff - 2*M_PI;
  else if(yawDiff < -M_PI )
    yawDiff = yawDiff + 2*M_PI;

  if(pitchDiff > M_PI )
    pitchDiff = pitchDiff - 2*M_PI;
  else if(pitchDiff < -M_PI )
    pitchDiff = pitchDiff + 2*M_PI;
  

  pan = yawDiff;
  tilt = pitchDiff;

  //build look at unitary vector
  // createVec(1,0,0,vecX);
  // createVec(0,1,0,vecY);
  // createVec(0,0,1,vecZ);
  // getMultVec(vecX,cos(yawDiff));
  // getMultVec(vecY,sin(yawDiff));
  // getVecsSum(vecX,vecY,vecXY);
  // getMultVec(vecXY,cos(pitchDiff));
  // getMultVec(vecZ,sin(pitchDiff));
  // getVecsSum(vecXY,vecZ,agentLookAtUnitaryVector);

  //printf("Agent %d : yaw base %f pitch base %f yaw eyes %f pitch eyes %f  yaw diff %f pitch diff %f  eyes x %f  eyes y %f  eyes z %f\n",agentIndex, yawBase , pitchBase , yawEyes ,pitchEyes, yawDiff ,pitchDiff,agentEyesCenter.x,agentEyesCenter.y,agentEyesCenter.z);

}

///
/// update four closest neighboors.
/// Use in the two functions below.
///

void updateFourClosestNeighboor(SPARK_DIST_ELEM_TYPE type, int index,double dist, SPARK_DIST_ELEM_TYPE & positionStatePosition1Type, int & positionStatePosition1Index, double & distance1, SPARK_DIST_ELEM_TYPE & positionStatePosition2Type, int & positionStatePosition2Index, double & distance2, SPARK_DIST_ELEM_TYPE & positionStatePosition3Type, int & positionStatePosition3Index, double & distance3, SPARK_DIST_ELEM_TYPE & positionStatePosition4Type, int & positionStatePosition4Index, double & distance4,SPARK_DIST_ELEM_TYPE & containerType, int & containerObjectIndex, double & containerDistance){

  /// manage closest container
  if(type == SPARK_DIST_CONTAINER){
    if(dist < containerDistance){
      containerObjectIndex = index;
      containerDistance = dist;
    }
  }

  int closestLevel = 5;
  if((dist < distance4) || (positionStatePosition4Index == -1)){
    closestLevel = 4;
    if((dist < distance3) || (positionStatePosition3Index == -1)){
      closestLevel = 3;
      if((dist < distance2) || (positionStatePosition2Index == -1)){
	closestLevel = 2;
	if((dist < distance1) || (positionStatePosition1Index == -1)){
	  closestLevel = 1;
	}
      }
    }
  }

      
  if(closestLevel < 5){
    if(closestLevel == 4){
      positionStatePosition4Type = type;
      positionStatePosition4Index = index;
      distance4 = dist;
    }
    else{
      /// 3 become 4 for sure
      positionStatePosition4Type = positionStatePosition3Type;
      positionStatePosition4Index = positionStatePosition3Index;
      distance4 = distance3;
	  
      if(closestLevel == 3){
	positionStatePosition3Type = type;
	positionStatePosition3Index = index;
	distance3 = dist;
      }
	  
      else{
	/// 2 become 3 for sure
	positionStatePosition3Type = positionStatePosition2Type;
	positionStatePosition3Index = positionStatePosition2Index;
	distance3 = distance2;
	    
	if(closestLevel == 2){
	  positionStatePosition2Type = type;
	  positionStatePosition2Index = index;
	  distance2 = dist;
	}
	else{	      
	  /// 1 become 2 for sure
	  positionStatePosition2Type = positionStatePosition1Type;
	  positionStatePosition2Index = positionStatePosition1Index;
	  distance2 = distance1;

	  positionStatePosition1Type = type;
	  positionStatePosition1Index = index;
	  distance1 = dist;
	}
      }
    }
  }    
}


//
// We asssume robot is on horizontal surface so that we
//

bool getTargetPanTiltToLookAtPosition(SPARK_3D_COORD agentEyesCenter,SPARK_3D_COORD targetPosition,double baseYaw, double & pan, double & tilt){
  SPARK_3D_COORD toTargetPositionVec;
  SPARK_3D_COORD verticalVec;
  SPARK_3D_COORD vecX;
  SPARK_3D_COORD vecY;
  SPARK_3D_COORD baseHorizontalVector;
  SPARK_3D_COORD toTargetPositionVecHorizontalComponent;
  SPARK_3D_COORD toTargetPositionVecVerticalComponent;
  double scalProd = 0.0;
  bool isLookingDown = true;
  SPARK_3D_COORD vecProd;
  
  getVecABFromPointAAndPointB(agentEyesCenter,targetPosition,toTargetPositionVec);
  createVec(0,0,1,verticalVec);
  
  // get vertical component through projection
  scalProd = getVecScalProd(toTargetPositionVec,verticalVec);
  if(scalProd > 0)
    isLookingDown = false;
  createVec(0,0,1,toTargetPositionVecVerticalComponent);
  getMultVec(toTargetPositionVecVerticalComponent,scalProd);
  
  // get vertical component taking out vertical component
  getVecBCFromVecACAndVecAB(toTargetPositionVec,toTargetPositionVecVerticalComponent,toTargetPositionVecHorizontalComponent);

  //normalize vertical and horizontal component.
  normalizeVec(toTargetPositionVecVerticalComponent);
  normalizeVec(toTargetPositionVecHorizontalComponent);
  normalizeVec(toTargetPositionVec);

  // printf"START PRINT LOOKAT TARGET : \n");
  // printVec(agentEyesCenter,0);
  // printVec(targetPosition,1);
  // printVec(toTargetPositionVec,2);
  // printVec(toTargetPositionVecVerticalComponent,3);
  // printVec(toTargetPositionVecHorizontalComponent,4);
  // printf("prod scal vertical component %f\n",scalProd);

  /////////////////////////
  //get pan/yaw
  //firt get base vector.
  /////////////////////////
  createVec(1,0,0,vecX);
  createVec(0,1,0,vecY);
  getMultVec(vecX,cos(baseYaw));
  getMultVec(vecY,sin(baseYaw));
  getVecsSum(vecX,vecY,baseHorizontalVector);
  normalizeVec(baseHorizontalVector);

  // scalar product between baseHorizontalVector and toTargetPositionVecHorizontalComponent
  scalProd=getVecScalProd(baseHorizontalVector,toTargetPositionVecHorizontalComponent); 
  pan = acos(scalProd);
  // is pan negative?
  // use vec product between baseHorizontalVector and toTargetPositionVecHorizontalComponent
  getVecProdVecOfHorizontalVec(baseHorizontalVector,toTargetPositionVecHorizontalComponent,vecProd);
  if(vecProd.z < 0)
    pan = -pan;

  // printVec(baseHorizontalVector,5);
  // printf("prod scal pan %f\n",scalProd);

  ///////////////
  //get tilt
  ///////////////
  //scalar product between toTargetPositionVecHorizontalComponent,toTargetPositionVecVerticalComponent
  scalProd=getVecScalProd(toTargetPositionVecHorizontalComponent,toTargetPositionVec); 
  tilt = acos(scalProd);
  if(!isLookingDown)
    tilt = -tilt;
  // printf("prod scal tilt %f\n",scalProd);
  // printf("pan and tilt %f %f\n",pan,tilt);
  ///printf("Agent %d : yaw base %f pitch base %f yaw eyes %f pitch eyes %f  yaw diff %f pitch diff %f  eyes x %f  eyes y %f  eyes z %f\n",agentIndex, yawBase , pitchBase , yawEyes ,pitchEyes, yawDiff ,pitchDiff,agentEyesCenter.x,agentEyesCenter.y,agentEyesCenter.z);
  
  return true;
}

bool getTargetPanTiltTargetPosition(int index,SPARK_DIST_ELEM_TYPE type,SPARK_3D_COORD & position){
  
  bool indexTooBig = false;

  if( type == SPARK_DIST_AGENT_HAND){
    if(index < SDI_F->distancesInfo.numHands)
      position = SDI_F->distancesInfo.hands[index].position;
    else
      indexTooBig = true;
  }

  // manage simple objects
  else if(type == SPARK_DIST_SIMPLE_OBJECT){
    if(index < SDI_F->distancesInfo.numObjects)
      position = SDI_F->distancesInfo.objects[index].position;
    else
      indexTooBig = true;
  }

  // manage table points
  else if(type == SPARK_DIST_SUPPORT_POINT){
    if(index < SDI_F->distancesInfo.numTablePoints)
      position = SDI_F->distancesInfo.tablePoints[index].position;
    else
      indexTooBig = true;

  }

  // manage heads
  else if( type == SPARK_DIST_AGENT_HEAD ){
    if(index < SDI_F->distancesInfo.numHeads)
      position = SDI_F->distancesInfo.heads[index].position;
    else
      indexTooBig = true;
  }

  if(indexTooBig){
    printf("getTargetPanTiltTargetPosition : index %d too big for type %d  \n",index,type);
    return false;
  }
  else
    return true;
}


bool computeRobotPanTiltToLookAtSignificantPositions(SPARK_3D_COORD robotEyesCenter,double robotBaseYaw, bool someTablesHaveMoved){
  //double curPan = SDI_F->allPanTiltInfos.curPan;
  //double curTilt = SDI_F->allPanTiltInfos.curTilt;
  //int numPanTiltMax = SDI_F->allPanTiltInfos.numMax;
  SPARK_3D_COORD targetPosition;
  int panTiltIndex = 0;
  int index = 0;
  SPARK_DIST_ELEM_TYPE type = SPARK_DIST_AGENT_HAND;

  int numHands = SDI_F->distancesInfo.numHands;
  int numHeads = SDI_F->distancesInfo.numHeads;
  int numObjects = SDI_F->distancesInfo.numObjects;
  //int numTables = SDI_F->distancesInfo.numTables;
  int numTablePoints = SDI_F->distancesInfo.numTablePoints;

  SPARK_LOOKAT_TARGET_PAN_TILT * curLookAtTargetPanTiltStruct;

  //int distanceInfoIndex = 0;
  int entityIndex = 0;
  int mainEntityIndex = 0;

  bool reinitIndex = false;
  bool recomputePanTilt = false;

  ///
  /// to test that trashbin are position before other objects
  /// Error otherwise : It should be improved.
  int numObject = 0;
  int numTrashbinObject = 0;

  // we manage :
  // 1 the heads
  // 2 the hands
  // 3 the container
  // 4 the object
  // 5 the table points
  
  while(((type != SPARK_DIST_AGENT_HEAD) || (index < numHeads)) && (panTiltIndex < SDI_F->allPanTiltInfos.numMax)){

    if(index == 0)
      reinitIndex = false;

    // manage hands
    if( type == SPARK_DIST_AGENT_HAND){
      if(index < numHands){
	recomputePanTilt = true;
	entityIndex = SDI_F->distancesInfo.hands[index].entityIndex;
	mainEntityIndex =  SDI_F->distancesInfo.hands[index].mainEntityIndex;
      }
      // switch to table point management
      else{
	type = SPARK_DIST_SUPPORT_POINT;
	reinitIndex = true;
      }
    }

    // manage table points
    else if(type == SPARK_DIST_SUPPORT_POINT){
      if(index < numTablePoints){
      ///recompute only if table has moved or order has changed.
      recomputePanTilt = someTablesHaveMoved;
      entityIndex = -1;
      mainEntityIndex =  SDI_F->distancesInfo.tablePoints[index].mainEntityIndex;
      }
      // switch to object management
      else{
	type = SPARK_DIST_SIMPLE_OBJECT;
	reinitIndex = true;
	SDI_F->allPanTiltInfos.curNumPanTiltPositionsStates = panTiltIndex; 
      }
    }

    // manage simple objects
    else if(type == SPARK_DIST_SIMPLE_OBJECT){
      if(index < numObjects){
      recomputePanTilt = true;
      entityIndex = SDI_F->distancesInfo.objects[index].entityIndex;
      mainEntityIndex =  -1;
      }
      // switch to heads
      else{
	type = SPARK_DIST_AGENT_HEAD;
	reinitIndex = true;
      }
    }
    //AAAA
    // manage heads
    else if( type == SPARK_DIST_AGENT_HEAD ){
      if(index < numHeads){
      recomputePanTilt = true;
      entityIndex = SDI_F->distancesInfo.heads[index].entityIndex;
      mainEntityIndex =  SDI_F->distancesInfo.heads[index].mainEntityIndex;
      }
    }
    
    /// 
    if(!reinitIndex){
      curLookAtTargetPanTiltStruct = &SDI_F->allPanTiltInfos.allLookAtPanTilt[panTiltIndex];    

      /// container is considered as an object in spark distance management 
      /// but is exported as a container for  pan tilt info.

      ///
      /// do we need to put pan tilt id? 
      ///

      if(type == SPARK_DIST_SIMPLE_OBJECT){
	numObject++;
	if(gEntities->entities[entityIndex]->subtype == HRI_OBJECT_CONTAINER){
	  type = SPARK_DIST_CONTAINER;
	  SDI_F->allPanTiltInfos.curNumPanTiltPositionsStates++; /// then it is a position state.
	  numTrashbinObject++;
	  if(numTrashbinObject < numObject){
	    printf("Distance Management : trashbin/container objects should be introduced before other objects for pgmHri-genom\n");
	  }
	}
      }
      
      if( (curLookAtTargetPanTiltStruct->distanceInfoIndex != index) || (curLookAtTargetPanTiltStruct->type != type ) || (curLookAtTargetPanTiltStruct->entityIndex != entityIndex) ||  (curLookAtTargetPanTiltStruct->mainEntityIndex != mainEntityIndex) ){      
	curLookAtTargetPanTiltStruct->distanceInfoIndex = index;
	curLookAtTargetPanTiltStruct->type = type;
	curLookAtTargetPanTiltStruct->entityIndex = entityIndex;
	curLookAtTargetPanTiltStruct->mainEntityIndex = mainEntityIndex;
      }

      ///
      /// Go back to distance info position state.
      if(type == SPARK_DIST_CONTAINER)
	type = SPARK_DIST_SIMPLE_OBJECT;
      
      if(recomputePanTilt){
	if(getTargetPanTiltTargetPosition(index,type,targetPosition))
	  getTargetPanTiltToLookAtPosition(robotEyesCenter,targetPosition,robotBaseYaw,curLookAtTargetPanTiltStruct->pan,curLookAtTargetPanTiltStruct->tilt);
	else 
	  printf("Impossible to compute pan tilt because position couldn't be retrieved \n");
      }      
      panTiltIndex++;
      index++;
    }
    /// We reinit index to zero and do not increment panTiltIndex
    else
	index = 0;
  }
  
  // total num of index.
  SDI_F->allPanTiltInfos.curNumPanTilt = panTiltIndex;
}


///
/// Get closest "position state position" : agent end effector / container / tablepoint from the entity position. 
///

void getObjectPositionFourClosestPositionStates(double x,double y,double z, SPARK_DIST_ELEM_TYPE & positionStatePosition1Type, int & positionStatePosition1Index, double & distance1, SPARK_DIST_ELEM_TYPE & positionStatePosition2Type, int & positionStatePosition2Index, double & distance2, SPARK_DIST_ELEM_TYPE & positionStatePosition3Type, int & positionStatePosition3Index, double & distance3, SPARK_DIST_ELEM_TYPE & positionStatePosition4Type, int & positionStatePosition4Index, double & distance4,SPARK_DIST_ELEM_TYPE & containerType, int & containerObjectIndex, double & containerDistance){
  
  int numPositionStates = SDI_F->allPanTiltInfos.curNumPanTiltPositionsStates;
  double dist;
  ///SPARK_3D_COORD perceivedPosition;
  SPARK_3D_COORD positionStatePosition;
  //int closestLevel = 10;
  int index;
  SPARK_DIST_ELEM_TYPE type;
  ///createVec(x,y,z,perceivedPosition);

  positionStatePosition1Index = -1;
  positionStatePosition2Index = -1;  
  positionStatePosition3Index = -1;
  positionStatePosition4Index = -1;  

  for(int i = 0; i< numPositionStates; i++){
    type = SDI_F->allPanTiltInfos.allLookAtPanTilt[i].type;
    index = SDI_F->allPanTiltInfos.allLookAtPanTilt[i].distanceInfoIndex;
    if(getTargetPanTiltTargetPosition(index,type,positionStatePosition)){
      dist = D3D(x,y,z,positionStatePosition.x,positionStatePosition.y,positionStatePosition.z);

      
      //
      // update the four closest neighboors
      // right now the computation is not updated at all.
      // For table point we could find directly the closest points.
      //
      updateFourClosestNeighboor(type,index,dist,positionStatePosition1Type,positionStatePosition1Index,distance1,positionStatePosition2Type,positionStatePosition2Index,distance2,positionStatePosition3Type,positionStatePosition3Index,distance3,positionStatePosition4Type,positionStatePosition4Index,distance4,containerType,containerObjectIndex,containerDistance);
    }
  }
}


///
/// Get cur pan and tilt of robot and the closest "position state position" : agent end effector / container / tablepoint 
/// closest according to angle from lookat direction. 
///

bool getAgentPanTiltAndFourMostLookedAtPositionStates(double & pan,double & tilt, SPARK_DIST_ELEM_TYPE & positionStatePosition1Type, int & positionStatePosition1Index, double & distance1, SPARK_DIST_ELEM_TYPE & positionStatePosition2Type, int & positionStatePosition2Index, double & distance2, SPARK_DIST_ELEM_TYPE & positionStatePosition3Type, int & positionStatePosition3Index, double & distance3, SPARK_DIST_ELEM_TYPE & positionStatePosition4Type, int & positionStatePosition4Index, double & distance4,SPARK_DIST_ELEM_TYPE & containerType, int & containerObjectIndex, double & containerDistance){
  SPARK_3D_COORD robotEyesCenter;
  SPARK_3D_COORD robotLookAtUnitaryVector;
  double robotBaseYaw;
  bool someTablesHaveMoved = true; /// we should check wether some tables have moved.

  //int closestLevel = 10;
  int index;
  SPARK_DIST_ELEM_TYPE type;
  double dist;
  int numPositionStates;
  double robotLookAtPosStatePan;
  double robotLookAtPosStateTilt;

  if(mySelfRobotAgentIndexInDistancesInfo != -1){
    getLookAtPanTilt(mySelfRobotAgentIndexInDistancesInfo,pan,tilt,robotEyesCenter,robotLookAtUnitaryVector,robotBaseYaw);
    computeRobotPanTiltToLookAtSignificantPositions(robotEyesCenter,robotBaseYaw,someTablesHaveMoved);

    /// find the four closest pan tilt.
    numPositionStates = SDI_F->allPanTiltInfos.curNumPanTiltPositionsStates;
    positionStatePosition1Index = -1;
    positionStatePosition2Index = -1;  
    positionStatePosition3Index = -1;
    positionStatePosition4Index = -1;  

    for(int i = 0; i< numPositionStates; i++){
      robotLookAtPosStatePan = SDI_F->allPanTiltInfos.allLookAtPanTilt[i].pan;
      robotLookAtPosStateTilt = SDI_F->allPanTiltInfos.allLookAtPanTilt[i].tilt;

      dist = D2D(pan,tilt,robotLookAtPosStatePan,robotLookAtPosStateTilt);

      type = SDI_F->allPanTiltInfos.allLookAtPanTilt[i].type;
      index = SDI_F->allPanTiltInfos.allLookAtPanTilt[i].distanceInfoIndex;
      
      //
      // update the four closest neighboors
      // right now the computation is not optimized at all.
      // For table point we could find directly the closest points.
      //
      updateFourClosestNeighboor(type,index,dist,positionStatePosition1Type,positionStatePosition1Index,distance1,positionStatePosition2Type,positionStatePosition2Index,distance2,positionStatePosition3Type,positionStatePosition3Index,distance3,positionStatePosition4Type,positionStatePosition4Index,distance4,containerType,containerObjectIndex,containerDistance);
    }
    return true;
  }
  else{
    printf("Unknown distances info index for perceiving robot. Impossible to get perceiving agent pan and tilt\n");
    return false;
  }
}


void computeDistancesAndFillDistanceManagementPoster2( void ){
  int numAgents = SDI_F->distancesInfo.numAgents;
  int numObjects = SDI_F->distancesInfo.numObjects;
  int numTables = SDI_F->distancesInfo.numTables;
  int numHands = SDI_F->distancesInfo.numHands;
  int numHeads = SDI_F->distancesInfo.numHeads;
  //int numTablePoints = SDI_F->distancesInfo.numTablePoints;
  double distThreshold = SDI_F->distancesInfo.distThreshold;
  //double distThresholdSmall = SDI_F->distancesInfo.distThresholdSmall;
  int saveHistory = SDI_F->distancesInfo.saveHistory;
  double saveHistoryDistanceThreshold = SDI_F->distancesInfo.saveHistoryDistanceThreshold;

  int entityIndex,agentEntityIndex;
  HRI_ENTITY * entity;
  p3d_rob* rob;
  double yaw;
  double pitch;

  SPARK_3D_COORD aPoint;
  int anInt;
  double aDist;
  int forceReinitMinDist = SDI_F->distancesInfo.forceReinitMinDist;
  double aTime = 0.0;
  int checkArmVectorLengthMax = SDI_F->distancesInfo.checkArmVectorLengthMax;
  int reinitArmVectorLengthMax = SDI_F->distancesInfo.reinitArmVectorLengthMax;

  //////////////////////
  //  update positions
  //////////////////////

  ///update agent positions and orientation.
  for(int i = 0; i< numAgents; i++){
    entityIndex = SDI_F->distancesInfo.agents[i].entityIndex;
    entity = gEntities->entities[entityIndex];
    //getEntityBBCenter(entity,false,SDI_F->distancesInfo.agents.position); 
    rob = entity->robotPt;
    //printf("rob name : %s %s\n",entity->robotPt->name,SDI_F->distancesInfo.agents[i].baseJointName.name);	 
    getJointXYZYaw ( rob, SDI_F->distancesInfo.agents[i].baseJointName ,SDI_F->distancesInfo.agents[i].position, yaw,pitch);
    getOrientationVector(yaw,SDI_F->distancesInfo.agents[i].orientation);
  }

  ///update object positions
  for(int i = 0; i< numObjects; i++){
    entityIndex = SDI_F->distancesInfo.objects[i].entityIndex;
    entity = gEntities->entities[entityIndex];
    getEntityBBCenter(entity,false,SDI_F->distancesInfo.objects[i].position); 
  }

  ///check table center position.
  for(int i = 0; i< numTables; i++){
    entityIndex = SDI_F->distancesInfo.tables[i].entityIndex;
    entity = gEntities->entities[entityIndex];

    //Check wether table has moved to test wether we should update corner and table points.
    getEntityBBCenter(entity,false,aPoint);
    aDist = getDistanceBetweenPoints(aPoint,SDI_F->distancesInfo.tables[i].position);
    if( (aDist > 0.2) || SDI_F->distancesInfo.forceUpdateTablesPosition){
      copyVec(aPoint,SDI_F->distancesInfo.tables[i].position);
      if(needToUpdateTableCorners(entityIndex,SDI_F->distancesInfo.tables[i].tableCornersPositions,0.4)|| SDI_F->distancesInfo.forceUpdateTablesPosition){
	deleteTableEntityTablePoints(entityIndex,SDI_F->distancesInfo.tables[i].numTablePoints,SDI_F->distancesInfo.tables[i].firstTablePointIndex);
	UpdateTablePointsInDistanceManagement(entityIndex,i);
      }    
      SDI_F->distancesInfo.forceUpdateTablesPosition = FALSE;
    }
  }  
  
  // update hands and arms
  for(int i = 0; i< numHands; i++){
    entityIndex = SDI_F->distancesInfo.hands[i].entityIndex;
    entity = gEntities->entities[entityIndex];
    getEntityBBCenter(entity,false,SDI_F->distancesInfo.hands[i].position); 

    agentEntityIndex = SDI_F->distancesInfo.arms[i].agentEntityIndex;

    //update shoulder positions
    entityIndex = SDI_F->distancesInfo.arms[i].agentEntityIndex;
    entity = gEntities->entities[entityIndex];
    rob = entity->robotPt;
    getJointXYZYaw ( rob, SDI_F->distancesInfo.arms[i].shoulderJointName ,SDI_F->distancesInfo.arms[i].shoulderPosition, yaw,pitch);

    //update arm direction
    if(SDI_F->distancesInfo.agents[agentEntityIndex].allowPointingComputation || SDI_F->distancesInfo.agents[agentEntityIndex].allowGestureComputation){
      getVecABFromPointAAndPointB(SDI_F->distancesInfo.arms[i].shoulderPosition,SDI_F->distancesInfo.hands[i].position,SDI_F->distancesInfo.arms[i].armVector);
      SDI_F->distancesInfo.arms[i].armVectorLength = getVecDistance(SDI_F->distancesInfo.arms[i].armVector);

      if(reinitArmVectorLengthMax || (checkArmVectorLengthMax && (SDI_F->distancesInfo.arms[i].armVectorLengthMax > 2))){
	SDI_F->distancesInfo.arms[i].armVectorLengthMax = 0;
      }
      if(SDI_F->distancesInfo.arms[i].armVectorLength > SDI_F->distancesInfo.arms[i].armVectorLengthMax)
	SDI_F->distancesInfo.arms[i].armVectorLengthMax = SDI_F->distancesInfo.arms[i].armVectorLength;

      if(SDI_F->distancesInfo.agents[agentEntityIndex].allowGestureComputation)
	updateArmSemaphoreAlphabet(SDI_F->distancesInfo.agents[agentEntityIndex].orientation,SDI_F->distancesInfo.arms[i].armVector,SDI_F->distancesInfo.arms[i].armVectorLength,SDI_F->distancesInfo.arms[i].armVectorLengthMax,SDI_F->distancesInfo.arms[i].armAngleInfo);
    }
  }
  
  SDI_F->distancesInfo.checkArmVectorLengthMax = FALSE;
  SDI_F->distancesInfo.reinitArmVectorLengthMax = FALSE;

  // update heads position
  for(int i = 0; i< numHeads; i++){
    entityIndex = SDI_F->distancesInfo.heads[i].entityIndex;
    entity = gEntities->entities[entityIndex];
    getEntityBBCenter(entity,false,SDI_F->distancesInfo.heads[i].position); 

    // update look at direction if needed
    //if(SDI_F->distancesInfo.agents[agentEntityIndex].allowLookAtComputation){
    //   getVecABFromPointAAndPointB(SDI_F->distancesInfo.arms[i].shoulderPosition,SDI_F->distancesInfo.hands[i].position,SDI_F->distancesInfo.arms[i].armVector);
    //   SDI_F->distancesInfo.arms[i].armVectorLength = getVecDistance(SDI_F->distancesInfo.arms[i].armVector);

    //   if(reinitArmVectorLengthMax || (checkArmVectorLengthMax && (SDI_F->distancesInfo.arms[i].armVectorLengthMax > 2))){
    // 	SDI_F->distancesInfo.arms[i].armVectorLengthMax = 0;
    //   }
    //   if(SDI_F->distancesInfo.arms[i].armVectorLength > SDI_F->distancesInfo.arms[i].armVectorLengthMax)
    // 	SDI_F->distancesInfo.arms[i].armVectorLengthMax = SDI_F->distancesInfo.arms[i].armVectorLength;
    // }
    //update look at direction if needed
    
    ///getLookAtPanTilt(i,yaw,pitch,aPoint,aPoint,aDist);
  }
  
  // all positions are older than this time
  aTime = getTimeInSecondMilisecond()-SDI_F->distancesInfo.initialTime;

  //////////////////////
  //  update distances and angles
  //////////////////////

  ///////////////////////////////
  // agent / agent distances.
  ///////////////////////////////

  int someIterAgentAgent = 0;
  int someIterAgentHand = 0;
  int someIterHandHand = 0;  
  int someIterAgentAgentAngle = 0; 
  int someIterAgentHeadPointing = 0; 

  int firstHandIndex = 0;
  int firstHandIndex2 = 0;
  int headIndex = 0;
  int headIndex2 = 0;
  numHands = 0;
  int numHands2 = 0;

  for(int i = 0; i< numAgents; i++){
    for(int j = i+1; j< numAgents; j++){
      if(someIterAgentAgent < SDI_F->distancesInfo.numAgentAgentDistsMax){
	updateDistAndMinDist(i,j,SDI_F->distancesInfo.agents[i].position,SDI_F->distancesInfo.agents[j].position,forceReinitMinDist, aTime,SDI_F->distancesInfo.agentAgentDists[someIterAgentAgent].index1,SDI_F->distancesInfo.agentAgentDists[someIterAgentAgent].index2,SDI_F->distancesInfo.agentAgentDists[someIterAgentAgent].curDist,SDI_F->distancesInfo.agentAgentDists[someIterAgentAgent].minDist, SDI_F->distancesInfo.agentAgentDists[someIterAgentAgent].minDistTime);

       
	////////////////////////////////////////
	//allow to compute distance involving these agent hands.
	/////////////////////////////////////////
	if(SDI_F->distancesInfo.agentAgentDists[someIterAgentAgent].curDist < distThreshold){
	  firstHandIndex = SDI_F->distancesInfo.agents[i].firstHandIndex;
	  firstHandIndex2 = SDI_F->distancesInfo.agents[j].firstHandIndex;
	  headIndex = SDI_F->distancesInfo.agents[i].headIndex;
	  headIndex2 = SDI_F->distancesInfo.agents[j].headIndex;
	  numHands = SDI_F->distancesInfo.agents[i].numHands;
	  numHands2 = SDI_F->distancesInfo.agents[j].numHands;

	  SDI_F->distancesInfo.agentAgentDists[someIterAgentAgent].firstHandHandIndex = someIterHandHand;
	  SDI_F->distancesInfo.agentAgentDists[someIterAgentAgent].firstAgentHandIndex = someIterAgentHand;
	 
	  ///////////////////
	  //  betwee nhands
	  ///////////////////
	  for(int k = firstHandIndex; k< firstHandIndex + numHands; k++){
	    for(int l = firstHandIndex2; l< firstHandIndex2 + numHands2; l++){
	      if(someIterHandHand < SDI_F->distancesInfo.numHandHandDistsMax){
		updateDistAndMinDist(k,l,SDI_F->distancesInfo.hands[k].position,SDI_F->distancesInfo.hands[l].position,forceReinitMinDist, aTime,SDI_F->distancesInfo.handHandDists[someIterHandHand].index1,SDI_F->distancesInfo.handHandDists[someIterHandHand].index2,SDI_F->distancesInfo.handHandDists[someIterHandHand].curDist,SDI_F->distancesInfo.handHandDists[someIterHandHand].minDist,SDI_F->distancesInfo.handHandDists[someIterHandHand].minDistTime);
		someIterHandHand++;
	      }
	      else
		printf("Too many hand hand dists. Max is %d\n",SDI_F->distancesInfo.numHandHandDistsMax);	    
	    }
	  }
	  SDI_F->distancesInfo.numHandHandDists = someIterHandHand;
	 
	  /////////////
	  // between hand and agent
	  ///////////

	  for(int k = firstHandIndex; k< firstHandIndex + numHands; k++){
	    if(someIterAgentHand < SDI_F->distancesInfo.numAgentHandDistsMax){
	      updateDistAndMinDist(j,k,SDI_F->distancesInfo.hands[k].position,SDI_F->distancesInfo.agents[j].position,forceReinitMinDist, aTime,SDI_F->distancesInfo.agentHandDists[someIterAgentHand].index1,SDI_F->distancesInfo.agentHandDists[someIterAgentHand].index2,SDI_F->distancesInfo.agentHandDists[someIterAgentHand].curDist,SDI_F->distancesInfo.agentHandDists[someIterAgentHand].minDist,SDI_F->distancesInfo.agentHandDists[someIterAgentHand].minDistTime);
	      someIterAgentHand++;
	    }
	    else
	      printf("Too many agent hand dists. Max is %d\n",SDI_F->distancesInfo.numAgentHandDistsMax);
	  }
	  SDI_F->distancesInfo.numAgentHandDists = someIterAgentHand;
	 
	  for(int l = firstHandIndex2; l< firstHandIndex2 + numHands2; l++){
	    if(someIterAgentHand < SDI_F->distancesInfo.numAgentHandDistsMax){	       
	      updateDistAndMinDist(i,l,SDI_F->distancesInfo.agents[i].position,SDI_F->distancesInfo.hands[l].position,forceReinitMinDist, aTime,SDI_F->distancesInfo.agentHandDists[someIterAgentHand].index1,SDI_F->distancesInfo.agentHandDists[someIterAgentHand].index2,SDI_F->distancesInfo.agentHandDists[someIterAgentHand].curDist,SDI_F->distancesInfo.agentHandDists[someIterAgentHand].minDist,SDI_F->distancesInfo.agentHandDists[someIterAgentHand].minDistTime);
	      someIterAgentHand ++;
	    }
	    else
	      printf("Too many agent hand dists. Max is %d\n",SDI_F->distancesInfo.numAgentHandDistsMax);
	  }
	  SDI_F->distancesInfo.numAgentHandDists = someIterAgentHand;
	
	  SDI_F->distancesInfo.agentAgentDists[someIterAgentAgent].lastHandHandIndex = someIterHandHand-1;
	  SDI_F->distancesInfo.agentAgentDists[someIterAgentAgent].lastAgentHandIndex = someIterAgentHand-1;

	 
	  /////////////
	  // Angle between agents
	  //////////////////

	  SDI_F->distancesInfo.agentAgentDists[someIterAgentAgent].firstAgentAgentAngleIndex = -1;
	  if(SDI_F->distancesInfo.agents[i].allowAngleComputation){
	    if(someIterAgentAgentAngle < SDI_F->distancesInfo.numAgentAgentAnglesMax){
	      SDI_F->distancesInfo.agentAgentDists[someIterAgentAgent].firstAgentAgentAngleIndex = someIterAgentAgentAngle;
	      updateAngleNew(i,j,SDI_F->distancesInfo.agents[i].orientation,SDI_F->distancesInfo.agents[i].position,SDI_F->distancesInfo.agents[j].position,SDI_F->distancesInfo.agentAgentAngles[someIterAgentAgentAngle].index1,SDI_F->distancesInfo.agentAgentAngles[someIterAgentAgentAngle].index2,SDI_F->distancesInfo.agentAgentAngles[someIterAgentAgentAngle].angle);
	      someIterAgentAgentAngle++;
	    }
	    else
	      printf("Too many agent agent angle. Max is %d\n",SDI_F->distancesInfo.numAgentAgentAnglesMax);
	  }
	  if(SDI_F->distancesInfo.agents[j].allowAngleComputation){	   
	    if(someIterAgentAgentAngle < SDI_F->distancesInfo.numAgentAgentAnglesMax){
	      if(SDI_F->distancesInfo.agentAgentDists[someIterAgentAgent].firstAgentAgentAngleIndex == -1)
		SDI_F->distancesInfo.agentAgentDists[someIterAgentAgent].firstAgentAgentAngleIndex = someIterAgentAgentAngle;
	      updateAngleNew(j,i,SDI_F->distancesInfo.agents[j].orientation,SDI_F->distancesInfo.agents[j].position,SDI_F->distancesInfo.agents[i].position,SDI_F->distancesInfo.agentAgentAngles[someIterAgentAgentAngle].index1,SDI_F->distancesInfo.agentAgentAngles[someIterAgentAgentAngle].index2,SDI_F->distancesInfo.agentAgentAngles[someIterAgentAgentAngle].angle);
	      someIterAgentAgentAngle++;
	    }
	    else
	      printf("Too many agent agent angle. Max is %d\n",SDI_F->distancesInfo.numAgentAgentAnglesMax);
	  }
	  
	  SDI_F->distancesInfo.numAgentAgentAngles = someIterAgentAgentAngle;
	  SDI_F->distancesInfo.agentAgentDists[someIterAgentAgent].lastAgentAgentAngleIndex = someIterAgentAgentAngle-1;
	
	  ////////////////////////////
	  // pointing betzeen agents and agent head
	  ///////////////////////////////
	  // head index is agent index : only one head!
	  // arm index same as hand index!

	  SDI_F->distancesInfo.agentAgentDists[someIterAgentAgent].firstAgentHeadPointingIndex = -1;
	  if(SDI_F->distancesInfo.agents[i].allowPointingComputation){
	    for(int k = firstHandIndex; k< firstHandIndex + numHands; k++){
	      if(someIterAgentHeadPointing < SDI_F->distancesInfo.numAgentArmAgentPointingMax){
		if(SDI_F->distancesInfo.agentAgentDists[someIterAgentAgent].firstAgentHeadPointingIndex == -1)
		  SDI_F->distancesInfo.agentAgentDists[someIterAgentAgent].firstAgentHeadPointingIndex = someIterAgentHeadPointing;
		updatePointing( k, headIndex2, SDI_F->distancesInfo.arms[k].armVector, SDI_F->distancesInfo.arms[k].shoulderPosition,SDI_F->distancesInfo.heads[headIndex2].position,SDI_F->distancesInfo.arms[k].armVectorLength,SDI_F->distancesInfo.arms[k].armVectorLengthMax,SDI_F->distancesInfo.agentArmAgentPointing[someIterAgentHeadPointing].agentIndex,SDI_F->distancesInfo.agentArmAgentPointing[someIterAgentHeadPointing].index,SDI_F->distancesInfo.agentArmAgentPointing[someIterAgentHeadPointing].optimalPointingVec , SDI_F->distancesInfo.agentArmAgentPointing[someIterAgentHeadPointing].optimalPointingPoint,  SDI_F->distancesInfo.agentArmAgentPointing[someIterAgentHeadPointing].curAngleFromOptimalPointingVec,  SDI_F->distancesInfo.agentArmAgentPointing[someIterAgentHeadPointing].pointingScore);
		someIterAgentHeadPointing++;
	      }
	      else
		printf("Too many agent agent pointing. Max is %d\n",SDI_F->distancesInfo.numAgentArmAgentPointingMax);
	    }
	  }

	  if(SDI_F->distancesInfo.agents[j].allowPointingComputation){
	    for(int l = firstHandIndex2; l< firstHandIndex2 + numHands2; l++){
	      if(someIterAgentHeadPointing < SDI_F->distancesInfo.numAgentArmAgentPointingMax){
		if(SDI_F->distancesInfo.agentAgentDists[someIterAgentAgent].firstAgentHeadPointingIndex == -1)
		  SDI_F->distancesInfo.agentAgentDists[someIterAgentAgent].firstAgentHeadPointingIndex = someIterAgentHeadPointing;
		updatePointing( l, headIndex, SDI_F->distancesInfo.arms[l].armVector, SDI_F->distancesInfo.arms[l].shoulderPosition,SDI_F->distancesInfo.heads[headIndex].position,SDI_F->distancesInfo.arms[l].armVectorLength,SDI_F->distancesInfo.arms[l].armVectorLengthMax,SDI_F->distancesInfo.agentArmAgentPointing[someIterAgentHeadPointing].agentIndex,SDI_F->distancesInfo.agentArmAgentPointing[someIterAgentHeadPointing].index,SDI_F->distancesInfo.agentArmAgentPointing[someIterAgentHeadPointing].optimalPointingVec , SDI_F->distancesInfo.agentArmAgentPointing[someIterAgentHeadPointing].optimalPointingPoint,  SDI_F->distancesInfo.agentArmAgentPointing[someIterAgentHeadPointing].curAngleFromOptimalPointingVec,  SDI_F->distancesInfo.agentArmAgentPointing[someIterAgentHeadPointing].pointingScore);
		someIterAgentHeadPointing++;
	      }
	      else
		printf("Too many agent agent pointing. Max is %d\n",SDI_F->distancesInfo.numAgentArmAgentPointingMax);	 
	    }
	  }
	  SDI_F->distancesInfo.numAgentArmAgentPointing = someIterAgentHeadPointing;
	  SDI_F->distancesInfo.agentAgentDists[someIterAgentAgent].lastAgentHeadPointingIndex = someIterAgentHeadPointing-1;
	}
	else{
	  SDI_F->distancesInfo.agentAgentDists[someIterAgentAgent].firstHandHandIndex = -1;
	  SDI_F->distancesInfo.agentAgentDists[someIterAgentAgent].firstAgentHandIndex = -1;
	  SDI_F->distancesInfo.agentAgentDists[someIterAgentAgent].firstAgentAgentAngleIndex = -1;
	  SDI_F->distancesInfo.agentAgentDists[someIterAgentAgent].firstAgentHeadPointingIndex = -1;
	}	
	someIterAgentAgent++;	       
      }
      else
	printf("Too many agent agent distance. Max is %d\n",SDI_F->distancesInfo.numAgentAgentDistsMax);
    }
  }
  SDI_F->distancesInfo.numAgentAgentDists = someIterAgentAgent;

  ///////////////////////////////
  // agent /  Object distances.
  ///////////////////////////////

  int someIterAgentObject = 0;
  int someIterHandObject = 0; 
  int someIterAgentObjectAngle = 0;
  int someIterAgentObjectPointing = 0;
 
  for(int i = 0; i< numAgents; i++){
    for(int j = 0; j< numObjects; j++){
      if(someIterAgentObject < SDI_F->distancesInfo.numAgentObjectDistsMax){
	updateDistAndMinDist(i,j,SDI_F->distancesInfo.agents[i].position,SDI_F->distancesInfo.objects[j].position,forceReinitMinDist, aTime,SDI_F->distancesInfo.agentObjectDists[someIterAgentObject].index1,SDI_F->distancesInfo.agentObjectDists[someIterAgentObject].index2,SDI_F->distancesInfo.agentObjectDists[someIterAgentObject].curDist,SDI_F->distancesInfo.agentObjectDists[someIterAgentObject].minDist, SDI_F->distancesInfo.agentObjectDists[someIterAgentObject].minDistTime);

	////////////////////////////////////////
	//allow to compute distance involving these agent hand and object
	/////////////////////////////////////////
	if(SDI_F->distancesInfo.agentObjectDists[someIterAgentObject].curDist < distThreshold){
	  firstHandIndex = SDI_F->distancesInfo.agents[i].firstHandIndex;
	  numHands = SDI_F->distancesInfo.agents[i].numHands;
	  SDI_F->distancesInfo.agentObjectDists[someIterAgentObject].firstHandObjectIndex = someIterHandObject;	 

	  for(int k = firstHandIndex; k< firstHandIndex + numHands; k++){
	    if(someIterHandObject < SDI_F->distancesInfo.numHandObjectDistsMax){
	      updateDistAndMinDist(k,j,SDI_F->distancesInfo.hands[k].position,SDI_F->distancesInfo.objects[j].position,forceReinitMinDist, aTime,SDI_F->distancesInfo.handObjectDists[someIterHandObject].index1,SDI_F->distancesInfo.handObjectDists[someIterHandObject].index2,SDI_F->distancesInfo.handObjectDists[someIterHandObject].curDist,SDI_F->distancesInfo.handObjectDists[someIterHandObject].minDist,SDI_F->distancesInfo.handObjectDists[someIterHandObject].minDistTime);
	      someIterHandObject++;
	    }
	    else 
	      printf("Too many hand object distance. Max is %d\n",SDI_F->distancesInfo.numHandObjectDistsMax);
	  }

	  SDI_F->distancesInfo.numHandObjectDists = someIterHandObject;
	  SDI_F->distancesInfo.agentObjectDists[someIterAgentObject].lastHandObjectIndex = someIterHandObject-1;


	  /////////////
	  // Angle between agent and object
	  //////////////////
	  SDI_F->distancesInfo.agentObjectDists[someIterAgentObject].agentObjectAngleIndex = -1;

	  if(SDI_F->distancesInfo.agents[i].allowAngleComputation){
	    if(someIterAgentObjectAngle < SDI_F->distancesInfo.numAgentObjectAnglesMax){
	      SDI_F->distancesInfo.agentObjectDists[someIterAgentObject].agentObjectAngleIndex = someIterAgentObjectAngle;
	      updateAngleNew(i,j,SDI_F->distancesInfo.agents[i].orientation,SDI_F->distancesInfo.agents[i].position,SDI_F->distancesInfo.objects[j].position,SDI_F->distancesInfo.agentObjectAngles[someIterAgentObjectAngle].index1,SDI_F->distancesInfo.agentObjectAngles[someIterAgentObjectAngle].index2,SDI_F->distancesInfo.agentObjectAngles[someIterAgentObjectAngle].angle);
	      someIterAgentObjectAngle++;
	    }
	    else
	      printf("Too many AgentObjectAngles. Max is %d\n",SDI_F->distancesInfo.numAgentObjectAnglesMax);
	  }
	  SDI_F->distancesInfo.numAgentObjectAngles = someIterAgentObjectAngle;

	  /////////////
	  // pointing between agent and object
	  //////////////////
	  SDI_F->distancesInfo.agentObjectDists[someIterAgentObject].firstAgentObjectPointingIndex = -1;
	  if(SDI_F->distancesInfo.agents[i].allowPointingComputation){
	    for(int k = firstHandIndex; k< firstHandIndex + numHands; k++){
	      if(someIterAgentObjectPointing < SDI_F->distancesInfo.numAgentArmObjectPointingMax){
		if(SDI_F->distancesInfo.agentObjectDists[someIterAgentObject].firstAgentObjectPointingIndex == -1)
		  SDI_F->distancesInfo.agentObjectDists[someIterAgentObject].firstAgentObjectPointingIndex = someIterAgentObjectPointing;
		updatePointing( k, j, SDI_F->distancesInfo.arms[k].armVector, SDI_F->distancesInfo.arms[k].shoulderPosition,SDI_F->distancesInfo.objects[j].position,SDI_F->distancesInfo.arms[k].armVectorLength,SDI_F->distancesInfo.arms[k].armVectorLengthMax,SDI_F->distancesInfo.agentArmObjectPointing[someIterAgentObjectPointing].agentIndex,SDI_F->distancesInfo.agentArmObjectPointing[someIterAgentObjectPointing].index,SDI_F->distancesInfo.agentArmObjectPointing[someIterAgentObjectPointing].optimalPointingVec , SDI_F->distancesInfo.agentArmObjectPointing[someIterAgentObjectPointing].optimalPointingPoint,  SDI_F->distancesInfo.agentArmObjectPointing[someIterAgentObjectPointing].curAngleFromOptimalPointingVec,  SDI_F->distancesInfo.agentArmObjectPointing[someIterAgentObjectPointing].pointingScore);	
		someIterAgentObjectPointing++;
	      }
	      else
		printf("Too many AgentArmObjectPointing. Max is %d\n",SDI_F->distancesInfo.numAgentArmObjectPointingMax);
	    }
	  }

	  SDI_F->distancesInfo.numAgentArmObjectPointing = someIterAgentObjectPointing;
	  SDI_F->distancesInfo.agentObjectDists[someIterAgentObject].lastAgentObjectPointingIndex = someIterAgentObjectPointing-1;
		      
	}
	else{
	  SDI_F->distancesInfo.agentObjectDists[someIterAgentObject].firstHandObjectIndex = -1;
	  SDI_F->distancesInfo.agentObjectDists[someIterAgentObject].agentObjectAngleIndex = -1;
	  SDI_F->distancesInfo.agentObjectDists[someIterAgentObject].firstAgentObjectPointingIndex = -1;
	}
      
	someIterAgentObject++;	 
      }
      else 
	printf("Too many agent object distance. Max is %d\n",SDI_F->distancesInfo.numAgentObjectDistsMax);
    }
  }
  SDI_F->distancesInfo.numAgentObjectDists = someIterAgentObject;

  ///////////////////////////////
  // agent /  Table distances.
  ///////////////////////////////

  int someIterAgentTable = 0;
  int someIterHandTable = 0;
  //int someIterHandTablePoint = 0;
  int someIterAgentTablePoint = 0; 
  int someIterAgentTablePointAngle = 0;  
  int someIterAgentTablePointPointing = 0;
  int someIterAgentTablePointingIntersection = 0;
  int firstTablePointIndex = 0;
  int numTablePoints = 0;
  bool allowSaveHistoryForThisHand;
  for(int i = 0; i< numAgents; i++){
    for(int j = 0; j< numTables; j++){
      if(someIterAgentTable < SDI_F->distancesInfo.numAgentTableDistsMax){
	updateDistAndMinDistToTable(0,i,j,false,false,0,SDI_F->distancesInfo.agents[i].position,SDI_F->distancesInfo.tables[j].tableCornersPositions,forceReinitMinDist, aTime,SDI_F->distancesInfo.agentTableDists[someIterAgentTable].index1,SDI_F->distancesInfo.agentTableDists[someIterAgentTable].index2,SDI_F->distancesInfo.agentTableDists[someIterAgentTable].curDist,SDI_F->distancesInfo.agentTableDists[someIterAgentTable].isAbove,SDI_F->distancesInfo.agentTableDists[someIterAgentTable].closestPoint,   SDI_F->distancesInfo.agentTableDists[someIterAgentTable].minDist, SDI_F->distancesInfo.agentTableDists[someIterAgentTable].minDistTime , SDI_F->distancesInfo.agentTableDists[someIterAgentTable].minDistPosition, anInt ,anInt,anInt);


	////////////////////////////////////////
	//allow to compute distance involving these agent hands and table.
	/////////////////////////////////////////
	if(SDI_F->distancesInfo.agentTableDists[someIterAgentTable].curDist < distThreshold){
	  firstHandIndex = SDI_F->distancesInfo.agents[i].firstHandIndex;
	  numHands = SDI_F->distancesInfo.agents[i].numHands;
	  firstTablePointIndex = SDI_F->distancesInfo.tables[i].firstTablePointIndex;
	  numTablePoints = SDI_F->distancesInfo.tables[i].numTablePoints;
	  SDI_F->distancesInfo.agentTableDists[someIterAgentTable].firstHandTableIndex = someIterHandTable;


	  /////////////
	  // between Agent  and table points if below small threshold
	  ///////////

	  //if it has not been yet initalized put it to the first value

	  SDI_F->distancesInfo.agentTableDists[someIterAgentTable].firstAgentTablePointIndex = someIterAgentTablePoint;
	  for(int l = firstTablePointIndex; l< firstTablePointIndex + numTablePoints; l++){
	    if(someIterAgentTablePoint < SDI_F->distancesInfo.numAgentTablePointDistsMax){
	      updateDistAndMinDist(i,l,SDI_F->distancesInfo.agents[i].position,SDI_F->distancesInfo.tablePoints[l].position,forceReinitMinDist, aTime,SDI_F->distancesInfo.agentTablePointDists[someIterAgentTablePoint].index1,SDI_F->distancesInfo.agentTablePointDists[someIterAgentTablePoint].index2,SDI_F->distancesInfo.agentTablePointDists[someIterAgentTablePoint].curDist,SDI_F->distancesInfo.agentTablePointDists[someIterAgentTablePoint].minDist,SDI_F->distancesInfo.agentTablePointDists[someIterAgentTablePoint].minDistTime);
	      someIterAgentTablePoint++;
	    }
	    else
	      printf("Too many TablePointDists. Max is %d\n",SDI_F->distancesInfo.numAgentTablePointDistsMax);
	  }

	  SDI_F->distancesInfo.numAgentTablePointDists = someIterAgentTablePoint;
	  SDI_F->distancesInfo.agentTableDists[someIterAgentTablePoint].lastAgentTablePointIndex = someIterAgentTablePoint-1;



	  //Initialize value
	  SDI_F->distancesInfo.agentTableDists[someIterAgentTable].firstHandTableIndex = -1;

	  for(int k = firstHandIndex; k< firstHandIndex + numHands; k++){
	    if(someIterHandTable < SDI_F->distancesInfo.numHandTableDistsMax){

	      //we save history for this hand only if the general saveHistory bool is true and we allow pointing computation for this arm.
	      allowSaveHistoryForThisHand = false;
	      if(SDI_F->distancesInfo.agents[i].allowPointingComputation)
		allowSaveHistoryForThisHand = saveHistory;

	      updateDistAndMinDistToTable(i,k,j,true,allowSaveHistoryForThisHand,saveHistoryDistanceThreshold,SDI_F->distancesInfo.hands[k].position,SDI_F->distancesInfo.tables[j].tableCornersPositions,forceReinitMinDist, aTime,SDI_F->distancesInfo.handTableDists[someIterHandTable].index1,SDI_F->distancesInfo.handTableDists[someIterHandTable].index2,SDI_F->distancesInfo.handTableDists[someIterHandTable].curDist,SDI_F->distancesInfo.handTableDists[someIterHandTable].isAbove,SDI_F->distancesInfo.handTableDists[someIterHandTable].closestPoint,SDI_F->distancesInfo.handTableDists[someIterHandTable].minDist,SDI_F->distancesInfo.handTableDists[someIterHandTable].minDistTime,SDI_F->distancesInfo.handTableDists[someIterHandTable].minDistPosition,SDI_F->distancesInfo.handTableDists[someIterHandTable].trajectoryIndex,SDI_F->distancesInfo.handTableDists[someIterHandTable].distanceHistoryIndex,SDI_F->distancesInfo.handTableDists[someIterHandTable].timeStepIndex);
	      someIterHandTable ++;
	    }
	    else
	      printf("Too many HandTableDists. Max is %d\n",SDI_F->distancesInfo.numHandTableDistsMax);	    
	  }

	  SDI_F->distancesInfo.numHandTableDists = someIterHandTable;
	  SDI_F->distancesInfo.agentTableDists[someIterAgentTable].lastHandTableIndex = someIterHandTable-1;

	  
	  /////////////
	  // Angle between agent and table points
	  //////////////////
	  SDI_F->distancesInfo.agentTableDists[someIterAgentTable].firstAgentTablePointAngleIndex = -1;
	  
	  if(SDI_F->distancesInfo.agents[i].allowAngleComputation){
	    SDI_F->distancesInfo.agentTableDists[someIterAgentTable].firstAgentTablePointAngleIndex = someIterAgentTablePointAngle;
		
	    for(int l = firstTablePointIndex; l< firstTablePointIndex + numTablePoints; l++){
	      if(someIterAgentTablePointAngle < SDI_F->distancesInfo.numAgentTablePointDistsMax){
		updateAngleNew(i,l,SDI_F->distancesInfo.agents[i].orientation,SDI_F->distancesInfo.agents[i].position,SDI_F->distancesInfo.tablePoints[l].position,SDI_F->distancesInfo.agentTablePointAngles[someIterAgentTablePointAngle].index1,SDI_F->distancesInfo.agentTablePointAngles[someIterAgentTablePointAngle].index2,SDI_F->distancesInfo.agentTablePointAngles[someIterAgentTablePointAngle].angle);
		someIterAgentTablePointAngle++;
	      }
	      else
		printf("Too many AgentTablePointDists. Max is %d\n",SDI_F->distancesInfo.numAgentTablePointDistsMax);
	    }
	    SDI_F->distancesInfo.numAgentTablePointDists = someIterAgentTablePointAngle;
	    SDI_F->distancesInfo.agentTableDists[someIterAgentTable].lastAgentTablePointAngleIndex = someIterAgentTablePointAngle-1;
	  }

	  ///////////////////////////////////////////
	  // Pointing between agent and table
	  ///////////////////////////////////////////////
	

	  SDI_F->distancesInfo.agentTableDists[someIterAgentTable].firstAgentTablePointPointingIndex = -1;
	  if(SDI_F->distancesInfo.agents[i].allowPointingComputation){	  
	    for(int k = firstHandIndex; k< firstHandIndex + numHands; k++){

	      bool isArmPointingTowardTableBool = isArmPointingTowardTable(0.2,SDI_F->distancesInfo.arms[k].shoulderPosition,SDI_F->distancesInfo.arms[k].armVector,SDI_F->distancesInfo.tables[j].tableCornersPositions,aPoint);
	      if(someIterAgentTablePointingIntersection < SDI_F->distancesInfo.numAgentArmTablePointingIntersectionMax){
		if(SDI_F->distancesInfo.agentTableDists[someIterAgentTable].firstAgentTablePointingIntersectionIndex == -1)
		  SDI_F->distancesInfo.agentTableDists[someIterAgentTable].firstAgentTablePointingIntersectionIndex = SDI_F->distancesInfo.numAgentArmTablePointingIntersection;
		SDI_F->distancesInfo.agentArmTablePointingIntersection[someIterAgentTablePointingIntersection].index1 = k;
		SDI_F->distancesInfo.agentArmTablePointingIntersection[someIterAgentTablePointingIntersection].index2 = j;
		copyVec(aPoint,SDI_F->distancesInfo.agentArmTablePointingIntersection[someIterAgentTablePointingIntersection].intersection);
		someIterAgentTablePointingIntersection++;
	      }
	      else
		printf("Too many AgentArmTablePointingIntersect. Max is %d\n",SDI_F->distancesInfo.numAgentArmTablePointingIntersectionMax);
	      
		
	      if(isArmPointingTowardTableBool){
		// for(int l = firstTablePointIndex; l< firstTablePointIndex + numTablePoints; l++){
		//   if(someIterAgentTablePointPointing < SDI_F->distancesInfo.numAgentArmTablePointPointingMax){
		//     if(SDI_F->distancesInfo.agentTableDists[someIterAgentTable].firstAgentTablePointPointingIndex == -1)
		//       SDI_F->distancesInfo.agentTableDists[someIterAgentTable].firstAgentTablePointPointingIndex = someIterAgentTablePointPointing;
		//     updatePointing( k, l, SDI_F->distancesInfo.arms[k].armVector, SDI_F->distancesInfo.arms[k].shoulderPosition,SDI_F->distancesInfo.tablePoints[l].position,SDI_F->distancesInfo.arms[k].armVectorLength,SDI_F->distancesInfo.arms[k].armVectorLengthMax,SDI_F->distancesInfo.agentArmTablePointPointing[someIterAgentTablePointPointing].agentIndex,SDI_F->distancesInfo.agentArmTablePointPointing[someIterAgentTablePointPointing].index,SDI_F->distancesInfo.agentArmTablePointPointing[someIterAgentTablePointPointing].optimalPointingVec , SDI_F->distancesInfo.agentArmTablePointPointing[someIterAgentTablePointPointing].optimalPointingPoint,  SDI_F->distancesInfo.agentArmTablePointPointing[someIterAgentTablePointPointing].curAngleFromOptimalPointingVec,  SDI_F->distancesInfo.agentArmTablePointPointing[someIterAgentTablePointPointing].pointingScore);	
		//     someIterAgentTablePointPointing++;
		//   }
		//   else
		//     printf("Too many AgentArmTablePointPointing. Max is %d\n",SDI_F->distancesInfo.numAgentArmTablePointPointingMax);
	      }
	    }
	  }
	  SDI_F->distancesInfo.numAgentArmTablePointingIntersection = someIterAgentTablePointingIntersection;
	  SDI_F->distancesInfo.numAgentArmTablePointPointing = someIterAgentTablePointPointing;
	  SDI_F->distancesInfo.agentTableDists[someIterAgentTable].lastAgentTablePointPointingIndex = someIterAgentTablePointPointing-1;
	  SDI_F->distancesInfo.agentTableDists[someIterAgentTable].lastAgentTablePointingIntersectionIndex = someIterAgentTablePointingIntersection-1;      
	}
	else{
	  SDI_F->distancesInfo.agentTableDists[someIterAgentTable].firstHandTableIndex = -1;
	  //SDI_F->distancesInfo.agentTableDists[someIterAgentTable].firstHandTablePointIndex = -1;
	  SDI_F->distancesInfo.agentTableDists[someIterAgentTable].firstAgentTablePointAngleIndex = -1;
	  SDI_F->distancesInfo.agentTableDists[someIterAgentTable].firstAgentTablePointPointingIndex = -1;
	  SDI_F->distancesInfo.agentTableDists[someIterAgentTable].firstAgentTablePointingIntersectionIndex = -1; 
	}
	
	someIterAgentTable++;	       
      }
      else
	printf("Too many AgentTableDists. Max is %d\n",SDI_F->distancesInfo.numAgentTableDistsMax);

    }
  }
  
  SDI_F->distancesInfo.numAgentTableDists = someIterAgentTable;

  ///////////////////////////////
  // object /  Object distances.
  ///////////////////////////////

  int someIterObjectObject = 0;
  for(int i = 0; i< numObjects; i++){
    for(int j = i+1; j< numObjects; j++){
      if(someIterObjectObject < SDI_F->distancesInfo.numObjectObjectDistsMax){
	updateDistAndMinDist(i,j,SDI_F->distancesInfo.objects[i].position,SDI_F->distancesInfo.objects[j].position,forceReinitMinDist, aTime,SDI_F->distancesInfo.objectObjectDists[someIterObjectObject].index1,SDI_F->distancesInfo.objectObjectDists[someIterObjectObject].index2,SDI_F->distancesInfo.objectObjectDists[someIterObjectObject].curDist,SDI_F->distancesInfo.objectObjectDists[someIterObjectObject].minDist,SDI_F->distancesInfo.objectObjectDists[someIterObjectObject].minDistTime);
	someIterObjectObject++;
      }
      else
	printf("Too many ObjectObjectDists. Max is %d\n",SDI_F->distancesInfo.numObjectObjectDistsMax);
    }
  }
  SDI_F->distancesInfo.numObjectObjectDists  = someIterObjectObject;

  ///////////////////////////////
  // object /  Table distances.
  ///////////////////////////////

  int someIterObjectTable = 0;
  //int someIterObjectTablePoint = 0;  
  for(int i = 0; i< numObjects; i++){
    for(int j = 0; j< numTables; j++){
      if(someIterObjectTable < SDI_F->distancesInfo.numObjectTableDistsMax){
	updateDistAndMinDistToTable(0,i,j,false,false,0,SDI_F->distancesInfo.objects[i].position,SDI_F->distancesInfo.tables[j].tableCornersPositions,forceReinitMinDist, aTime,SDI_F->distancesInfo.objectTableDists[someIterObjectTable].index1,SDI_F->distancesInfo.objectTableDists[someIterObjectTable].index2,SDI_F->distancesInfo.objectTableDists[someIterObjectTable].curDist,SDI_F->distancesInfo.objectTableDists[someIterObjectTable].isAbove,SDI_F->distancesInfo.objectTableDists[someIterObjectTable].closestPoint,SDI_F->distancesInfo.objectTableDists[someIterObjectTable].minDist, SDI_F->distancesInfo.objectTableDists[someIterObjectTable].minDistTime,SDI_F->distancesInfo.objectTableDists[someIterObjectTable].minDistPosition,anInt,anInt,anInt);


	////////////////////////////////////////
	//allow to compute distance to closest table point
	/////////////////////////////////////////
	// if(SDI_F->distancesInfo.objectTableDists[someIterAgentAgent].curDist < distThresholdSmall){
	//   firstTablePointIndex = SDI_F->distancesInfo.tables[i].firstTablePointIndex;
	//   numTablePoints = SDI_F->distancesInfo.tables[i].numTablePoints;
	//   SDI_F->distancesInfo.objectTableDists[someIterObjectTable].firstObjectTablePointIndex = someIterObjectTablePoint;
	// for(int l = firstTablePointIndex; l< firstTablePointIndex + numTablePoints; l++){
	//   updateDistAndMinDist(i,l,SDI_F->distancesInfo.objects[i].position,SDI_F->distancesInfo.tablePoints[l].position,forceReinitMinDist, aTime,SDI_F->distancesInfo.objectTablePointDists[someIterObjectTablePoint].index1,SDI_F->distancesInfo.objectTablePointDists[someIterObjectTablePoint].index2,SDI_F->distancesInfo.objectTablePointDists[someIterObjectTablePoint].curDist,SDI_F->distancesInfo.objectTablePointDists[someIterObjectTablePoint].minDist,SDI_F->distancesInfo.objectTablePointDists[someIterObjectTablePoint].minDistTime);
	//   someIterObjectTablePoint++;
	// }       	 
	//   SDI_F->distancesInfo.objectTableDists[someIterObjectTable].lastObjectTableIndex = someIterObjectTablePoint-1;
	// }
	// else
	//   SDI_F->distancesInfo.objectTableDists[someIterObjectTable].firstObjectTablePointIndex = -1;       

	someIterObjectTable++;	 
      }
      else
	printf("Too many ObjectTableDists. Max is %d\n",SDI_F->distancesInfo.numObjectTableDistsMax);

    }
  }
  SDI_F->distancesInfo.numObjectTableDists = someIterObjectTable;


  // reinitialize distance only once.
  if(forceReinitMinDist)
    SDI_F->distancesInfo.forceReinitMinDist = FALSE;

}

ACTIVITY_EVENT
sparkDistanceManagementStart(int *report)
{
  // SDI_F->distancesInfo.distAgentAgentThreshold = 2.0;
  // SDI_F->distancesInfo.distAgentObjectThreshold = 0.5;
  // SDI_F->distancesInfo.distAgentTableThreshold = 0.5;
  // SDI_F->distancesInfo.distObjectTableThreshold = 0.1;
  // SDI_F->distancesInfo.distFromTableThreshold = 2.0;
  SDI_F->distancesInfo.tableStep = 0.25;
 

  SDI_F->distancesInfo.distThreshold = 1.0;
  SDI_F->distancesInfo.distThresholdSmall = 0.5;
  SDI_F->distancesInfo.saveHistory = FALSE;  
  SDI_F->distancesInfo.saveHistoryDistanceThreshold = 0.8;

  SDI_F->distancesInfo.forceReinitMinDist = FALSE;
  SDI_F->distancesInfo.checkArmVectorLengthMax = FALSE;
  SDI_F->distancesInfo.reinitArmVectorLengthMax = FALSE;
  SDI_F->distancesInfo.forceUpdateTablesPosition = FALSE;
  SDI_F->distancesInfo.initialTime = getTimeInSecondMilisecond();



  SDI_F->distancesInfo.numAgents = 0;
  SDI_F->distancesInfo.numAgentsMax = SPARK_DISTANCES_INFO_NUM_AGENTS_MAX;

  SDI_F->distancesInfo.numTables = 0;
  SDI_F->distancesInfo.numTablesMax = SPARK_DISTANCES_INFO_NUM_TABLES_MAX;

  SDI_F->distancesInfo.numObjects = 0;
  SDI_F->distancesInfo.numObjectsMax = SPARK_DISTANCES_INFO_NUM_OBJECTS_MAX;

  SDI_F->distancesInfo.numHands = 0;
  SDI_F->distancesInfo.numHandsMax = 2*SPARK_DISTANCES_INFO_NUM_AGENTS_MAX;
  //numArms == numHands

  SDI_F->distancesInfo.numHeads = 0;
  SDI_F->distancesInfo.numHeadsMax = SPARK_DISTANCES_INFO_NUM_AGENTS_MAX;

  SDI_F->distancesInfo.numTablePoints = 0;
  SDI_F->distancesInfo.numTablePointsMax = SPARK_DISTANCES_INFO_NUM_TABLE_POINTS_MAX;

  SDI_F->distancesInfo.numAgentAgentDists = 0;
  SDI_F->distancesInfo.numAgentAgentDistsMax = SPARK_DISTANCES_INFO_NUM_AGENTS_MAX*(SPARK_DISTANCES_INFO_NUM_AGENTS_MAX-1)/2;

  SDI_F->distancesInfo.numAgentObjectDists = 0;
  SDI_F->distancesInfo.numAgentObjectDistsMax = SPARK_DISTANCES_INFO_NUM_AGENTS_MAX*SPARK_DISTANCES_INFO_NUM_OBJECTS_MAX;

  SDI_F->distancesInfo.numAgentTableDists = 0;
  SDI_F->distancesInfo.numAgentTableDistsMax = SPARK_DISTANCES_INFO_NUM_AGENTS_MAX*SPARK_DISTANCES_INFO_NUM_TABLES_MAX;

  SDI_F->distancesInfo.numObjectTableDists = 0;
  SDI_F->distancesInfo.numObjectTableDistsMax = SPARK_DISTANCES_INFO_NUM_OBJECTS_MAX*SPARK_DISTANCES_INFO_NUM_TABLES_MAX;

  SDI_F->distancesInfo.numAgentTablePointDists = 0;
  SDI_F->distancesInfo.numAgentTablePointDistsMax = SPARK_DISTANCES_INFO_NUM_AGENTS_MAX*SPARK_DISTANCES_INFO_NUM_TABLE_POINTS_MAX/2;

  SDI_F->distancesInfo.numAgentHandDists = 0;
  SDI_F->distancesInfo.numAgentHandDistsMax = 2*SPARK_DISTANCES_INFO_NUM_AGENTS_MAX*(SPARK_DISTANCES_INFO_NUM_AGENTS_MAX-1);

  SDI_F->distancesInfo.numHandHandDists = 0;
  SDI_F->distancesInfo.numHandHandDistsMax = 8*SPARK_DISTANCES_INFO_NUM_AGENTS_MAX*(SPARK_DISTANCES_INFO_NUM_AGENTS_MAX-1);

  SDI_F->distancesInfo.numObjectObjectDists = 0;
  SDI_F->distancesInfo.numObjectObjectDistsMax = SPARK_DISTANCES_INFO_NUM_OBJECTS_MAX*(SPARK_DISTANCES_INFO_NUM_OBJECTS_MAX-1)/2;

  SDI_F->distancesInfo.numHandObjectDists = 0;
  SDI_F->distancesInfo.numHandObjectDistsMax = 2*SPARK_DISTANCES_INFO_NUM_AGENTS_MAX*SPARK_DISTANCES_INFO_NUM_OBJECTS_MAX;


  SDI_F->distancesInfo.numHandTableDists = 0;
  SDI_F->distancesInfo.numHandTableDistsMax = 2*SPARK_DISTANCES_INFO_NUM_AGENTS_MAX*SPARK_DISTANCES_INFO_NUM_TABLES_MAX;  

  SDI_F->distancesInfo.numAgentAgentAngles = 0;
  SDI_F->distancesInfo.numAgentAgentAnglesMax = SPARK_DISTANCES_INFO_NUM_AGENTS_MAX*(SPARK_DISTANCES_INFO_NUM_AGENTS_MAX-1)/2;

  SDI_F->distancesInfo.numAgentObjectAngles = 0;
  SDI_F->distancesInfo.numAgentObjectAnglesMax = SPARK_DISTANCES_INFO_NUM_AGENTS_MAX*SPARK_DISTANCES_INFO_NUM_OBJECTS_MAX;

  SDI_F->distancesInfo.numAgentTablePointAngles = 0;
  SDI_F->distancesInfo.numAgentTablePointAnglesMax = SPARK_DISTANCES_INFO_NUM_AGENTS_MAX*SPARK_DISTANCES_INFO_NUM_TABLE_POINTS_MAX/2;
  
  SDI_F->distancesInfo.numAgentArmAgentPointing = 0;
  SDI_F->distancesInfo.numAgentArmAgentPointingMax = 2*SPARK_DISTANCES_INFO_NUM_AGENTS_MAX*(SPARK_DISTANCES_INFO_NUM_AGENTS_MAX-1);

  SDI_F->distancesInfo.numAgentArmObjectPointing = 0;
  SDI_F->distancesInfo.numAgentArmObjectPointingMax = 2*SPARK_DISTANCES_INFO_NUM_AGENTS_MAX*SPARK_DISTANCES_INFO_NUM_OBJECTS_MAX;

  SDI_F->distancesInfo.numAgentArmTablePointPointing = 0;
  SDI_F->distancesInfo.numAgentArmTablePointPointingMax = SPARK_DISTANCES_INFO_NUM_AGENTS_MAX*SPARK_DISTANCES_INFO_NUM_TABLE_POINTS_MAX;

  SDI_F->distancesInfo.numAgentArmTablePointingIntersection = 0;
  SDI_F->distancesInfo.numAgentArmTablePointingIntersectionMax = SPARK_DISTANCES_INFO_NUM_AGENTS_MAX*SPARK_DISTANCES_INFO_NUM_TABLES_MAX*2;

  //initialize histories arrays
  for(int i = 0; i< SPARK_DISTANCES_INFO_NUM_TRAJECTORIES_MAX; i++){
    SDI_F->distancesInfo.trajectories[i].agentIndex = -1;
    SDI_F->distancesInfo.trajectories[i].handIndex = -1;
    SDI_F->distancesInfo.trajectories[i].entityIndex = -1;
    SDI_F->distancesInfo.trajectories[i].curTimeStepIndex = -1;

    SDI_F->distancesInfo.distanceHistories[i].agentIndex = -1;
    SDI_F->distancesInfo.distanceHistories[i].handIndex = -1;
    SDI_F->distancesInfo.distanceHistories[i].entityIndex = -1;
    SDI_F->distancesInfo.distanceHistories[i].curTimeStepIndex = -1;
  }

  ///
  /// then we can start using fill pose perception info poster.
  ///
  SDI_F->state.fillPosePerceptionInfoPosterStartReady = TRUE;
    
  return EXEC;
}

ACTIVITY_EVENT
sparkDistanceManagementMain(int *report)
{
  /// time assessment
  double timeStart;
  getRequestTimeStart(SDI_F->requestTimeInfo.printLoopRequestExecutionTimeEnabled,timeStart);

  computeDistancesAndFillDistanceManagementPoster2();
  publishDistancesInfoPoster();
  
  trackRequestTime(SDI_F->requestTimeInfo.printLoopRequestExecutionTimeEnabled,timeStart,SDI_F->requestTimeInfo.printLoopRequestExecutionTimeDistanceInfoIndex);

  return EXEC;
}



ACTIVITY_EVENT
sparkUpdateEntityInDistanceManagementStart(SPARK_UPDATE_ENTITY_IN_DISTANCES_INFO_INPUT_PARAMS *entityInDistanceManagementUpdateInput,int *report)
{
  //manage possible threshold update
  int modifyParameterCommand = entityInDistanceManagementUpdateInput->modifyParameterCommand;




  if(modifyParameterCommand > -1){
    if(modifyParameterCommand == 0){
       SDI_F->distancesInfo.tableStep = entityInDistanceManagementUpdateInput->doubleParameterValue;
       printf("DistanceManagement table step value set to %f\n",entityInDistanceManagementUpdateInput->doubleParameterValue);
    }
    else if(modifyParameterCommand == 1){
      SDI_F->distancesInfo.distThreshold = entityInDistanceManagementUpdateInput->doubleParameterValue;
      printf("DistanceManagement distThreshold value set to %f\n",entityInDistanceManagementUpdateInput->doubleParameterValue);
    }
    else if(modifyParameterCommand == 2){
      SDI_F->distancesInfo.distThresholdSmall = entityInDistanceManagementUpdateInput->doubleParameterValue; 
      printf("DistanceManagement distThresholdSmall value set to %f\n",entityInDistanceManagementUpdateInput->doubleParameterValue);
    }
    else if(modifyParameterCommand == 3){
      if(SDI_F->distancesInfo.saveHistory){
	SDI_F->distancesInfo.saveHistory = FALSE;  
	printf("Stop saving history\n");
      }
      else{
	SDI_F->distancesInfo.saveHistory = TRUE;  
	SDI_F->distancesInfo.saveHistoryDistanceThreshold = entityInDistanceManagementUpdateInput->doubleParameterValue;
	printf("Start saving history with saveHistoryDistanceThreshold value set to %f\n",entityInDistanceManagementUpdateInput->doubleParameterValue);
      }
    }
    else if(modifyParameterCommand == 4){
      SDI_F->distancesInfo.forceReinitMinDist = TRUE;
      printf("Reinit minDist\n");
     }
    else if(modifyParameterCommand == 5){
      SDI_F->distancesInfo.checkArmVectorLengthMax = TRUE;
      printf("Check arm vector length\n");
    }
    else if(modifyParameterCommand == 6){
      SDI_F->distancesInfo.reinitArmVectorLengthMax = TRUE;
      printf("reinit arm vector length\n");
    }
    else if(modifyParameterCommand == 7){
      SDI_F->distancesInfo.forceUpdateTablesPosition = TRUE; 
      printf("force table position update\n");
    }
   else if(modifyParameterCommand == 8){
     // allow to compute pointing.
     int indexAgent = entityInDistanceManagementUpdateInput->intParameterValue;
     if(indexAgent < SPARK_DISTANCES_INFO_NUM_AGENTS_MAX){
       if(SDI_F->distancesInfo.agents[indexAgent].allowPointingComputation){
	 SDI_F->distancesInfo.agents[indexAgent].allowPointingComputation = FALSE;
	 printf("Stop computing pointing for agent %d\n",indexAgent);
       }
       else{
	 SDI_F->distancesInfo.agents[indexAgent].allowPointingComputation = TRUE;
	 printf("Start computing pointing for agent %d\n",indexAgent);
       }
     }
   }
   else if(modifyParameterCommand == 9){
     // allow to compute pointing.
     int indexAgent = entityInDistanceManagementUpdateInput->intParameterValue;
     if(indexAgent < SPARK_DISTANCES_INFO_NUM_AGENTS_MAX){
       if(SDI_F->distancesInfo.agents[indexAgent].allowGestureComputation){
	 SDI_F->distancesInfo.agents[indexAgent].allowGestureComputation = FALSE;
	 printf("Stop computing gesture for agent %d\n",indexAgent);
       }
       else{
	 SDI_F->distancesInfo.agents[indexAgent].allowGestureComputation = TRUE;
	 printf("Start computing gesture for agent %d\n",indexAgent);
       }
     }
   }
   else if(modifyParameterCommand == 10){
     int trajIndex = entityInDistanceManagementUpdateInput->intParameterValue;
     if(trajIndex < SPARK_DISTANCES_INFO_NUM_TRAJECTORIES_MAX){
       printf("Delete hand trajectory with index %d\n",trajIndex);
       SDI_F->distancesInfo.trajectories[trajIndex].agentIndex = -1;
       SDI_F->distancesInfo.trajectories[trajIndex].handIndex = -1;
       SDI_F->distancesInfo.trajectories[trajIndex].entityIndex = -1;
     }
   }
   else if(modifyParameterCommand == 11){
     if(SDI_F->state.fillPosePerceptionInfoPoster){
       SDI_F->state.fillPosePerceptionInfoPoster = FALSE;
       printf("Start filling position perception info and pan tilt info\n");
     }
     else{
       SDI_F->state.fillPosePerceptionInfoPoster = TRUE;
       printf("Stop filling position perception info and pan tilt info\n");
     }
   }


   else{
      *report = S_spark_UNMANAGED_THRESHOLD_INDEX;
      return ETHER;
    }
  }
  return EXEC;
}

ACTIVITY_EVENT
sparkUpdateEntityInDistanceManagementMain(SPARK_UPDATE_ENTITY_IN_DISTANCES_INFO_INPUT_PARAMS *entityInDistanceManagementUpdateInput, int *report)
{
  int e_i;
  bool addOrUpdate = false;
  bool isAlreadyPresent = false;
  int modifyParameterCommand = entityInDistanceManagementUpdateInput->modifyParameterCommand;
  double doubleParameterValue = entityInDistanceManagementUpdateInput->doubleParameterValue;
  int intParameterValue = entityInDistanceManagementUpdateInput->intParameterValue;

  // Get input element index in entity array
  for(e_i=0; e_i<gEntities->entities_nb; e_i++) {
    if (!strcmp(entityInDistanceManagementUpdateInput->entityName.name, gEntities->entities[e_i]->robotPt->name)){
      break;
    }
  }  
  if( e_i == gEntities->entities_nb) {
    *report = S_spark_ENTITY_NOT_FOUND;
    return ETHER;
  }

  //do we add/update or delete this entity
  if(entityInDistanceManagementUpdateInput->addEntity == GEN_TRUE){
    addOrUpdate = true;
  }

  // Look for entity in distance management entity table
  isAlreadyPresent = checkIsPresentObjectEntity(e_i);
  
  //update
  if(addOrUpdate && isAlreadyPresent){
    deleteEntityInDistanceManagement(e_i);
    if(!addEntityInDistanceManagement(e_i,modifyParameterCommand, doubleParameterValue, intParameterValue,entityInDistanceManagementUpdateInput->baseJointName,entityInDistanceManagementUpdateInput->shoulderJointName0,entityInDistanceManagementUpdateInput->shoulderJointName1)){
      *report = S_spark_IMPOSSIBLE_TO_ADD_ENTITY;
      return ETHER;
    }
  }


  //add
  else if (addOrUpdate && !isAlreadyPresent){
    if(!addEntityInDistanceManagement(e_i,modifyParameterCommand, doubleParameterValue, intParameterValue,entityInDistanceManagementUpdateInput->baseJointName,entityInDistanceManagementUpdateInput->shoulderJointName0,entityInDistanceManagementUpdateInput->shoulderJointName1)){
      *report = S_spark_IMPOSSIBLE_TO_ADD_ENTITY;
      return ETHER;
    }
  }

  //delete
  else if (!addOrUpdate && isAlreadyPresent){
    // update array by deleting the line of the entity to delete
    deleteEntityInDistanceManagement(e_i);    
  }

  //It means deleting an unpresent entity : nothing to do!
  else{
    //*report = S_spark_ENTITY_NOT_ALREADY_MANAGED;
    return ETHER;
  }
  return ETHER;
}
  

