/**
 ** sparkm3d.cc
 **
 ** Utility functions for codels
 **
 ** Author: E. Akin Sisbot
 ** Date: Aug 10
 **
 **/


#include <portLib.h>

#include "server/sparkHeader.h"
#include "sparkm3d.h"

int timeval_subtract(struct timeval *result, struct timeval *t2, struct timeval *t1)
{
  long int diff = (t2->tv_usec + 1000000 * t2->tv_sec) - (t1->tv_usec + 1000000 * t1->tv_sec);
  result->tv_sec = diff / 1000000;
  result->tv_usec = diff % 1000000;

  return (diff<0);
}


double getTimeInSecondMilisecond(){
  struct timeval tv;
  gettimeofday(&tv, NULL);
  return (double) tv.tv_sec + (double) 1e-6 * tv.tv_usec;
}

void getRequestTimeStart(int printLoopRequestExecutionTimeEnabled, double& timeStart){
  if(printLoopRequestExecutionTimeEnabled){
    timeStart = getTimeInSecondMilisecond();
  }
}

void printRequestTime(int index){
  double curTime =  SDI_F->requestTimeInfo.curTime[index];
  double curTimePercent = SDI_F->requestTimeInfo.curTimePercents[index];
  double sumTimePercent = SDI_F->requestTimeInfo.timeSinceBeginningPercents[index];

  if(SDI_F->requestTimeInfo.printLoopRequestExecutionTimeReadObjectsIndex == index){
    printf("ReadObjects request  cur duration is %f , cur percentage is %f , historical duration is %f\n",curTime,curTimePercent,sumTimePercent);
  }
  else if(SDI_F->requestTimeInfo.printLoopRequestExecutionTimeReadRobotIndex == index){
    printf("ReadRobot request cur duration is %f , cur percentage is %f , historical duration is %f\n",curTime,curTimePercent,sumTimePercent);
  }
  else if(SDI_F->requestTimeInfo.printLoopRequestExecutionTimeReadHumansIndex == index){
    printf("ReadHumans request cur duration is %f , cur percentage is %f , historical duration is %f\n",curTime,curTimePercent,sumTimePercent);
  }
  else if(SDI_F->requestTimeInfo.printLoopRequestExecutionTimeComputeFactsIndex == index){
    printf("ComputeFacts request cur duration is %f , cur percentage is %f , historical duration is %f\n",curTime,curTimePercent,sumTimePercent);
  }
  else if(SDI_F->requestTimeInfo.printLoopRequestExecutionTimeTestSpheresIndex == index){
    printf("TestSpheres request cur duration is %f , cur percentage is %f , historical duration is %f\n",curTime,curTimePercent,sumTimePercent);
  }
  else if(SDI_F->requestTimeInfo.printLoopRequestExecutionTimeDistanceInfoIndex == index){
    printf("Distance Info request cur duration is %f , cur percentage is %f , historical duration is %f\n",curTime,curTimePercent,sumTimePercent);
  }
}

void trackRequestTime(int printLoopRequestExecutionTimeEnabled, double timeStart, int& requestTimeId){
  if(printLoopRequestExecutionTimeEnabled){
    if( requestTimeId == -1){
      if(SDI_F->requestTimeInfo.numRequestTracked < SPARK_REQUEST_TIME_MAX){
        requestTimeId = SDI_F->requestTimeInfo.numRequestTracked;
        SDI_F->requestTimeInfo.numRequestTracked++;
      }
      else{
        printf("Max Request whose time duration can be tracked is %d\n",SPARK_REQUEST_TIME_MAX);        
      }
    }
    else{
      //compute sumary
      if( requestTimeId == 0){
        double sumCurTime = 0.0;
        double sumTimeSinceBeginning = 0.0;
        for(int i = 0; i < SDI_F->requestTimeInfo.numRequestTracked ; i++){
          sumCurTime+=SDI_F->requestTimeInfo.curTime[i];
          sumTimeSinceBeginning+=SDI_F->requestTimeInfo.timeSinceBeginning[i];
        }
        for(int i = 0; i < SDI_F->requestTimeInfo.numRequestTracked ; i++){
          if(sumCurTime>0)
            SDI_F->requestTimeInfo.curTimePercents[i] = SDI_F->requestTimeInfo.curTime[i]/sumCurTime;
          if(sumTimeSinceBeginning>0)
            SDI_F->requestTimeInfo.timeSinceBeginningPercents[i] = SDI_F->requestTimeInfo.timeSinceBeginning[i]/sumTimeSinceBeginning;
          if(SDI_F->requestTimeInfo.printLoopRequestExecutionTimePrintPeriod == SDI_F->requestTimeInfo.printLoopRequestExecutionTimePrintIter)
            printRequestTime(i);
        }

        if(SDI_F->requestTimeInfo.printLoopRequestExecutionTimePrintPeriod == SDI_F->requestTimeInfo.printLoopRequestExecutionTimePrintIter)
          SDI_F->requestTimeInfo.printLoopRequestExecutionTimePrintIter = 0;
        else
          SDI_F->requestTimeInfo.printLoopRequestExecutionTimePrintIter++;
      }
    }
    //update cur value
    double curDuration = getTimeInSecondMilisecond() - timeStart;
    SDI_F->requestTimeInfo.curTime[requestTimeId]=curDuration;
    SDI_F->requestTimeInfo.timeSinceBeginning[requestTimeId]+=curDuration;
    if(SDI_F->requestTimeInfo.maxTime[requestTimeId]<curDuration)
      SDI_F->requestTimeInfo.maxTime[requestTimeId] = curDuration;
  }
}
