/*****************************************************************************
*
* Emrah Akin Sisbot
* August 10
*
* File    : sparkStruct.h
* Brief   : Data structures for Spark module
*
******************************************************************************/

#ifndef SPARK_STRUCT_H
#define SPARK_STRUCT_H

/*----------------------------------------------------------------------------
  INCLUDES
-----------------------------------------------------------------------------*/

#include <genBasic/genBasicStruct.h>

#include "sparkConst.h"

/*----------------------------------------------------------------------------
  STRUCTURES
-----------------------------------------------------------------------------*/
/* DON'T FORGET: All fields in the STR of the module should have fix lengths         */
/*               Thus use of dynamic memory fields, and pointers in STR is forbidden */

typedef struct STRUCT_SPARK_P3D {
  GEN_STRING128 p3dFileName;
  int enableGraphic;
  int enableMight;
} SPARK_P3D;

typedef struct STRUCT_SPARK_ENTITY_STATE {
  GEN_STRING128 name;
  GEN_BOOL state;
} SPARK_ENTITY_STATE;

typedef struct STRUCT_SPARK_SCE {
  GEN_STRING128 name;
} SPARK_SCE;

typedef struct STRUCT_SPARK_STATE {
  GEN_STRING128 savePath; //Not used anymore
  int posPosterAv;
  int armConfPosterAv;
  int handPosterAv;
  int ptPosterAv;
  int robotConfPosterAv;
  int vimanPosterAv;
  int objectsFromKinectPosterAv;
  int sparkObjPosPosterAv;
  int useObjInsideSparkEnabled;
  int hrp2ConfPosterAv;
  int mocapPosterAv;
  int mocapHandPosterAv;
  int humanConfPosterAv;
  int bodyPartsPosterAv;
  int oroConnectionAv;
  int oroPrintEnabled;
  int vimanPrintEnabled;
  int mightabilitiesEnabled;
  int robotHandPosterAv;
  int niutPosterAv;
  int kinectPosterAv;
  int robotFingersPosterAv;
  int inhibitLookDirectionOroFactsSending; // to decide wether to send look at facts to oro
  int reexportReachabilityFactsToOro; // to decide wether to reexport reachability facts to oro.
  int fillPosePerceptionInfoPoster; // to decide wether to fill pose perception info poster used to reason about object observability.
  int fillPosePerceptionInfoPosterStartReady; // to be sure distance info has been initialized.
  

#ifdef PR2
  int pr2ConfigPostersAv; //are Pose and Joint state posters for PR2 available?
#endif
} SPARK_STATE;

typedef struct STRUCT_SPARK_SAVE_SCENARIO {
  GEN_STRING128 savePath;
  GEN_STRING128 fileName;
} SPARK_SAVE_SCENARIO;


//-------------------- TOOLS ---------------------------

typedef struct STRUCT_SPARK_6D_COORD {
  double x;
  double y;
  double z;
  double yaw;
  double pitch;
  double roll;
} SPARK_6D_COORD;

typedef struct STRUCT_SPARK_3D_COORD {
  double x;
  double y;
  double z;
} SPARK_3D_COORD;

//-------------------- INTERFACE ------------------------

typedef struct STRUCT_SPARK_CAM_POS {
  double xdest;
  double ydest;
  double zdest;
  double dist;
  double hrot;
  double vrot;
} SPARK_CAM_POS;

typedef struct STRUCT_SPARK_INTERFACE_PARAMS {
  int width;
  int height;
  GEN_BOOL saveInterface;
  GEN_BOOL floor;
  GEN_BOOL walls;
  GEN_BOOL shadows;
  GEN_BOOL tiles;
  GEN_BOOL antialiasing;
} SPARK_INTERFACE_PARAMS;

typedef struct STRUCT_SPARK_INTERFACE_AGENT_PARAMS {
  GEN_STRING64 agentName;
  GEN_BOOL showFov;
  GEN_BOOL showPoiting;
  GEN_BOOL showVisibleObjects;
} SPARK_INTERFACE_AGENT_PARAMS;

typedef struct STRUCT_SPARK_INTERFACE_MIGHTABILITY_PARAMS {
  int showVisibleObjects;
  int showReachableObjects;
} SPARK_INTERFACE_MIGHTABILITY_PARAMS;


typedef enum ENUM_SPARK_INTERFACE_CHANGE_OPTION {
  BB = 0,
  GHOST = 1
} SPARK_INTERFACE_CHANGE_OPTION;

typedef struct STRUCT_SPARK_CHANGE_DISPLAY_PARAMS {
  SPARK_INTERFACE_CHANGE_OPTION interface_change_option; 
  GEN_BOOL is_option_true;
} SPARK_CHANGE_DISPLAY_PARAMS;

//-------------------- ENVIRONMENT -----------------------

typedef struct STRUCT_SPARK_OBJECT_PARAMS {
  double MHP_OBJECT_UPDATE_DIST;
  double MHP_OBJECT_UPDATE_ANGLE;
} SPARK_OBJECT_PARAMS;

typedef struct STRUCT_SPARK_M3D_OBJECT {
  GEN_STRING64 name;
  SPARK_6D_COORD coords;
} SPARK_M3D_OBJECT;

typedef struct STRUCT_SPARK_M3D_OBJECT_SIMU_PARAMS {
  GEN_STRING64 name;
  SPARK_6D_COORD coords;
  int hasDisappeared;
} SPARK_M3D_OBJECT_SIMU_PARAMS;

typedef struct STRUCT_SPARK_M3D_OBJECT_SIMU {
  GEN_STRING64 name;
  SPARK_6D_COORD coords;
  int hasDisappeared;
  int isnew;
} SPARK_M3D_OBJECT_SIMU;

typedef enum ENUM_SPARK_OBJECT_MOTION {
  OBJ_NONE = 0,
  OBJ_STATIC = 1,
  OBJ_MOVING = 2
} SPARK_OBJECT_MOTION;

typedef struct STRUCT_SPARK_ROBOT_POSTERS {
  GEN_STRING64 armConfPostername;
  GEN_STRING64 handConfPostername;
  GEN_STRING64 posPostername; 
  GEN_STRING64 ptPostername;
  GEN_STRING64 confPosterName;
#ifdef PR2
  GEN_STRING64 pr2PosePosterName;
  GEN_STRING64 pr2JointStatePosterName;
  GEN_STRING64 pr2JointMapPosterName;
#endif
} SPARK_ROBOT_POSTERS;

typedef enum STRUCT_SPARK_POSTER_TYPE_ENUM {
  USE_MOCAP  = 0,
  USE_GEST   = 1,
  USE_HUMPOS = 2,
  USE_CONF  = 3,
  USE_BODY_PARTS = 4,
  USE_VIMAN = 5,
  USE_OBJ_POS = 6,
  USE_NIUT = 7,
  USE_OBJ_INSIDE_SPARK = 8,
  USE_OBJECT_FROM_KINECT = 9
} SPARK_POSTER_TYPE_ENUM;

typedef enum STRUCT_SPARK_HUMAN_STATE_ENUM {
  // must be the same as BT_... constants in Move3d hri_planner
  STANDING = 0,
  SITTING = 1,
  MOVING = 2,
  STANDING_TRANSPARENT = 3, // For optimistic planning
  DISAPPEARING = 4 // To simulate that human disappear with PlaceAgent
} SPARK_HUMAN_STATE_ENUM;

typedef struct STRUCT_SPARK_AGENT_POS {
  GEN_STRING64 name;
  SPARK_3D_COORD pos;
  double pan;
  double tilt;
  SPARK_HUMAN_STATE_ENUM state;
  int nouse;
} SPARK_AGENT_POS;

typedef struct STRUCT_SPARK_AGENT_ARMS_POS {
  GEN_STRING64 name;
  int degRotDofR0;
  int degRotDofR1;
  int degRotDofR2;
  int degRotDofR3;
  int degRotDofR4;
  int degRotDofR5;
  int degRotDofR6;
  int degRotDofL0;
  int degRotDofL1;
  int degRotDofL2;
  int degRotDofL3;
  int degRotDofL4;
  int degRotDofL5;
  int degRotDofL6;
} SPARK_AGENT_ARMS_POS;

typedef struct STRUCT_SPARK_JOINT_ABS_POS_PARAM {
    // Joint AbsPos
    GEN_STRING128 robotName;
    GEN_STRING128 jointName;
} SPARK_JOINT_ABS_POS_PARAM;

typedef struct STRUCT_SPARK_BB_POINTS {
  SPARK_3D_COORD bbPointsTable[8];
  double side1Length;
  double side2Length;
} SPARK_BB_POINTS;

typedef struct STRUCT_SPARK_CONFIGURATION {
  int changed;
  int dofNb;
  double dof[60];
} SPARK_CONFIGURATION;

/* nx  ox  ax  pz
   ny  oy  ay  py
   nz  oz  az  pz  */
typedef struct STRUCT_SPARK_POS_MATRIX {
  double nx, ny, nz;
  double ox, oy, oz;
  double ax, ay, az;
  double px, py, pz;
} SPARK_POS_MATRIX;

typedef struct STRUCT_SPARK_OBJECT_POSITION {
  unsigned long tacq_usec;	/** time of acquisition (microseconds) of the left image */
  unsigned long tacq_sec;	/** time of acquisition (seconds)  of the left image*/
  GEN_STRING64 name;	/* object name */
  int visible;
  SPARK_OBJECT_MOTION motion;
  SPARK_POS_MATRIX absPosMatrix;
} SPARK_OBJECT_POSITION;

typedef struct STRUCT_SPARK_OBJECT_POSITION_ARRAY {
  int nbObjects;
  int unused;
  SPARK_OBJECT_POSITION objects[SPARK_MAX_THINGS_NB];
} SPARK_OBJECT_POSITION_ARRAY;

typedef struct STRUCT_SPARK_INPUT_POSTER {
  SPARK_POSTER_TYPE_ENUM posterType;
  int unused;
  GEN_STRING64 name;
} SPARK_INPUT_POSTER;

typedef struct STRUCT_SPARK_INPUT_POSTERS {
    SPARK_INPUT_POSTER poster1;
    SPARK_INPUT_POSTER poster2;
} SPARK_INPUT_POSTERS;

typedef struct STRUCT_M3D_ROBOT {
  GEN_STRING64 name;
  double q[M3D_MAX_DOF];
  int length;
  int unused;
} M3D_ROBOT;

typedef struct STRUCT_M3D_FREEFLYER {
  GEN_STRING64 name;
  double q[6];
} M3D_FREEFLYER;

typedef struct STRUCT_SPARK_BOUNDING_BOX {
  GEN_STRING64 name;
  double x;
  double y;
  double z;
} SPARK_BOUNDING_BOX;

typedef struct STRUCT_SPARK_HUMAN_BODY_PARTS {
  GEN_STRING64 name;
  int headAv;
  int torsoAv;
  int leftHandAv;
  int rightHandAv;
  int gazeAv;
  int available;
  SPARK_6D_COORD head;
  SPARK_6D_COORD torso;
  SPARK_6D_COORD leftHand;
  SPARK_6D_COORD rightHand;
  SPARK_6D_COORD gaze;
} SPARK_HUMAN_BODY_PARTS;

typedef struct STRUCT_SPARK_HUMAN_BODY_PARTS_LIST {
  SPARK_HUMAN_BODY_PARTS human[SPARK_MAX_AGENT_NB];
  int humanNb;
  int unused;
} SPARK_HUMAN_BODY_PARTS_LIST;

typedef struct STRUCT_SPARK_DISTANCE {
  double minDistRobToHum;
  double costDistRobToHum;
} SPARK_DISTANCE;

/** Inferrence Management in sparkEnvironment*/

/** different type of inferrences managed*/
typedef enum ENUM_SPARK_INFERRENCE_TYPE {
  SPARK_NO_INFERRENCE =  0,
  SPARK_PRECISE_ROBOT_HAND = 1,
  SPARK_ROUGH_ROBOT_HAND = 2,
  SPARK_HUMAN_HAND =  3,
  SPARK_IS_IN = 4
} SPARK_INFERRENCE_TYPE;

/** different type of inferrences managed*/
typedef enum ENUM_SPARK_INFERRENCE_VALIDITY {
  SPARK_NO_PROBABILITY =  0,
  SPARK_HIGHLY_PROBABLE = 1,
  SPARK_PROBABLE = 2,
  SPARK_AVERAGE =  3,
  SPARK_UNPROBABLE = 4,
  SPARK_HIGHLY_UNPROBABLE =  5
} SPARK_INFERRENCE_VALIDITY;

typedef struct SPARK_STRUCT_GTP_FACT {
  GEN_STRING128 factType;
  GEN_STRING128 factSubType;
  GEN_STRING128 mainEntity;
  GEN_STRING128 targetEntity;
  GEN_BOOL isComputed;
  GEN_BOOL boolValue;
  int valueType; //0 for bool, 1 for specificValue
  int dummy;
  double specificValue;
} SPARK_GTP_FACT;

typedef struct SPARK_STRUCT_LIST_GTP_FACTS {
  int nbFacts;
  int wsId;
  SPARK_GTP_FACT gtp_fact[SPARK_NB_MAX_FACTS];
} SPARK_LIST_GTP_FACTS;

typedef struct STRUCT_SPARK_SET_INFERRENCE_FOR_OBJECT_PARAMS {
  GEN_STRING64 objectName;
  int hasInferrence;
  GEN_STRING64 agentOrObjectName;
  int agentPartNum;
  SPARK_INFERRENCE_TYPE inferrenceType;
  double perceptionInferrenceConflictThresholdMultiply;
} SPARK_SET_INFERRENCE_FOR_OBJECT_PARAMS;

typedef struct STRUCT_SPARK_SET_GRASPED_OBJECT_PARAMS {
  GEN_STRING64 name;
  int is_grasping_object;
  int grasped_object_arm_id;
} SPARK_SET_GRASPED_OBJECT_PARAMS;

typedef struct STRUCT_SPARK_SET_OBJECT_TO_GRIPPER_POSE_PARAMS {
  GEN_STRING64 name;
  int arm_id;
  double x;
  double y;
  double z;
  double rx;
  double ry;
  double rz;
} SPARK_SET_OBJECT_TO_GRIPPER_POSE_PARAMS;

typedef struct STRUCT_SPARK_KINECT_FRAME {
  GEN_BOOL enable;
  double x;
  double y;
  double z;
  double rx;
  double ry;
  double rz;
} SPARK_KINECT_FRAME;


//---------------- SITUATION ASSESSMENT ------------------

typedef enum ENUM_SPARK_FACT_OPERATION_TYPE {
  SPARK_NO_FACT_OPERATION = 0,
  SPARK_ADD_FACT = 1,
  SPARK_REMOVE_FACT = 2,
  SPARK_CLEAR_FACT = 3,
  SPARK_UPDATE_FACT = 4
} SPARK_FACT_OPERATION_TYPE;

typedef struct STRUCT_SPARK_FACT_OPERATION {
  SPARK_FACT_OPERATION_TYPE type;
  GEN_STRING64  agentModel;
  GEN_STRING64  subject;
  GEN_STRING64  predicate;
  GEN_STRING64  value;
} SPARK_FACT_OPERATION;


/* typedef struct STRUCT_SPARK_OBJECT_STATE { */
/*   GEN_STRING64  name; */
/*   unsigned long lastTimeSeen; */
/*   unsigned long lastTimeStatic; */
/*   int detected; */
/*   int disappeared; */
/*   SPARK_6D_COORD pos; */
/*   SPARK_OBJECT_MOTION motion; */
/*   SPARK_VISIBILITY_STATE agentVisibilityState[SPARK_MAX_AGENT_NB]; */
/*   int agentVisibilityStateNb; */
/*   SPARK_SYMB_PLACEMENTS placements[SPARK_OBJ_STATE_MAX]; */
/*   int placementLength; */
/*   int exists; */
/* } SPARK_OBJECT_STATE; */

//Type of situation assessment processing or control
typedef enum ENUM_SPARK_UPDATE_CONTROL_SPECIFIC_SA_TYPE {
  SPARK_REACH_PROC = 1,
  SPARK_LOOKAT_PROC = 2,
  SPARK_PLACEMENT_REL_PROC = 3,
  SPARK_DELETE_DIVERGENT_POSITION = 4,
  SPARK_FORCE_AGENT_KNOWS_POSITION = 5,
  SPARK_FORCE_AGENT_DOESNT_KNOW_POSITION = 6,
  SPARK_START_MANAGE_DISTANCE_INFO = 7,
  SPARK_STOP_MANAGE_DISTANCE_INFO = 8
} SPARK_UPDATE_CONTROL_SPECIFIC_SA_TYPE;

// Type of situation assessment update
// use to force reprocessing, to print value or update some parameters values.
typedef enum ENUM_SPARK_SA_UPDATE_TYPE {
  SPARK_UPDATE_ALL = 1,
  SPARK_BLOCK_UPDATE_ALL = 2,
  SPARK_DEBLOCK_UPDATE_ALL = 3,
  SPARK_UPDATE_LOOK_AT = 4,
  SPARK_PRINT_ALL_UPDATE = 5,
  SPARK_NOPRINT_ALL_UPDATE = 6,
  SPARK_PRINT_ALL_SPARK_ORO = 7,
  SPARK_CREATE_VISIBILITY_IMAGES_ON = 8,
  SPARK_CREATE_VISIBILITY_IMAGES_OFF = 9,
  SPARK_REEXPORT_ALL_SPARK_ORO = 10,
  SPARK_ALLOW_DISAPPEAR = 11,
  SPARK_FOREBID_DISAPPEAR = 12,
  SPARK_PRINT_VIMAN_INPUT = 13,
  SPARK_NOPRINT_VIMAN_INPUT = 14,
  SPARK_SWITCH_PRINT_SA_RECOMPUTE_STATUS = 15,
  SPARK_UPDATE_DISAPEAR_ITER_MAX = 16,
  SPARK_UPDATE_REACHABILITY_DISTANCE = 17,
  SPARK_UPDATE_IS_ON_MAX_DIST = 18,
  SPARK_SWITCH_DRAW_ACTION_MONITORING_SPHERES = 19,
  SPARK_ENABLE_DIVERGENT_POSITION = 20,
  SPARK_DISABLE_DIVERGENT_POSITION = 21,
  SPARK_UPDATE_AVERAGE_MOTION_EXPORT_THRESHOLD = 22,
  SPARK_SWITCH_DISPLAY_SA_RECOMPUTE_STATUS = 23,
  SPARK_UPDATE_NOT_STATIC_WITHOUT_RECOMPUTATION_ITER_MAX = 24,
  SPARK_UPDATE_NO_EVENT_STEPS_BEFORE_STATIC_ITER_MIN = 25,
  SPARK_START_SEND_LOOK_DIRECTION_ORO_FACTS = 26,
  SPARK_STOP_SEND_LOOK_DIRECTION_ORO_FACTS = 27,
  SPARK_FORCE_REACHABILITY_REEXPORT_TO_ORO = 28,
  SPARK_PRINT_LOOPING_REQUEST_TIME = 29,
  SPARK_PRINT_LOOPING_REQUEST_TIME_ITER = 30
} SPARK_SA_UPDATE_TYPE;
  
typedef struct STRUCT_SPARK_SA_UPDATE_PARAM {
  SPARK_SA_UPDATE_TYPE type;
  int intUpdatedValue;
  double doubleUpdatedValue;
} SPARK_SA_UPDATE_PARAM;

typedef struct STRUCT_SPARK_SPECIFIC_SA_PARAM {
  GEN_STRING64 agent;
  GEN_STRING64 entity1;
  GEN_STRING64 entity2;
  int agentPartIndex;
  SPARK_UPDATE_CONTROL_SPECIFIC_SA_TYPE updateControlType;
} SPARK_SPECIFIC_SA_PARAM;

// Type of agent activity motion update
typedef enum ENUM_SPARK_AGENT_ACTIVITY_MOTION_TYPE {
 SPARK_AGENT_ACTIVITY_MOTION_MOVING=0,
 SPARK_AGENT_ACTIVITY_MOTION_STATIC,
 SPARK_AGENT_ACTIVITY_MOTION_HAS_MOVED,
 SPARK_AGENT_ACTIVITY_MOTION_DID_NOT_MOVE,  
 SPARK_AGENT_ACTIVITY_MOTION_STATUS_UNKNOWN, 
 SPARK_MAXI_NUM_AGENTS_ACTIVITY_MOTION_STATUS
} SPARK_AGENT_ACTIVITY_MOTION_TYPE;

// Type of agent activity motion update
typedef enum ENUM_SPARK_AGENT_ACTIVITY_MOTION_TRANSITION {
 SPARK_AGENT_ACTIVITY_ISMOVING_BECOME_TRUE=0,
 SPARK_AGENT_ACTIVITY_ISMOVING_BECOME_FALSE=1,
 SPARK_AGENT_ACTIVITY_ISMOVING_NO_CHANGE=2,
 SPARK_AGENT_ACTIVITY_ISMOVING_IMPOSSIBLE_TRANSITION=3
} SPARK_AGENT_ACTIVITY_MOTION_TRANSITION;


/* typedef struct STRUCT_SPARK_VISREACH_RESULT { */
/*   GEN_STRING64 result1; */
/*   GEN_STRING64 result2; */
/* } SPARK_VISREACH_RESULT; */

/* typedef enum ENUM_SPARK_REACHABILITY { */
/*   UNREACHABLE = 0, */
/*   REACHABLE = 1, */
/*   REACHABLE_WITH_COLLISION = 2, */
/*   REACHABILITY_UNKNOWN = 3 */
/* } SPARK_REACHABILITY; */

/* typedef struct STRUCT_SPARK_AGENT_REACHABILITY { */
/*   GEN_STRING128 agentName; */
/*   GEN_STRING128 objectName; */
/*   SPARK_REACHABILITY reachability; */
/* } SPARK_AGENT_REACHABILITY; */

typedef struct STRUCT_SPARK_SERVER_ADDRESS {
  GEN_STRING64 address;
  GEN_STRING64 port;
} SPARK_SERVER_ADDRESS;

/* ------------- BEGIN NEW STRUCTURES --------------- */
/* NEW STRUCTURES TO REPLACE OBJECT STATES */

/* ----- FACTS ABOUT OBJECTS ------- */
/* typedef struct  STRUCT_SPARK_PLLOCATION_COUPLE { */
/*   int plLocation; */
/*   int agentIdx; */
/* } SPARK_PLLOCATION_COUPLE; */

/* typedef struct  STRUCT_SPARK_SPLOCATION_COUPLE { */
/*   SPARK_PLACEMENT_RELATION spLocation; */
/*   int objectIdx; */
/* } SPARK_SPLOCATION_COUPLE; */

/* typedef struct  STRUCT_SPARK_VISLOCATION_COUPLE { */
/*   SPARK_VISIBILITY_PLACEMENT visLocation; */
/*   int agentIdx; */
/* } SPARK_VISLOCATION_COUPLE; */

/* typedef struct STRUCT_SPARK_OBJECT_FACTS { */
/*   int isPresent; */
/*   int plLocationNb; */
/*   int spLocationNb; */
/*   int visLocationNb; */
/*   SPARK_PLLOCATION_COUPLE plLocation[SPARK_OBJ_STATE_MAX]; */
/*   SPARK_SPLOCATION_COUPLE spLotation[SPARK_OBJ_STATE_MAX]; */
/*   SPARK_VISLOCATION_COUPLE visLocation[SPARK_OBJ_STATE_MAX]; */
/* } SPARK_OBJECT_FACTS; */

/* ----- FACTS ABOUT AGENTS --------- */
/* typedef struct STRUCT_SPARK_SEES_COUPLE { */
/*   SPARK_VISIBILITY visibility; */
/*   int objectIdx; */
/* } SPARK_SEES_COUPLE; */

/* typedef struct STRUCT_SPARK_REACHES_COUPLE { */
/*   SPARK_REACHABILITY reachability; */
/*   int objectIdx; */
/* } SPARK_REACHES_COUPLE; */

/* typedef struct STRUCT_SPARK_AGENT_FACTS { */
/*   int isPresent; */
  
/*   int handsAreMoving; */
/*   int headIsMoving; */
/*   int bodyIsMoving; */
  
/*   int seesNb; */
/*   int reachesNb; */
/*   SPARK_SEES_COUPLE sees[SPARK_OBJ_STATE_MAX]; */
/*   SPARK_REACHES_COUPLE reaches[SPARK_OBJ_STATE_MAX]; */
  
/*   int looksAt[SPARK_OBJ_STATE_MAX]; */
/*   int pointsAt[SPARK_OBJ_STATE_MAX]; */
  
/* } SPARK_AGENT_FACTS; */

/* ------------- END NEW STRUCTURES --------------- */

/** ------------- BEGIN VISUAL ATTENTION STRUCTURES --------------- */
/** This information will be used in supervisor to drive visual attention according to (among other) last time seen, level of precision, */

typedef struct STRUCT_SPARK_OBJECT_POSITION_UPDATE_MANAGEMENT_STATE {
  GEN_STRING64  objectName;
  int entityIndex;
  SPARK_3D_COORD lastPrecisePosition;
  unsigned long lastTimeSeenPrecisely;
  SPARK_3D_COORD lastRoughPosition;
  unsigned long lastTimeSeenRoughly;
  SPARK_3D_COORD lastInferredPosition;
  unsigned long lastTimeInferred;
  int hasInferrence; //Is current object position
  SPARK_INFERRENCE_TYPE inferrenceType; //what is the type of this infference
  GEN_STRING64  inferrenceObjectOrAgentPartName; //Agent, object name
  int agentPartNum; // int to discriminate betwee one Agent parts ( example hands )
  double perceptionInferrenceConflictThreshold; //threshold use to decide of perception inference conflicts classes
  double perceptionInferrenceConflictValue; // perception inference conflicts value
  SPARK_INFERRENCE_VALIDITY inferrenceValidity; // perception inference conflicts class
  unsigned long lastTimePerceptionInferrenceConflict; // last time for perception inference conflict
  double speed;
  double average_speed;
  double dist;
  double average_dist;
  double distanceRoughToPrecise;
  int detected;
  int detectedOnce;
  int fixedPosition;
  int disappeared;
  double curObservability; //Hack to get isOn for given furniture to bypass ontology.
} SPARK_OBJECT_POSITION_UPDATE_MANAGEMENT_STATE;

/** status to describe current object position measurement*/
typedef enum ENUM_SPARK_OBJECT_MEASUREMENT_TYPE {
  PRECISE_MOVING = 0,
  PRECISE_STATIC = 1,
  ROUGH_MOVING =  2,
  ROUGH_STATIC = 3,
  INFERRED_MOVING =  4,
  INFERRED_STATIC = 5,
  NO_INFO = 6
} SPARK_OBJECT_MEASUREMENT_TYPE;

/** ------------- END VISUAL ATTENTION STRUCTURES  --------------- */

/** ------------- BEGIN ACTION MONITORING STRUCTURES --------------- */
/** This information will be used in supervisor to monitor agents actions by detection the exit from or the entrance in some sphere */

/** sphere type used for initial sphere geometric dimension processing and monitoring trigger assessment*/
/** Duplicate m3d enum. Can it be avoided simply? */
typedef enum ENUM_SPARK_SPHERE_TYPE {
  SPARK_SIMPLE_ENTRY = 0, // simple sphere defined through radius and center. To monitor entrance in the sphere;
  SPARK_SIMPLE_EXIT = 1,  // simple sphere defined through radius and center. To monitor exit from the sphere.
  SPARK_THROW_IN_CONTAINER = 2, // simple sphere those radius and center is automatically defined as fitting above the container.
  SPARK_PICK_OBJECT =  3, // simple sphere those radius and center is automatically defined as around object to pick.
  SPARK_PERMANENT_STOP_MONITOR = 4  //
} SPARK_SPHERE_TYPE; // Monitor that hands are fixed at some points.

typedef struct STRUCT_SPARK_ACTION_MONITORING_SPHERE {
  int isSphereActive; // TRUE if monitor is active, FALSE otherwise.
  GEN_STRING64  agentName; // agent name whose action is monitored.
  int agentIndex; // agent index.
  GEN_STRING64  objectName; // Object name used to define sphere for certain sphereType
  int entityIndex; // entity index of the object 
  int handIndexInput; // -1 if we don't want to precise hand index > -1 otherwise.
  SPARK_3D_COORD sphereCenter; //  sphere center. 
  double sphereRadius; // sphere radius.
  double filteringTimeThreshold; // time delay to wait for monitor success condition to triger monitor success
  SPARK_SPHERE_TYPE sphereType; // what is the type of this sphere as far as creating it is concerned
  int monitorEnterInResult; // TRUE if monitor trigger for enter in spheres and FALSE otherwise.
  int monitorGetOutResult; // TRUE if monitor trigger for get out of spheres (if it was in) and FALSE otherwise.
  int handIndexResult; // Whose agent hands trigger monitor.  
  int modifIndex; // each time there is something new on this sphere the index is incremented. This is used to update poster from move3d spheres.
  int dummy;
} SPARK_ACTION_MONITORING_SPHERE;

typedef struct STRUCT_SPARK_ACTION_MONITORING_SPHERE_INPUT_PARAMS {
  int monitorIndex; // index in the monitor array
  GEN_BOOL activateSphere; // TRUE to start monitor, FALSE to stop it. 
  GEN_STRING64 objectName; // Object name used to define sphere for certain sphereType
  int handIndexInput; // -1 if we don't want to precise hand index > -1 otherwise.
  GEN_STRING64 agentName; // agent name whose action is monitored.
  SPARK_3D_COORD sphereCenter; //  sphere center. 
  double sphereRadius; // sphere radius.
  double sphereRadiusMultiply; // Factor to increase or decrease Defaults Sphere radius 
  double sphereCenterTranslationMultiply; //Factor to increase or decrease Default translation used to get sphere center for compute.
  double filteringTimeThreshold; // time delay to wait for monitor success condition to triger monitor success
  SPARK_SPHERE_TYPE sphereType; // what is the type of this sphere as far as creating it is concerned
} SPARK_ACTION_MONITORING_SPHERE_INPUT_PARAMS;

/** ------------- END  ACTION MONITORING STRUCTURES  --------------- */

/** ------------- BEGIN Positions Distances Occupancy STRUCTURES --------------- */
/** This information will be used in supervisor to monitor distance between agents objects table and table occupancy */

/** sphere type used for initial sphere geometric dimension processing and monitoring trigger assessment*/
typedef enum ENUM_SPARK_DIST_ELEM_TYPE {
  SPARK_DIST_UNKNOWN = 0,  //
  SPARK_DIST_SIMPLE_OBJECT = 1, // ;
  SPARK_DIST_AGENT = 2,  // .
  SPARK_DIST_SUPPORT = 3,  // .
  SPARK_DIST_SUPPORT_POINT = 4,  //
  SPARK_DIST_CONTAINER = 5,  // .
  SPARK_DIST_CONTAINER_POINT = 6,  //
  SPARK_DIST_AGENT_HEAD = 7,  //
  SPARK_DIST_AGENT_HAND = 8  //
} SPARK_DIST_ELEM_TYPE; 

typedef struct STRUCT_SPARK_DISTANCES_INFO_AGENTS {
  //int indexInTab; // index in this 
  GEN_STRING64  entityName; // entity name which will be included in the distance management.
  int entityIndex; // entity index
  SPARK_3D_COORD orientation;
  SPARK_3D_COORD position;
  GEN_STRING64  baseJointName;
  int allowPointingComputation;
  int allowGestureComputation;
  int allowAngleComputation;
  int firstHandIndex;
  int headIndex;
  int numHands;
} SPARK_DISTANCES_INFO_AGENTS;

typedef struct STRUCT_SPARK_DISTANCES_INFO_TABLES {
  //int indexInTab; // index in this 
  GEN_STRING64  entityName; // entity name which will be included in the distance management.
  int entityIndex; // entity index
  SPARK_3D_COORD position;
  SPARK_3D_COORD tableCornersPositions[4];
  int firstTablePointIndex;
  int numTablePoints;
} SPARK_DISTANCES_INFO_TABLES;

typedef struct STRUCT_SPARK_DISTANCES_INFO_OBJECTS {
  //int indexInTab; // index in this 
  GEN_STRING64  entityName; // entity name which will be included in the distance management.
  int entityIndex; // entity index
  double diameter;
  SPARK_3D_COORD position;
} SPARK_DISTANCES_INFO_OBJECTS;


typedef struct STRUCT_SPARK_DISTANCES_INFO_POSITIONS_GENERIC {
  //int indexInTab; // .
  /* GEN_STRING64 entityName; // entity name which will be included in . */
  int mainEntityIndex; // entity index of the main entity from which this position was created 
  int entityIndex; // index of the entity if is an entity(hands for example)
  int mainEntityFamillyIndex; // index among the positions created from the same entity with the same type (table areas for example) 
  SPARK_3D_COORD  position;
} SPARK_DISTANCES_INFO_POSITIONS_GENERIC;

typedef struct STRUCT_SPARK_DISTANCES_ARM_ANGLE_INFO {
  double curAngle;
  double curAngleScore;
  double angleRef;
  double angleRefScore;
} SPARK_DISTANCES_ARM_ANGLE_INFO;

typedef struct STRUCT_SPARK_DISTANCES_INFO_ARMS {
  GEN_STRING64 shoulderJointName; 
  int agentEntityIndex; // entity index of the main entity from which this position was created 
  int mainEntityFamillyIndex; // index among the positions created from the same entity with the same type (table areas for example) 
  SPARK_3D_COORD  shoulderPosition;
  SPARK_3D_COORD  armVector;
  double armVectorLength;
  double armVectorLengthMax;
  //double armVectorLengthThreshold;
  SPARK_DISTANCES_ARM_ANGLE_INFO armAngleInfo;
} SPARK_DISTANCES_INFO_ARMS;


typedef struct STRUCT_SPARK_DISTANCES_INFO_HEADS {
  GEN_STRING64 eyesJointName; 
  int mainEntityIndex; // entity index of the main entity from which this position was created
  int entityIndex; // entity index
  int mainEntityFamillyIndex; // index among the positions created from the same entity with the same type (table areas for example) 
  SPARK_3D_COORD  position;
  SPARK_3D_COORD  eyesPosition;
  SPARK_3D_COORD  lookAtVector;
} SPARK_DISTANCES_INFO_HEADS;


typedef struct STRUCT_SPARK_DISTANCES_INFO_POINTING_FROM_AGENT {
  int agentIndex;
  int index;
  SPARK_3D_COORD optimalPointingVec;
  SPARK_3D_COORD optimalPointingPoint;
  double curAngleFromOptimalPointingVec;
  double pointingScore;
} SPARK_DISTANCES_INFO_POINTING_FROM_AGENT;

typedef struct STRUCT_SPARK_DISTANCES_INFO_DISTANCES_AGENT_AGENT {
  int index1;
  int index2;
  double curDist;
  int firstHandHandIndex;
  int firstAgentHandIndex;
  int lastHandHandIndex;
  int lastAgentHandIndex;
  int firstAgentAgentAngleIndex;
  int lastAgentAgentAngleIndex;
  int firstAgentHeadPointingIndex;
  int lastAgentHeadPointingIndex;
  double minDist;
  double minDistTime;
} SPARK_DISTANCES_INFO_DISTANCES_AGENT_AGENT;

typedef struct STRUCT_SPARK_DISTANCES_INFO_DISTANCES_AGENT_OBJECT {
  int index1;
  int index2;
  double curDist;
  int firstHandObjectIndex;
  int lastHandObjectIndex;
  int agentObjectAngleIndex;
  int firstAgentObjectPointingIndex;
  int lastAgentObjectPointingIndex;
  double minDist;
  double minDistTime;
} SPARK_DISTANCES_INFO_DISTANCES_AGENT_OBJECT;

typedef struct STRUCT_SPARK_DISTANCES_INFO_DISTANCES_AGENT_TABLE {
  int index1;
  int index2;
  double curDist;
  int firstHandTableIndex;
  //int firstHandTablePointIndex;
  int lastHandTableIndex;
  //int lastHandTablePointIndex;
  int firstAgentTablePointIndex;
  int lastAgentTablePointIndex;
  int firstAgentTablePointAngleIndex;
  int lastAgentTablePointAngleIndex;
  int firstAgentTablePointPointingIndex;
  int lastAgentTablePointPointingIndex;
  int firstAgentTablePointingIntersectionIndex;
  int lastAgentTablePointingIntersectionIndex;
  int isAbove;
  SPARK_3D_COORD  closestPoint;
  double minDist;
  SPARK_3D_COORD minDistPosition;
  double minDistTime;
} SPARK_DISTANCES_INFO_DISTANCES_AGENT_TABLE;

typedef struct STRUCT_SPARK_DISTANCES_INFO_DISTANCES_OBJECT_TABLE {
  int index1;
  int index2;
  double curDist;
  /* int firstObjectTablePointIndex; */
  /* int lastObjectTablePointIndex; */
  int isAbove;
  SPARK_3D_COORD closestPoint;
  double minDist;
  SPARK_3D_COORD minDistPosition;
  double minDistTime;
} SPARK_DISTANCES_INFO_DISTANCES_OBJECT_TABLE;

typedef struct STRUCT_SPARK_DISTANCES_INFO_DISTANCES_HAND_TABLE {
  int index1;
  int index2;
  double curDist;
  /* int firstObjectTablePointIndex; */
  /* int lastObjectTablePointIndex; */
  int isAbove;
  SPARK_3D_COORD closestPoint;
  int trajectoryIndex;
  int distanceHistoryIndex;
  int timeStepIndex;
  double minDist;
  SPARK_3D_COORD minDistPosition;
  double minDistTime;
} SPARK_DISTANCES_INFO_DISTANCES_HAND_TABLE;

typedef struct STRUCT_SPARK_DISTANCES_INFO_DISTANCES_HAND_OBJECT {
  int index1;
  int index2;
  double curDist;
  /* int distanceHistoryIndex; */
  /* int timeStepIndex; */
  double minDist;
  double minDistTime;
} SPARK_DISTANCES_INFO_DISTANCES_HAND_OBJECT;

typedef struct STRUCT_SPARK_DISTANCES_INFO_HAND_TRAJECTORY {
  int agentIndex;
  int handIndex;
  int entityIndex;
  int curTimeStepIndex;
  SPARK_3D_COORD positions[SPARK_DISTANCES_INFO_NUM_STEP_IN_TRAJECTORY_MAX];
  double timeSteps[SPARK_DISTANCES_INFO_NUM_STEP_IN_TRAJECTORY_MAX];
} SPARK_DISTANCES_INFO_HAND_TRAJECTORY;

typedef struct STRUCT_SPARK_DISTANCES_INFO_DISTANCE_HISTORY {
  int agentIndex;
  int handIndex;
  int entityIndex;
  int curTimeStepIndex;
  double  distances[SPARK_DISTANCES_INFO_NUM_STEP_IN_TRAJECTORY_MAX];
  double timeSteps[SPARK_DISTANCES_INFO_NUM_STEP_IN_TRAJECTORY_MAX];
} SPARK_DISTANCES_INFO_DISTANCE_HISTORY;

typedef struct STRUCT_SPARK_DISTANCES_INFO_DISTANCES_GENERIC {
  int index1;
  int index2;
  double curDist;
  double minDist;
  double minDistTime;
} SPARK_DISTANCES_INFO_DISTANCES_GENERIC;

typedef struct STRUCT_SPARK_DISTANCES_INFO_ANGLES {
  int index1;
  int index2;
  double angle;
} SPARK_DISTANCES_INFO_ANGLES;

/// intersection between a vector and a plan
/// mostly used for arm vector intersection with table plan.

typedef struct STRUCT_SPARK_DISTANCES_INFO_PLAN_INTERSECTION {
  int index1;
  int index2;
  SPARK_3D_COORD intersection;
} SPARK_DISTANCES_INFO_PLAN_INTERSECTION;


/* typedef struct STRUCT_SPARK_DISTANCES_INFO_OCCUPANCIES { */
/*   int indexElem1; */
/*   int indexElem2; */
/*   double minDistance; */
/* } SPARK_DISTANCES_INFO_OCCUPANCIES; */

typedef struct STRUCT_SPARK_UPDATE_ENTITY_IN_DISTANCES_INFO_INPUT_PARAMS {
  GEN_STRING64  entityName;
  GEN_BOOL addEntity; // TRUE to add or update this entity, FALSE to put if off. 
  int modifyParameterCommand; // -1 if no parameter
  double doubleParameterValue; // double value needed by parameter modification
  int intParameterValue; // int value needed by parameter modification
  GEN_STRING64  baseJointName; // for agents we need to specify base joint namze
  GEN_STRING64  shoulderJointName0; // for agents we need to specify shoulders joint name
  GEN_STRING64  shoulderJointName1; // for agents we need to specify shoulders joint name 
  ///GEN_STRING64  headJointName; // for agents we need to specify head joint name 

}  SPARK_UPDATE_ENTITY_IN_DISTANCES_INFO_INPUT_PARAMS;

/* typedef struct STRUCT_SPARK_DISTANCE_CONTENTS { */
/*   int indexElem1; */
/*   int indexElem2; */
/*   double contentScore; */
/* } SPARK_DISTANCES_INFO_CONTENTS; */

/** ------------- Distance Management  --------------- */

/** ------------- BEGIN Visualisation Info STRUCTURES --------------- */
/** This information will be used to add some visualisation in the remote viewer */

// what kind of visualisation it is
typedef enum ENUM_SPARK_VISU_INFO_TYPE {
  SPARK_VISU_INFO_SPHERE = 0, 
  SPARK_VISU_INFO_CONE = 1, 
  SPARK_VISU_INFO_CYLINDRE = 2, 
  SPARK_VISU_INFO_RECTANGLE = 3, 
  SPARK_VISU_INFO_ARROW = 4, 
  SPARK_VISU_INFO_TYPE_ENUM_MAX = 5
} SPARK_VISU_INFO_TYPE; 

typedef struct STRUCT_SPARK_VISU_INFO_MANAGEMENT{
  GEN_BOOL showReachability;
  GEN_BOOL showDivergentBelief;
  GEN_BOOL showTablePoints;
  int showReachabilityEntityVisuMapping[SPARK_VISU_AGENT_MAX][SPARK_VISU_REACHABILITY_BY_AGENT_MAX];
  int showDivergentBeliefEntityVisuMapping[SPARK_VISU_AGENT_MAX][SPARK_VISU_DIVERGENT_BELIEF_BY_AGENT_MAX];
  SPARK_3D_COORD verticalOffSet; 
  SPARK_3D_COORD horizontalOffSet[SPARK_VISU_AGENT_MAX]; // end point if needed (only for arrows)
  double r[SPARK_VISU_AGENT_MAX]; // red
  double g[SPARK_VISU_AGENT_MAX]; // green
  double b[SPARK_VISU_AGENT_MAX]; // blue
  double radius;
  double opacity;
  double distBetweenSphereCenters; // for each agent we translate the sphere center from this distance.
  double distMaxBeforeUpdate;
  int numVisuUpdate;
} SPARK_VISU_INFO_MANAGEMENT;

typedef struct STRUCT_SPARK_VISU_INFO{
  int isVisuActive; // is active
  int dummy1;
  SPARK_3D_COORD point1; //starting point
  SPARK_3D_COORD point2; // end point if needed (only for arrows)
  double r; // red
  double g; // green
  double b; // blue
  double opacity; // opacity
  double radius; // radius if needed
  double length1; // first length if needed
  double length2; // second left if needed
  int createrId; // id of component creating this
  int createrVisuId; // id of visu for component creating this visu
  SPARK_VISU_INFO_TYPE type;
  int dummy2;
} SPARK_VISU_INFO;

// what kind of visualisation it is
typedef enum ENUM_SPARK_VISU_INFO_PARAM_CHANGE_TYPE {
  SPARK_VISU_INFO_PARAM_NO_PARAM_CHANGE = 0,
  SPARK_VISU_INFO_PARAM_REACH_RADIUS = 1, 
  SPARK_VISU_INFO_PARAM_REACH_ZOFFSET = 2, 
  SPARK_VISU_INFO_PARAM_REACH_DIST_BETWEEN_AGENTS_SPHERES = 3,
  SPARK_VISU_INFO_PARAM_REACH_MAX_DIST_BEFORE_UPDATE = 4,
  SPARK_VISU_INFO_PARAM_SWITCH_SHOW_REACHABILITY = 5,
  SPARK_VISU_INFO_PARAM_SWITCH_SHOW_DIVERGENT_BELIEFS = 6,
  SPARK_VISU_INFO_PARAM_SWITCH_SHOW_TABLE_POINTS = 7
} SPARK_VISU_INFO_PARAM_CHANGE_TYPE; 

typedef struct STRUCT_SPARK_VISU_INFO_INPUT{
  GEN_BOOL addVisuInfo; // add or delete
  SPARK_VISU_INFO visuInfo; // visu info
  SPARK_VISU_INFO_PARAM_CHANGE_TYPE autoVisuInSparkManagementParamChangeType;
  int autoVisuInSparkManagementIntParam;
  double autoVisuInSparkManagementDoubleParam;
} SPARK_VISU_INFO_INPUT;


/** ------------- END  Visualisation Info STRUCTURES --------------- */

/** ------------- BEGIN Request Time Info STRUCTURES --------------- */
/** This information will be used to monitor request time */

typedef struct STRUCT_SPARK_REQUEST_TIME_INFO{
  int printLoopRequestExecutionTimeEnabled; // to decide wether to print loop request execution time
  int printLoopRequestExecutionTimePrintPeriod; // how many iterations to wait before next print.
  int printLoopRequestExecutionTimePrintIter;

  int printLoopRequestExecutionTimeReadObjectsIndex;
  int printLoopRequestExecutionTimeReadRobotIndex;
  int printLoopRequestExecutionTimeReadHumansIndex;
  int printLoopRequestExecutionTimeComputeFactsIndex;
  int printLoopRequestExecutionTimeTestSpheresIndex;
  int printLoopRequestExecutionTimeDistanceInfoIndex;
  int numRequestTracked; // num of requests whose duration is tracked
  double maxTime[SPARK_REQUEST_TIME_MAX]; // max time
  double curTime[SPARK_REQUEST_TIME_MAX]; // cur Time duration for all request
  int curTimePercents[SPARK_REQUEST_TIME_MAX]; // cur Time duration percents for all request
  double timeSinceBeginning[SPARK_REQUEST_TIME_MAX]; //total time since beginning for all requests
  int timeSinceBeginningPercents[SPARK_REQUEST_TIME_MAX]; //total time since beginning percents for all requests
} SPARK_REQUEST_TIME_INFO;

/** END Request Time Info STRUCTURES */


////////////////////////////////////////////////
/// Matthieu Warnier 23/09/2013
/// structure to have statistical observability
////////////////////////////////////////////////

typedef struct STRUCT_SPARK_OBSERVABILITY_COMPUTATION {
  unsigned long tacq_usec; 
  unsigned long tacq_sec;
  int curVimanIter; // iter on viman poster
  int curVimanRelativIter; // last viman iter with modulo 40
  int lastPerceptionIter[SPARK_OBJECT_POSITION_UPDATE_MANAGEMENT_STATE_OBJ_MAX]; /// for all manipulable object with perceived so far. Give the last viman iter.
  double curObservability[SPARK_OBJECT_POSITION_UPDATE_MANAGEMENT_STATE_OBJ_MAX];
  // current observability
  double curNonZeroStableObservability[SPARK_OBJECT_POSITION_UPDATE_MANAGEMENT_STATE_OBJ_MAX]; // last stable observability.
  int perceptionBool[SPARK_OBJECT_POSITION_UPDATE_MANAGEMENT_STATE_OBJ_MAX][40];
  /// history of last perception
} SPARK_OBSERVABILITY_COMPUTATION;

///
// Struct to export in a poster info about object position perception.
//
// where robot lookat  and which object are seen and how (stereo / m)
///

typedef enum ENUM_SPARK_POSE_PERCEPTION_INFO_TYPE {
  SPARK_POSE_PERCEPTION_INFO_NO_INFO = 0,
  SPARK_POSE_PERCEPTION_INFO_ROBOT_LOOKAT_FOR_NEW_VIMAN_IMAGE = 1, 
  SPARK_POSE_PERCEPTION_INFO_ENTITY_PERCEPTION = 2
} SPARK_POSE_PERCEPTION_INFO_TYPE; 



typedef struct STRUCT_SPARK_POSE_PERCEPTION_INFO {
  SPARK_POSE_PERCEPTION_INFO_TYPE type;
  /// viman image index if new perception step : (viman image)
  /// entity index if entity perception info for this perception step.
  int curVimanIter;
  int entityIndex;
  int vimanPerceptionType;

  /// robot look At pan and tilt if new perception step : (viman image)
  double robotLookAtPan;
  double robotLookAtTilt;

  /// closet position state position 1 info
  SPARK_DIST_ELEM_TYPE positionStatePositionType1;
  int positionStatePositionIndex1;
  double positionStatePositionDistance1;
  /// closet position state position 2 info
  SPARK_DIST_ELEM_TYPE positionStatePositionType2;
  int positionStatePositionIndex2;
  double positionStatePositionDistance2;
  /// closet position state position 3 info
  SPARK_DIST_ELEM_TYPE positionStatePositionType3;
  int positionStatePositionIndex3;
  double positionStatePositionDistance3;
  /// closet position state position 4 info
  SPARK_DIST_ELEM_TYPE positionStatePositionType4;
  int positionStatePositionIndex4;
  double positionStatePositionDistance4;

  /// closet Container position state position 4 info
  SPARK_DIST_ELEM_TYPE containerType;
  int containerObjectIndex;
  double containerDistance;
} SPARK_POSE_PERCEPTION_INFO;

///
/// Ronot Target Pan Tilt To Look At Special Position 
///


typedef struct STRUCT_SPARK_LOOKAT_TARGET_PAN_TILT {
  SPARK_DIST_ELEM_TYPE type;
  int distanceInfoIndex;
  int entityIndex;
  int mainEntityIndex;
  double pan;
  double tilt;
} SPARK_LOOKAT_TARGET_PAN_TILT;





//-------------------- POSTERS -----------------------
/// poster on pose perception info
typedef struct STRUCT_SPARK_ALL_POSE_PERCEPTION_INFO {
  int lastAddedLineIndex; /// index of the last added line
  int numFullPosterIter; /// iter on
  int numPosePerceptionInfosMax;
  int unused;
  SPARK_POSE_PERCEPTION_INFO posePerceptionInfos[SPARK_POSE_PERCEPTION_MAX];
} SPARK_ALL_POSE_PERCEPTION_INFO;

/// poster on look at pan tilt info
/// used to get pan tilt to look at some useful positions.
typedef struct STRUCT_SPARK_ALL_LOOK_AT_PAN_TILT {
  int numMax; /// max possible number for pan tilt info.
  int curNumPanTilt; /// current total number for pan tilt info.
  int curNumPanTiltPositionsStates; /// current total number for pan tilt info without heads and non containter objects (positions states ...)
  int updateTables; /// need to update table
  double curPan; /// current robot pan tilt index
  double curTilt; /// current robot tilt index
  int firstClosestPanTiltIndex; /// index of first "closest" pan/tilt coordinate of position point.
  int secondClosestPanTiltIndex; /// index of second "closest" pan/tilt coordinate of position point.
  SPARK_LOOKAT_TARGET_PAN_TILT allLookAtPanTilt[SPARK_PAN_TILT_MAX];
} SPARK_ALL_LOOK_AT_PAN_TILT;

/// poster for remote viewer visualization
typedef struct STRUCT_SPARK_ALL_VISU_INFO {
  int numVisusInfo;
  int dummy;
  SPARK_VISU_INFO visusInfos[SPARK_VISU_NUM_MAX];
} SPARK_ALL_VISU_INFO;

typedef struct STRUCT_SPARK_ALL_DISTANCES_INFO {
  /* double distAgentAgentThreshold; //threshold 0 */
  /* double distAgentObjectThreshold; */
  /* double distAgentTableThreshold; */
  /* double distObjectTableThreshold; */
  /* double distFromTableThreshold; */
  double tableStep; // for table discretization.
  double distThreshold;
  double distThresholdSmall;
  int forceUpdateTablesPosition;
  int forceReinitMinDist;

  int saveHistory;
  int checkArmVectorLengthMax;
  int reinitArmVectorLengthMax;
  double saveHistoryDistanceThreshold;  
  double initialTime;

  /* int numEntities; */
  /* SPARK_DISTANCES_INFO_ENTITIES entities[SPARK_DISTANCES_INFO_NUM_ENITIES_MAX]; */

  /* int numPositions;   */
  /* SPARK_DISTANCES_INFO_POSITIONS positions[SPARK_DISTANCES_INFO_NUM_POSITIONS_MAX]; */

  /* int numDistances; */
  /* SPARK_DISTANCES_INFO_DISTANCES distances[SPARK_DISTANCES_INFO_NUM_DISTANCES_MAX]; */

  /* int numAngles; */
  /* SPARK_DISTANCES_INFO_ANGLES angles[SPARK_DISTANCES_INFO_NUM_ANGLES_MAX]; */

  /* int numOccupancies; */
  /* SPARK_DISTANCES_INFO_OCCUPANCIES occupancies[SPARK_DISTANCES_INFO_NUM_OCCUPANCIES_MAX]; */

  
  int numAgents;
  int numAgentsMax;
  SPARK_DISTANCES_INFO_AGENTS agents[SPARK_DISTANCES_INFO_NUM_AGENTS_MAX];

  int numTables;
  int numTablesMax;
  SPARK_DISTANCES_INFO_TABLES tables[SPARK_DISTANCES_INFO_NUM_TABLES_MAX];

  int numObjects;
  int numObjectsMax;
  SPARK_DISTANCES_INFO_OBJECTS objects[SPARK_DISTANCES_INFO_NUM_OBJECTS_MAX];

  int numHands;
  int numHandsMax;
  SPARK_DISTANCES_INFO_POSITIONS_GENERIC hands[2*SPARK_DISTANCES_INFO_NUM_AGENTS_MAX];
  SPARK_DISTANCES_INFO_ARMS arms[2*SPARK_DISTANCES_INFO_NUM_AGENTS_MAX];  

  int numHeads;
  int numHeadsMax;
  SPARK_DISTANCES_INFO_HEADS heads[SPARK_DISTANCES_INFO_NUM_AGENTS_MAX];

  int numTablePoints;
  int numTablePointsMax;
  SPARK_DISTANCES_INFO_POSITIONS_GENERIC tablePoints[SPARK_DISTANCES_INFO_NUM_TABLE_POINTS_MAX];


  int numAgentAgentDists;
  int numAgentAgentDistsMax;
  SPARK_DISTANCES_INFO_DISTANCES_AGENT_AGENT agentAgentDists[SPARK_DISTANCES_INFO_NUM_AGENTS_MAX*(SPARK_DISTANCES_INFO_NUM_AGENTS_MAX-1)/2];

  int numAgentObjectDists;
  int numAgentObjectDistsMax;
  SPARK_DISTANCES_INFO_DISTANCES_AGENT_OBJECT agentObjectDists[SPARK_DISTANCES_INFO_NUM_AGENTS_MAX*SPARK_DISTANCES_INFO_NUM_OBJECTS_MAX];

  int numAgentTableDists;
  int numAgentTableDistsMax;
  SPARK_DISTANCES_INFO_DISTANCES_AGENT_TABLE agentTableDists[SPARK_DISTANCES_INFO_NUM_AGENTS_MAX*SPARK_DISTANCES_INFO_NUM_TABLES_MAX];

  int numObjectTableDists;
  int numObjectTableDistsMax;
  SPARK_DISTANCES_INFO_DISTANCES_OBJECT_TABLE objectTableDists[SPARK_DISTANCES_INFO_NUM_OBJECTS_MAX*SPARK_DISTANCES_INFO_NUM_TABLES_MAX];

  int numAgentTablePointDists;
  int numAgentTablePointDistsMax;
  SPARK_DISTANCES_INFO_DISTANCES_GENERIC  agentTablePointDists[SPARK_DISTANCES_INFO_NUM_AGENTS_MAX*SPARK_DISTANCES_INFO_NUM_TABLE_POINTS_MAX/2];

  int numAgentHandDists;
  int numAgentHandDistsMax;
  SPARK_DISTANCES_INFO_DISTANCES_GENERIC agentHandDists[2*SPARK_DISTANCES_INFO_NUM_AGENTS_MAX*(SPARK_DISTANCES_INFO_NUM_AGENTS_MAX-1)];

  int numHandHandDists;
  int numHandHandDistsMax;
  SPARK_DISTANCES_INFO_DISTANCES_GENERIC handHandDists[8*SPARK_DISTANCES_INFO_NUM_AGENTS_MAX*(SPARK_DISTANCES_INFO_NUM_AGENTS_MAX-1)];

  int numObjectObjectDists;
  int numObjectObjectDistsMax;
  SPARK_DISTANCES_INFO_DISTANCES_GENERIC objectObjectDists[SPARK_DISTANCES_INFO_NUM_OBJECTS_MAX*(SPARK_DISTANCES_INFO_NUM_OBJECTS_MAX-1)/2];

  int numHandObjectDists;
  int numHandObjectDistsMax;
  SPARK_DISTANCES_INFO_DISTANCES_HAND_OBJECT handObjectDists[2*SPARK_DISTANCES_INFO_NUM_AGENTS_MAX*SPARK_DISTANCES_INFO_NUM_OBJECTS_MAX];

  int numHandTableDists;
  int numHandTableDistsMax;
  SPARK_DISTANCES_INFO_DISTANCES_HAND_TABLE handTableDists[2*SPARK_DISTANCES_INFO_NUM_AGENTS_MAX*SPARK_DISTANCES_INFO_NUM_TABLES_MAX];

  int numAgentAgentAngles;
  int numAgentAgentAnglesMax;
  SPARK_DISTANCES_INFO_ANGLES agentAgentAngles[SPARK_DISTANCES_INFO_NUM_AGENTS_MAX*(SPARK_DISTANCES_INFO_NUM_AGENTS_MAX-1)/2];

  int numAgentObjectAngles;
  int numAgentObjectAnglesMax;
  SPARK_DISTANCES_INFO_ANGLES agentObjectAngles[SPARK_DISTANCES_INFO_NUM_AGENTS_MAX*SPARK_DISTANCES_INFO_NUM_OBJECTS_MAX];

  int numAgentTablePointAngles;
  int numAgentTablePointAnglesMax;
  SPARK_DISTANCES_INFO_ANGLES agentTablePointAngles[SPARK_DISTANCES_INFO_NUM_AGENTS_MAX*SPARK_DISTANCES_INFO_NUM_TABLE_POINTS_MAX/2];
  
  int numAgentArmAgentPointing;
  int numAgentArmAgentPointingMax;
  SPARK_DISTANCES_INFO_POINTING_FROM_AGENT agentArmAgentPointing[2*SPARK_DISTANCES_INFO_NUM_AGENTS_MAX*(SPARK_DISTANCES_INFO_NUM_AGENTS_MAX-1)];

  int numAgentArmObjectPointing;
  int numAgentArmObjectPointingMax;
  SPARK_DISTANCES_INFO_POINTING_FROM_AGENT agentArmObjectPointing[2*SPARK_DISTANCES_INFO_NUM_AGENTS_MAX*SPARK_DISTANCES_INFO_NUM_OBJECTS_MAX];

  int numAgentArmTablePointPointing;
  int numAgentArmTablePointPointingMax;
  SPARK_DISTANCES_INFO_POINTING_FROM_AGENT agentArmTablePointPointing[SPARK_DISTANCES_INFO_NUM_AGENTS_MAX*SPARK_DISTANCES_INFO_NUM_TABLE_POINTS_MAX];

  int numAgentArmTablePointingIntersection;
  int numAgentArmTablePointingIntersectionMax;
  SPARK_DISTANCES_INFO_PLAN_INTERSECTION agentArmTablePointingIntersection[SPARK_DISTANCES_INFO_NUM_AGENTS_MAX*SPARK_DISTANCES_INFO_NUM_TABLES_MAX*2];

  SPARK_DISTANCES_INFO_HAND_TRAJECTORY trajectories[SPARK_DISTANCES_INFO_NUM_TRAJECTORIES_MAX];
  SPARK_DISTANCES_INFO_DISTANCE_HISTORY distanceHistories[SPARK_DISTANCES_INFO_NUM_TRAJECTORIES_MAX];

} SPARK_ALL_DISTANCES_INFO;

typedef struct STRUCT_ALL_OBJECTS_SPARK_POSITION_UPDATE_MANAGEMENT_STATE {
  SPARK_OBJECT_POSITION_UPDATE_MANAGEMENT_STATE objects[SPARK_OBJECT_POSITION_UPDATE_MANAGEMENT_STATE_OBJ_MAX];
  int length;
  unsigned long time;
} SPARK_ALL_OBJECTS_POSITION_UPDATE_MANAGEMENT_STATE;

typedef struct STRUCT_SPARK_ALL_MONITORING_SPHERES {
  SPARK_ACTION_MONITORING_SPHERE spheres[SPARK_NUM_SPHERES_MAX];
  int modifIndex;  //each time there is something new in spheres the index is incremented. This is used to update poster from move3d spheres.
} SPARK_ALL_MONITORING_SPHERES;

typedef struct STRUCT_SPARK_ALL_FACT_OPERATIONS {
  int curLineIndex; //index of last lign added
  int curFullTableIter; //how many times was the end of the table reached
  int numLineMax; // how many lines in poster
  int maxFactOperationsUpdateNum; // what was the maximum of operations addded in during one loop.
  SPARK_FACT_OPERATION factOperation[SPARK_FACT_OPERATION_MAX];
} SPARK_ALL_FACT_OPERATIONS;

//typedef struct STRUCT_SPARK_CURRENT_ENVIRONMENT {
//  GEN_STRING64 envName;
//  M3D_ROBOT robot[SPARK_MAX_THINGS_NB];
//  int robotNb;
//  int time;
//} SPARK_CURRENT_ENVIRONMENT;


typedef struct STRUCT_SPARK_CURRENT_ENVIRONMENT {
  GEN_STRING64 envName;
  M3D_ROBOT robot[SPARK_MAX_AGENT_NB];
  M3D_FREEFLYER freeflyer[SPARK_MAX_FREEFLYER_NB];
  int robotNb;
  int freeflyerNb;
  int time;
  int unused;
} SPARK_CURRENT_ENVIRONMENT;


typedef struct STRUCT_OUTSIDE_ROBOT {
    GEN_STRING128 robot_name;
    GEN_STRING128 server_name;
}OUTSIDE_ROBOT;



typedef struct STRUCT_SPARK_BOUNDING_BOXES {
  GEN_STRING64 envName;
  SPARK_BOUNDING_BOX bbox[SPARK_MAX_THINGS_NB*2];
  int robotNb;
  int unused;
} SPARK_BOUNDING_BOXES;

#if defined(USE_MIGHTABILITY_MAPS)
typedef struct STRUCT_MIGHTABILITY_FLAGS {
  int MIGHTABILITY_INITIALISED;
  int SATRT_MIGHTABILITY_UPDATE;
  int STOP_MIGHTABILITY_UPDATE;
  int PAUSE_MIGHTABILITY_UPDATE;
  
}STRUCT_MIGHTABILITY_FLAGS;

typedef struct STRUCT_AROUND_OBJ_INFO_FOR_MM
{
char around_obj_name[100];
}STRUCT_AROUND_OBJ_INFO_FOR_MM;

typedef struct STRUCT_AGENT_ACTIVITY_FACTS
{
  char agent_name[100];
 char agent_motion[50];
 char body_turn[50];
 char torso_turn[50];
 char head_turn[50];

 char right_hand_status[50];
 char right_hand_config_mode[50];
 char right_hand_occup_mode[50];
 char right_hand_rest_mode_type[50];
 char right_hand_on_support[50];
 char right_hand_holding_object[50];

 char left_hand_status[50];
 char left_hand_config_mode[50];
 char left_hand_occup_mode[50];
 char left_hand_rest_mode_type[50];
 char left_hand_on_support[50];
 char left_hand_holding_object[50];
}STRUCT_AGENT_ACTIVITY_FACTS;

typedef struct STRUCT_AGENTS_ACTIVITY_FACTS
{
  STRUCT_AGENT_ACTIVITY_FACTS Agent_Facts[4];
}STRUCT_AGENTS_ACTIVITY_FACTS;

typedef struct STRUCT_SHOW_OBJECT_FACTS_FLAGS
{

//int show_by_circle;
int show_arrow;

}STRUCT_SHOW_OBJECT_FACTS_FLAGS;


#endif

#endif
